{
  "cells": [
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "---\n",
        "title: \"✅ Week 02 Lab - Student Solutions\"\n",
        "subtitle: \"2024/25 Winter Term\"\n",
        "author: The DS202 Team\n",
        "date: 27 January 2025\n",
        "---\n",
        "\n",
        "This solution file follows the format of the Jupyter notebook `.ipynb` you had to fill in during the lab session.\n",
        "\n",
        "# ⚙️ Setup\n",
        "\n",
        "**Import required libraries:**"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 2,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "\n",
              "            <div id=\"YOposr\"></div>\n",
              "            <script type=\"text/javascript\" data-lets-plot-script=\"library\">\n",
              "                if(!window.letsPlotCallQueue) {\n",
              "                    window.letsPlotCallQueue = [];\n",
              "                }; \n",
              "                window.letsPlotCall = function(f) {\n",
              "                    window.letsPlotCallQueue.push(f);\n",
              "                };\n",
              "                (function() {\n",
              "                    var script = document.createElement(\"script\");\n",
              "                    script.type = \"text/javascript\";\n",
              "                    script.src = \"https://cdn.jsdelivr.net/gh/JetBrains/lets-plot@v4.5.2/js-package/distr/lets-plot.min.js\";\n",
              "                    script.onload = function() {\n",
              "                        window.letsPlotCall = function(f) {f();};\n",
              "                        window.letsPlotCallQueue.forEach(function(f) {f();});\n",
              "                        window.letsPlotCallQueue = [];\n",
              "                        \n",
              "                    };\n",
              "                    script.onerror = function(event) {\n",
              "                        window.letsPlotCall = function(f) {};    // noop\n",
              "                        window.letsPlotCallQueue = [];\n",
              "                        var div = document.createElement(\"div\");\n",
              "                        div.style.color = 'darkred';\n",
              "                        div.textContent = 'Error loading Lets-Plot JS';\n",
              "                        document.getElementById(\"YOposr\").appendChild(div);\n",
              "                    };\n",
              "                    var e = document.getElementById(\"YOposr\");\n",
              "                    e.appendChild(script);\n",
              "                })()\n",
              "            </script>\n",
              "            "
            ]
          },
          "metadata": {},
          "output_type": "display_data"
        }
      ],
      "source": [
        "import numpy as np\n",
        "import pandas as pd\n",
        "from lets_plot import *\n",
        "LetsPlot.setup_html()"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# 📋 Lab Tasks\n",
        "\n",
        "## 🛠 Part 1: Data manipulation with `pandas` (45 min)\n",
        "\n",
        "### ✨ **`pandas` data frame attributes**\n",
        "\n",
        "A `pandas` data frame has a lot of attributes, several of which we shall explore:\n",
        "\n",
        "-   `shape`: shows the number of rows and columns.\n",
        "-   `index`: prints the name of each row in the data frame.\n",
        "-   `columns`: prints the name of each column in the data frame.\n",
        "\n",
        "### 🔧 **`pandas` data frame methods**\n",
        "\n",
        "`pandas` data frames have a lot of methods! We will focus our attention on the following:\n",
        "\n",
        "-   `head` / `tail`: shows the first / last *n* observations of a data frame.\n",
        "-   `to_frame`: converts a series (`pandas` will automatically convert a data frame into a series if one variable is selected) to a data frame.\n",
        "-   `unique`: shows all unique values for qualitative features selected.\n",
        "-   `value_counts`: counts the number of times a unique value appears in a qualitative feature.\n",
        "-   `query`: keeps rows that conform to one or more logical criteria.\n",
        "-   `reset_index`: a subset of a data frame will keep the index of an old data frame. We use this method to change the index to one integer increments.\n",
        "-   `filter`: keeps columns that are included in a user-supplied list (via the `items` parameter).\n",
        "-   `drop`: drops columns that are included in a user-supplied list.\n",
        "-   `rename`: renames already existing columns based on a user-supplied dictionary.\n",
        "-   `assign`: creates a new variable based on a `lambda` function.\n",
        "-   `apply`: applies a `lambda` function across a set of variables.\n",
        "-   `get_dummies`: transforms qualitative features into a series of dummy features.\n",
        "-   `groupby`: perform grouped calculations within qualitative features.\n",
        "\n",
        "**Import `gapminder`**"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 3,
      "metadata": {},
      "outputs": [],
      "source": [
        "gapminder = pd.read_csv(\"data/gapminder.csv\")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "### 1.1: Printing `pandas` data frames\n",
        "\n",
        "Let's print `gapminder`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 4,
      "metadata": {},
      "outputs": [
        {
          "name": "stdout",
          "output_type": "stream",
          "text": [
            "          country continent  year  lifeExp       pop   gdpPercap\n",
            "0     Afghanistan      Asia  1952   28.801   8425333  779.445314\n",
            "1     Afghanistan      Asia  1957   30.332   9240934  820.853030\n",
            "2     Afghanistan      Asia  1962   31.997  10267083  853.100710\n",
            "3     Afghanistan      Asia  1967   34.020  11537966  836.197138\n",
            "4     Afghanistan      Asia  1972   36.088  13079460  739.981106\n",
            "...           ...       ...   ...      ...       ...         ...\n",
            "1699     Zimbabwe    Africa  1987   62.351   9216418  706.157306\n",
            "1700     Zimbabwe    Africa  1992   60.377  10704340  693.420786\n",
            "1701     Zimbabwe    Africa  1997   46.809  11404948  792.449960\n",
            "1702     Zimbabwe    Africa  2002   39.989  11926563  672.038623\n",
            "1703     Zimbabwe    Africa  2007   43.487  12311143  469.709298\n",
            "\n",
            "[1704 rows x 6 columns]\n",
            "       country continent  year  lifeExp       pop   gdpPercap\n",
            "0  Afghanistan      Asia  1952   28.801   8425333  779.445314\n",
            "1  Afghanistan      Asia  1957   30.332   9240934  820.853030\n",
            "2  Afghanistan      Asia  1962   31.997  10267083  853.100710\n",
            "3  Afghanistan      Asia  1967   34.020  11537966  836.197138\n",
            "4  Afghanistan      Asia  1972   36.088  13079460  739.981106\n",
            "       country continent  year  lifeExp       pop   gdpPercap\n",
            "1699  Zimbabwe    Africa  1987   62.351   9216418  706.157306\n",
            "1700  Zimbabwe    Africa  1992   60.377  10704340  693.420786\n",
            "1701  Zimbabwe    Africa  1997   46.809  11404948  792.449960\n",
            "1702  Zimbabwe    Africa  2002   39.989  11926563  672.038623\n",
            "1703  Zimbabwe    Africa  2007   43.487  12311143  469.709298\n"
          ]
        }
      ],
      "source": [
        "# Printing the object itself\n",
        "print(gapminder)\n",
        "\n",
        "# Printing the first 5 rows\n",
        "print(gapminder.head(5))\n",
        "\n",
        "# Printing the last 5 rows\n",
        "print(gapminder.tail(5))"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "We can see the dimensions of our data by calling the shape attribute of `gapminder`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 5,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "(1704, 6)"
            ]
          },
          "execution_count": 5,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder.shape"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "Here, we find that `gapminder` has 1,704 rows and 6 columns.\n",
        "\n",
        "### 1.2: Data frames and series\n",
        "\n",
        "We can see that `gapminder` has a list of countries and continents. We will explore continents further. To select only the continent column, we quote the variable name inside brackets next to `gapminder`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 6,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "0         Asia\n",
              "1         Asia\n",
              "2         Asia\n",
              "3         Asia\n",
              "4         Asia\n",
              "         ...  \n",
              "1699    Africa\n",
              "1700    Africa\n",
              "1701    Africa\n",
              "1702    Africa\n",
              "1703    Africa\n",
              "Name: continent, Length: 1704, dtype: object"
            ]
          },
          "execution_count": 6,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[\"continent\"]"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "You can also reference variables using `.`:"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 7,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "0         Asia\n",
              "1         Asia\n",
              "2         Asia\n",
              "3         Asia\n",
              "4         Asia\n",
              "         ...  \n",
              "1699    Africa\n",
              "1700    Africa\n",
              "1701    Africa\n",
              "1702    Africa\n",
              "1703    Africa\n",
              "Name: continent, Length: 1704, dtype: object"
            ]
          },
          "execution_count": 7,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder.continent"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "👉 **NOTE:** If you have variables separated by any white space or variables that contain special characters such as `-` or `@`, you can only use the brackets notation to select a column. The . (dot) notation can only be used for column names that are valid Python identifiers (e.g., no spaces, must start with a letter or underscore, and contain only alphanumeric characters or underscores).\n",
        "\n",
        "This changes the data frame to a series. If you want the output to remain a data frame, however, you can use the `to_frame` method."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 8,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>continent</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1699</th>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1700</th>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1701</th>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1702</th>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1703</th>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>1704 rows × 1 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "     continent\n",
              "0         Asia\n",
              "1         Asia\n",
              "2         Asia\n",
              "3         Asia\n",
              "4         Asia\n",
              "...        ...\n",
              "1699    Africa\n",
              "1700    Africa\n",
              "1701    Africa\n",
              "1702    Africa\n",
              "1703    Africa\n",
              "\n",
              "[1704 rows x 1 columns]"
            ]
          },
          "execution_count": 8,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[\"continent\"].to_frame()"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "### 1.3: Finding / counting unique values\n",
        "\n",
        "To find the names of all the continents we can use the `unique` method."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 9,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "array(['Asia', 'Europe', 'Africa', 'Americas', 'Oceania'], dtype=object)"
            ]
          },
          "execution_count": 9,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[\"continent\"].unique()"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "To see how many times each continent appears, we can use the `value_counts` method. We use reset index in order to turn `continent` into its own column."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 10,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>continent</th>\n",
              "      <th>count</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Africa</td>\n",
              "      <td>624</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Asia</td>\n",
              "      <td>396</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Europe</td>\n",
              "      <td>360</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Americas</td>\n",
              "      <td>300</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Oceania</td>\n",
              "      <td>24</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "</div>"
            ],
            "text/plain": [
              "  continent  count\n",
              "0    Africa    624\n",
              "1      Asia    396\n",
              "2    Europe    360\n",
              "3  Americas    300\n",
              "4   Oceania     24"
            ]
          },
          "execution_count": 10,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[\"continent\"].value_counts().reset_index()"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "❓**Question:** Do you see anything odd?\n",
        "\n",
        "*Yes, there are only 24 observations for Oceania, despite the fact that `gapminder` contains multiple measurements of the same country from 1952 to 2007.*\n",
        "\n",
        "### 1.4: Performing grouped calculations\n",
        "\n",
        "Suppose we want to calculate average GDP per capita across time. We can use a combination of the `groupby` and `mean` methods from `Pandas`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 11,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>year</th>\n",
              "      <th>gdpPercap</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>1952</td>\n",
              "      <td>3725.276046</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>1957</td>\n",
              "      <td>4299.408345</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>1962</td>\n",
              "      <td>4725.812342</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>1967</td>\n",
              "      <td>5483.653047</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>1972</td>\n",
              "      <td>6770.082815</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>5</th>\n",
              "      <td>1977</td>\n",
              "      <td>7313.166421</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>6</th>\n",
              "      <td>1982</td>\n",
              "      <td>7518.901673</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>7</th>\n",
              "      <td>1987</td>\n",
              "      <td>7900.920218</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>8</th>\n",
              "      <td>1992</td>\n",
              "      <td>8158.608521</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>9</th>\n",
              "      <td>1997</td>\n",
              "      <td>9090.175363</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>10</th>\n",
              "      <td>2002</td>\n",
              "      <td>9917.848365</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>11</th>\n",
              "      <td>2007</td>\n",
              "      <td>11680.071820</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "</div>"
            ],
            "text/plain": [
              "    year     gdpPercap\n",
              "0   1952   3725.276046\n",
              "1   1957   4299.408345\n",
              "2   1962   4725.812342\n",
              "3   1967   5483.653047\n",
              "4   1972   6770.082815\n",
              "5   1977   7313.166421\n",
              "6   1982   7518.901673\n",
              "7   1987   7900.920218\n",
              "8   1992   8158.608521\n",
              "9   1997   9090.175363\n",
              "10  2002   9917.848365\n",
              "11  2007  11680.071820"
            ]
          },
          "execution_count": 11,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder.groupby(\"year\", as_index=False)[\"gdpPercap\"].mean()"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "### 1.5: Subsetting rows\n",
        "\n",
        "Suppose we want to investigate as to why Oceania has only 24 observations (see **Part 1.3**), we can start by using the `query` method, which filters rows by one or more conditions."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 12,
      "metadata": {},
      "outputs": [],
      "source": [
        "gapminder_oceania = gapminder.query(\"continent == 'Oceania'\")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "📝**Task:** Find the unique values of country in `gapminder_oceania`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 13,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "array(['Australia', 'New Zealand'], dtype=object)"
            ]
          },
          "execution_count": 13,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_oceania[\"country\"].unique()"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "❓**Question:** Do you see the reason now?\n",
        "\n",
        "*Yes, this is because only Australia and New Zealand are included in Oceania. No other island nations are included.*\n",
        "\n",
        "### 1.6: Subsetting columns\n",
        "\n",
        "Suppose we only want our data frame to include country, year and population. We can use the `filter` method in a Pandas data frame setting the `items` parameter equal to a list of feature names that we want to include."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 106,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>year</th>\n",
              "      <th>pop</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1952</td>\n",
              "      <td>8425333</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1957</td>\n",
              "      <td>9240934</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1962</td>\n",
              "      <td>10267083</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1967</td>\n",
              "      <td>11537966</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1972</td>\n",
              "      <td>13079460</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1699</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1987</td>\n",
              "      <td>9216418</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1700</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1992</td>\n",
              "      <td>10704340</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1701</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1997</td>\n",
              "      <td>11404948</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1702</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2002</td>\n",
              "      <td>11926563</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1703</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2007</td>\n",
              "      <td>12311143</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>1704 rows × 3 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "          country  year       pop\n",
              "0     Afghanistan  1952   8425333\n",
              "1     Afghanistan  1957   9240934\n",
              "2     Afghanistan  1962  10267083\n",
              "3     Afghanistan  1967  11537966\n",
              "4     Afghanistan  1972  13079460\n",
              "...           ...   ...       ...\n",
              "1699     Zimbabwe  1987   9216418\n",
              "1700     Zimbabwe  1992  10704340\n",
              "1701     Zimbabwe  1997  11404948\n",
              "1702     Zimbabwe  2002  11926563\n",
              "1703     Zimbabwe  2007  12311143\n",
              "\n",
              "[1704 rows x 3 columns]"
            ]
          },
          "execution_count": 106,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder.filter(items=[\"country\",\"year\",\"pop\"])"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "Another option is to use the `drop` method which takes a list of features to drop. Here, we specify `axis=1` which signifies columns, not rows (to specify rows, we set `axis=0`)."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 14,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>year</th>\n",
              "      <th>pop</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1952</td>\n",
              "      <td>8425333</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1957</td>\n",
              "      <td>9240934</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1962</td>\n",
              "      <td>10267083</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1967</td>\n",
              "      <td>11537966</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1972</td>\n",
              "      <td>13079460</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1699</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1987</td>\n",
              "      <td>9216418</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1700</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1992</td>\n",
              "      <td>10704340</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1701</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1997</td>\n",
              "      <td>11404948</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1702</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2002</td>\n",
              "      <td>11926563</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1703</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2007</td>\n",
              "      <td>12311143</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>1704 rows × 3 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "          country  year       pop\n",
              "0     Afghanistan  1952   8425333\n",
              "1     Afghanistan  1957   9240934\n",
              "2     Afghanistan  1962  10267083\n",
              "3     Afghanistan  1967  11537966\n",
              "4     Afghanistan  1972  13079460\n",
              "...           ...   ...       ...\n",
              "1699     Zimbabwe  1987   9216418\n",
              "1700     Zimbabwe  1992  10704340\n",
              "1701     Zimbabwe  1997  11404948\n",
              "1702     Zimbabwe  2002  11926563\n",
              "1703     Zimbabwe  2007  12311143\n",
              "\n",
              "[1704 rows x 3 columns]"
            ]
          },
          "execution_count": 14,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder.drop([\"lifeExp\",\"continent\",\"gdpPercap\"],axis=1)"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "Yet another way to achieve the same result as the `filter` and `drop` methods we've just shown is the double square bracket subsetting:"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 44,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>year</th>\n",
              "      <th>pop</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1952</td>\n",
              "      <td>8425333</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1957</td>\n",
              "      <td>9240934</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1962</td>\n",
              "      <td>10267083</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1967</td>\n",
              "      <td>11537966</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1972</td>\n",
              "      <td>13079460</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1699</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1987</td>\n",
              "      <td>9216418</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1700</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1992</td>\n",
              "      <td>10704340</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1701</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1997</td>\n",
              "      <td>11404948</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1702</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2002</td>\n",
              "      <td>11926563</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1703</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2007</td>\n",
              "      <td>12311143</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>1704 rows × 3 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "          country  year       pop\n",
              "0     Afghanistan  1952   8425333\n",
              "1     Afghanistan  1957   9240934\n",
              "2     Afghanistan  1962  10267083\n",
              "3     Afghanistan  1967  11537966\n",
              "4     Afghanistan  1972  13079460\n",
              "...           ...   ...       ...\n",
              "1699     Zimbabwe  1987   9216418\n",
              "1700     Zimbabwe  1992  10704340\n",
              "1701     Zimbabwe  1997  11404948\n",
              "1702     Zimbabwe  2002  11926563\n",
              "1703     Zimbabwe  2007  12311143\n",
              "\n",
              "[1704 rows x 3 columns]"
            ]
          },
          "execution_count": 44,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[[\"country\",\"year\",\"pop\"]]"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "👉 **NOTE:** If you have used `pandas`, you may have used the `loc` and `iloc` methods on data frames. These functions enable users to select both columns and rows in one function. While, in theory, this sounds great, these methods are computationally inefficient, so we advise that you do not use these methods and, instead, opt for `query` and `filter`.\n",
        "\n",
        "### 1.7: Renaming columns\n",
        "\n",
        "It is good practice to convert variable names to \"snake case\" whereby all characters are lower case and each word in the variable is separated by an underscore. To find the variable names expressed as an index, we call the `columns` attribute."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 15,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "Index(['country', 'continent', 'year', 'lifeExp', 'pop', 'gdpPercap'], dtype='object')"
            ]
          },
          "execution_count": 15,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder.columns"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "From there, we can amend the variable names using a dictionary where the *key* is the current variable name and the *value* is the variable name you would like it to be. We then use the `rename` method, setting the `columns` parameter equal to the dictionary created."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 16,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "Index(['country', 'continent', 'year', 'life_exp', 'pop', 'gdp_per_cap'], dtype='object')"
            ]
          },
          "execution_count": 16,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "# Create a dictionary of variable names using snake case\n",
        "snake_case_var_names = {\n",
        "  \"country\":\"country\",\n",
        "  \"continent\":\"continent\",\n",
        "  \"year\":\"year\",\n",
        "  \"lifeExp\": \"life_exp\",\n",
        "  \"pop\":\"pop\",\n",
        "  \"gdpPercap\":\"gdp_per_cap\"\n",
        "}\n",
        "\n",
        "# Set the columns attribute to this list\n",
        "gapminder = gapminder.rename(columns=snake_case_var_names)\n",
        "\n",
        "# Check the columns attribute\n",
        "gapminder.columns"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "### 1.8: Creating new variables\n",
        "\n",
        "We know that Gross Domestic Product can be obtained from multiplying GDP per capita and population. To do this in Pandas, we simply insert `*` between the `gdp_per_cap` and `pop` columns found in `gapminder`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 17,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>continent</th>\n",
              "      <th>year</th>\n",
              "      <th>life_exp</th>\n",
              "      <th>pop</th>\n",
              "      <th>gdp_per_cap</th>\n",
              "      <th>gdp</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>Asia</td>\n",
              "      <td>1952</td>\n",
              "      <td>28.801</td>\n",
              "      <td>8425333</td>\n",
              "      <td>779.445314</td>\n",
              "      <td>6.567086e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>Asia</td>\n",
              "      <td>1957</td>\n",
              "      <td>30.332</td>\n",
              "      <td>9240934</td>\n",
              "      <td>820.853030</td>\n",
              "      <td>7.585449e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>Asia</td>\n",
              "      <td>1962</td>\n",
              "      <td>31.997</td>\n",
              "      <td>10267083</td>\n",
              "      <td>853.100710</td>\n",
              "      <td>8.758856e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>Asia</td>\n",
              "      <td>1967</td>\n",
              "      <td>34.020</td>\n",
              "      <td>11537966</td>\n",
              "      <td>836.197138</td>\n",
              "      <td>9.648014e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>Asia</td>\n",
              "      <td>1972</td>\n",
              "      <td>36.088</td>\n",
              "      <td>13079460</td>\n",
              "      <td>739.981106</td>\n",
              "      <td>9.678553e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1699</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>Africa</td>\n",
              "      <td>1987</td>\n",
              "      <td>62.351</td>\n",
              "      <td>9216418</td>\n",
              "      <td>706.157306</td>\n",
              "      <td>6.508241e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1700</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>Africa</td>\n",
              "      <td>1992</td>\n",
              "      <td>60.377</td>\n",
              "      <td>10704340</td>\n",
              "      <td>693.420786</td>\n",
              "      <td>7.422612e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1701</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>Africa</td>\n",
              "      <td>1997</td>\n",
              "      <td>46.809</td>\n",
              "      <td>11404948</td>\n",
              "      <td>792.449960</td>\n",
              "      <td>9.037851e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1702</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>Africa</td>\n",
              "      <td>2002</td>\n",
              "      <td>39.989</td>\n",
              "      <td>11926563</td>\n",
              "      <td>672.038623</td>\n",
              "      <td>8.015111e+09</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1703</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>Africa</td>\n",
              "      <td>2007</td>\n",
              "      <td>43.487</td>\n",
              "      <td>12311143</td>\n",
              "      <td>469.709298</td>\n",
              "      <td>5.782658e+09</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>1704 rows × 7 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "          country continent  year  life_exp       pop  gdp_per_cap  \\\n",
              "0     Afghanistan      Asia  1952    28.801   8425333   779.445314   \n",
              "1     Afghanistan      Asia  1957    30.332   9240934   820.853030   \n",
              "2     Afghanistan      Asia  1962    31.997  10267083   853.100710   \n",
              "3     Afghanistan      Asia  1967    34.020  11537966   836.197138   \n",
              "4     Afghanistan      Asia  1972    36.088  13079460   739.981106   \n",
              "...           ...       ...   ...       ...       ...          ...   \n",
              "1699     Zimbabwe    Africa  1987    62.351   9216418   706.157306   \n",
              "1700     Zimbabwe    Africa  1992    60.377  10704340   693.420786   \n",
              "1701     Zimbabwe    Africa  1997    46.809  11404948   792.449960   \n",
              "1702     Zimbabwe    Africa  2002    39.989  11926563   672.038623   \n",
              "1703     Zimbabwe    Africa  2007    43.487  12311143   469.709298   \n",
              "\n",
              "               gdp  \n",
              "0     6.567086e+09  \n",
              "1     7.585449e+09  \n",
              "2     8.758856e+09  \n",
              "3     9.648014e+09  \n",
              "4     9.678553e+09  \n",
              "...            ...  \n",
              "1699  6.508241e+09  \n",
              "1700  7.422612e+09  \n",
              "1701  9.037851e+09  \n",
              "1702  8.015111e+09  \n",
              "1703  5.782658e+09  \n",
              "\n",
              "[1704 rows x 7 columns]"
            ]
          },
          "execution_count": 17,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[\"gdp\"] = gapminder[\"gdp_per_cap\"] * gapminder[\"pop\"]\n",
        "gapminder"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "After having calculated GDP, you may be interested in coding whether or not a country has above median GDP. We can turn `where` in Numpy into a function and create a new column using the `assign` method."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 18,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>year</th>\n",
              "      <th>gdp</th>\n",
              "      <th>above_median</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1952</td>\n",
              "      <td>6.567086e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1957</td>\n",
              "      <td>7.585449e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1962</td>\n",
              "      <td>8.758856e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1967</td>\n",
              "      <td>9.648014e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1972</td>\n",
              "      <td>9.678553e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1699</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1987</td>\n",
              "      <td>6.508241e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1700</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1992</td>\n",
              "      <td>7.422612e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1701</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1997</td>\n",
              "      <td>9.037851e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1702</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2002</td>\n",
              "      <td>8.015111e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1703</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2007</td>\n",
              "      <td>5.782658e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>1704 rows × 4 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "          country  year           gdp  above_median\n",
              "0     Afghanistan  1952  6.567086e+09         False\n",
              "1     Afghanistan  1957  7.585449e+09         False\n",
              "2     Afghanistan  1962  8.758856e+09         False\n",
              "3     Afghanistan  1967  9.648014e+09         False\n",
              "4     Afghanistan  1972  9.678553e+09         False\n",
              "...           ...   ...           ...           ...\n",
              "1699     Zimbabwe  1987  6.508241e+09         False\n",
              "1700     Zimbabwe  1992  7.422612e+09         False\n",
              "1701     Zimbabwe  1997  9.037851e+09         False\n",
              "1702     Zimbabwe  2002  8.015111e+09         False\n",
              "1703     Zimbabwe  2007  5.782658e+09         False\n",
              "\n",
              "[1704 rows x 4 columns]"
            ]
          },
          "execution_count": 18,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "# Returns a booleian array if a quantitative feature is above median values\n",
        "def is_above_median(var):\n",
        "  return np.where(var > np.median(var), True, False)\n",
        "\n",
        "# Apply the function to GDP\n",
        "gapminder[[\"country\",\"year\",\"gdp\"]].assign(above_median = lambda x: is_above_median(x[\"gdp\"]))"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 19,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>year</th>\n",
              "      <th>gdp</th>\n",
              "      <th>above_median</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1952</td>\n",
              "      <td>6.567086e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1957</td>\n",
              "      <td>7.585449e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1962</td>\n",
              "      <td>8.758856e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1967</td>\n",
              "      <td>9.648014e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1972</td>\n",
              "      <td>9.678553e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1699</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1987</td>\n",
              "      <td>6.508241e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1700</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1992</td>\n",
              "      <td>7.422612e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1701</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1997</td>\n",
              "      <td>9.037851e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1702</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2002</td>\n",
              "      <td>8.015111e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1703</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2007</td>\n",
              "      <td>5.782658e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>1704 rows × 4 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "          country  year           gdp  above_median\n",
              "0     Afghanistan  1952  6.567086e+09         False\n",
              "1     Afghanistan  1957  7.585449e+09         False\n",
              "2     Afghanistan  1962  8.758856e+09         False\n",
              "3     Afghanistan  1967  9.648014e+09         False\n",
              "4     Afghanistan  1972  9.678553e+09         False\n",
              "...           ...   ...           ...           ...\n",
              "1699     Zimbabwe  1987  6.508241e+09         False\n",
              "1700     Zimbabwe  1992  7.422612e+09         False\n",
              "1701     Zimbabwe  1997  9.037851e+09         False\n",
              "1702     Zimbabwe  2002  8.015111e+09         False\n",
              "1703     Zimbabwe  2007  5.782658e+09         False\n",
              "\n",
              "[1704 rows x 4 columns]"
            ]
          },
          "execution_count": 19,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "# Returns a booleian array if a quantitative feature is above median values\n",
        "def is_above_median(var):\n",
        "  return np.where(var > np.median(var), True, False)\n",
        "\n",
        "# Apply the function to GDP\n",
        "gapminder[[\"country\",\"year\",\"gdp\"]].assign(above_median = lambda x: is_above_median(x[\"gdp\"]))"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "👉 **NOTE:** When using `assign` you can see that we use `lambda x:` followed by the function. All we are doing is using `x` as a placeholder for our data frame (`country`, `year` and `gdp`). In doing so, we can select the column we are interested in using to create our new booleian variable.\n",
        "\n",
        "💁**Tip:** You may have noticed that we can string multiple methods together in Pandas. This is extremely useful, but you might find that your code will get too \"long\". If you find this to be the case, you can use `\\` to spread your code over multiple lines. Here's an example of how to do this with the above code:"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 20,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>year</th>\n",
              "      <th>gdp</th>\n",
              "      <th>above_median</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1952</td>\n",
              "      <td>6.567086e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1957</td>\n",
              "      <td>7.585449e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1962</td>\n",
              "      <td>8.758856e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1967</td>\n",
              "      <td>9.648014e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>1972</td>\n",
              "      <td>9.678553e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1699</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1987</td>\n",
              "      <td>6.508241e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1700</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1992</td>\n",
              "      <td>7.422612e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1701</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>1997</td>\n",
              "      <td>9.037851e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1702</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2002</td>\n",
              "      <td>8.015111e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1703</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>2007</td>\n",
              "      <td>5.782658e+09</td>\n",
              "      <td>False</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>1704 rows × 4 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "          country  year           gdp  above_median\n",
              "0     Afghanistan  1952  6.567086e+09         False\n",
              "1     Afghanistan  1957  7.585449e+09         False\n",
              "2     Afghanistan  1962  8.758856e+09         False\n",
              "3     Afghanistan  1967  9.648014e+09         False\n",
              "4     Afghanistan  1972  9.678553e+09         False\n",
              "...           ...   ...           ...           ...\n",
              "1699     Zimbabwe  1987  6.508241e+09         False\n",
              "1700     Zimbabwe  1992  7.422612e+09         False\n",
              "1701     Zimbabwe  1997  9.037851e+09         False\n",
              "1702     Zimbabwe  2002  8.015111e+09         False\n",
              "1703     Zimbabwe  2007  5.782658e+09         False\n",
              "\n",
              "[1704 rows x 4 columns]"
            ]
          },
          "execution_count": 20,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[[\"country\",\"year\",\"gdp\"]].\\\n",
        "    assign(above_median = lambda x: is_above_median(x[\"gdp\"]))"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "📝**Task:** This output is not very helpful. Try using some of the commands we have gone over to create a more useful data frame."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 21,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>year</th>\n",
              "      <th>gdp</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>2007</td>\n",
              "      <td>3.107929e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Algeria</td>\n",
              "      <td>1952</td>\n",
              "      <td>2.272563e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Algeria</td>\n",
              "      <td>1957</td>\n",
              "      <td>3.095611e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Algeria</td>\n",
              "      <td>1962</td>\n",
              "      <td>2.806140e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Algeria</td>\n",
              "      <td>1967</td>\n",
              "      <td>4.143324e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>847</th>\n",
              "      <td>Vietnam</td>\n",
              "      <td>2007</td>\n",
              "      <td>2.081746e+11</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>848</th>\n",
              "      <td>Yemen, Rep.</td>\n",
              "      <td>1992</td>\n",
              "      <td>2.512511e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>849</th>\n",
              "      <td>Yemen, Rep.</td>\n",
              "      <td>1997</td>\n",
              "      <td>3.351236e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>850</th>\n",
              "      <td>Yemen, Rep.</td>\n",
              "      <td>2002</td>\n",
              "      <td>4.179396e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>851</th>\n",
              "      <td>Yemen, Rep.</td>\n",
              "      <td>2007</td>\n",
              "      <td>5.065987e+10</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>852 rows × 3 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "         country  year           gdp\n",
              "0    Afghanistan  2007  3.107929e+10\n",
              "1        Algeria  1952  2.272563e+10\n",
              "2        Algeria  1957  3.095611e+10\n",
              "3        Algeria  1962  2.806140e+10\n",
              "4        Algeria  1967  4.143324e+10\n",
              "..           ...   ...           ...\n",
              "847      Vietnam  2007  2.081746e+11\n",
              "848  Yemen, Rep.  1992  2.512511e+10\n",
              "849  Yemen, Rep.  1997  3.351236e+10\n",
              "850  Yemen, Rep.  2002  4.179396e+10\n",
              "851  Yemen, Rep.  2007  5.065987e+10\n",
              "\n",
              "[852 rows x 3 columns]"
            ]
          },
          "execution_count": 21,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[[\"country\",\"year\",\"gdp\"]].\\\n",
        "  assign(above_median = lambda x: is_above_median(x[\"gdp\"])).\\\n",
        "    query(\"above_median == True\").\\\n",
        "      drop(\"above_median\", axis=1).\\\n",
        "        reset_index(drop=True)"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "### 1.9: Preparing data for `scikit-learn`, an example\n",
        "\n",
        "📝**Task:** Filter the data to only include observations from 2007."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 22,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>country</th>\n",
              "      <th>continent</th>\n",
              "      <th>year</th>\n",
              "      <th>life_exp</th>\n",
              "      <th>pop</th>\n",
              "      <th>gdp_per_cap</th>\n",
              "      <th>gdp</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>Afghanistan</td>\n",
              "      <td>Asia</td>\n",
              "      <td>2007</td>\n",
              "      <td>43.828</td>\n",
              "      <td>31889923</td>\n",
              "      <td>974.580338</td>\n",
              "      <td>3.107929e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>Albania</td>\n",
              "      <td>Europe</td>\n",
              "      <td>2007</td>\n",
              "      <td>76.423</td>\n",
              "      <td>3600523</td>\n",
              "      <td>5937.029526</td>\n",
              "      <td>2.137641e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>Algeria</td>\n",
              "      <td>Africa</td>\n",
              "      <td>2007</td>\n",
              "      <td>72.301</td>\n",
              "      <td>33333216</td>\n",
              "      <td>6223.367465</td>\n",
              "      <td>2.074449e+11</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>Angola</td>\n",
              "      <td>Africa</td>\n",
              "      <td>2007</td>\n",
              "      <td>42.731</td>\n",
              "      <td>12420476</td>\n",
              "      <td>4797.231267</td>\n",
              "      <td>5.958390e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>Argentina</td>\n",
              "      <td>Americas</td>\n",
              "      <td>2007</td>\n",
              "      <td>75.320</td>\n",
              "      <td>40301927</td>\n",
              "      <td>12779.379640</td>\n",
              "      <td>5.150336e+11</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>137</th>\n",
              "      <td>Vietnam</td>\n",
              "      <td>Asia</td>\n",
              "      <td>2007</td>\n",
              "      <td>74.249</td>\n",
              "      <td>85262356</td>\n",
              "      <td>2441.576404</td>\n",
              "      <td>2.081746e+11</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>138</th>\n",
              "      <td>West Bank and Gaza</td>\n",
              "      <td>Asia</td>\n",
              "      <td>2007</td>\n",
              "      <td>73.422</td>\n",
              "      <td>4018332</td>\n",
              "      <td>3025.349798</td>\n",
              "      <td>1.215686e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>139</th>\n",
              "      <td>Yemen, Rep.</td>\n",
              "      <td>Asia</td>\n",
              "      <td>2007</td>\n",
              "      <td>62.698</td>\n",
              "      <td>22211743</td>\n",
              "      <td>2280.769906</td>\n",
              "      <td>5.065987e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>140</th>\n",
              "      <td>Zambia</td>\n",
              "      <td>Africa</td>\n",
              "      <td>2007</td>\n",
              "      <td>42.384</td>\n",
              "      <td>11746035</td>\n",
              "      <td>1271.211593</td>\n",
              "      <td>1.493170e+10</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>141</th>\n",
              "      <td>Zimbabwe</td>\n",
              "      <td>Africa</td>\n",
              "      <td>2007</td>\n",
              "      <td>43.487</td>\n",
              "      <td>12311143</td>\n",
              "      <td>469.709298</td>\n",
              "      <td>5.782658e+09</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>142 rows × 7 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "                country continent  year  life_exp       pop   gdp_per_cap  \\\n",
              "0           Afghanistan      Asia  2007    43.828  31889923    974.580338   \n",
              "1               Albania    Europe  2007    76.423   3600523   5937.029526   \n",
              "2               Algeria    Africa  2007    72.301  33333216   6223.367465   \n",
              "3                Angola    Africa  2007    42.731  12420476   4797.231267   \n",
              "4             Argentina  Americas  2007    75.320  40301927  12779.379640   \n",
              "..                  ...       ...   ...       ...       ...           ...   \n",
              "137             Vietnam      Asia  2007    74.249  85262356   2441.576404   \n",
              "138  West Bank and Gaza      Asia  2007    73.422   4018332   3025.349798   \n",
              "139         Yemen, Rep.      Asia  2007    62.698  22211743   2280.769906   \n",
              "140              Zambia    Africa  2007    42.384  11746035   1271.211593   \n",
              "141            Zimbabwe    Africa  2007    43.487  12311143    469.709298   \n",
              "\n",
              "              gdp  \n",
              "0    3.107929e+10  \n",
              "1    2.137641e+10  \n",
              "2    2.074449e+11  \n",
              "3    5.958390e+10  \n",
              "4    5.150336e+11  \n",
              "..            ...  \n",
              "137  2.081746e+11  \n",
              "138  1.215686e+10  \n",
              "139  5.065987e+10  \n",
              "140  1.493170e+10  \n",
              "141  5.782658e+09  \n",
              "\n",
              "[142 rows x 7 columns]"
            ]
          },
          "execution_count": 22,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_07 = gapminder.query(\"year == 2007\").reset_index(drop=True)\n",
        "gapminder_07"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 115,
      "metadata": {},
      "outputs": [
        {
          "name": "stdout",
          "output_type": "stream",
          "text": [
            "                country continent  year  life_exp       pop   gdp_per_cap  \\\n",
            "0           Afghanistan      Asia  2007    43.828  31889923    974.580338   \n",
            "1               Albania    Europe  2007    76.423   3600523   5937.029526   \n",
            "2               Algeria    Africa  2007    72.301  33333216   6223.367465   \n",
            "3                Angola    Africa  2007    42.731  12420476   4797.231267   \n",
            "4             Argentina  Americas  2007    75.320  40301927  12779.379640   \n",
            "..                  ...       ...   ...       ...       ...           ...   \n",
            "137             Vietnam      Asia  2007    74.249  85262356   2441.576404   \n",
            "138  West Bank and Gaza      Asia  2007    73.422   4018332   3025.349798   \n",
            "139         Yemen, Rep.      Asia  2007    62.698  22211743   2280.769906   \n",
            "140              Zambia    Africa  2007    42.384  11746035   1271.211593   \n",
            "141            Zimbabwe    Africa  2007    43.487  12311143    469.709298   \n",
            "\n",
            "              gdp  \n",
            "0    3.107929e+10  \n",
            "1    2.137641e+10  \n",
            "2    2.074449e+11  \n",
            "3    5.958390e+10  \n",
            "4    5.150336e+11  \n",
            "..            ...  \n",
            "137  2.081746e+11  \n",
            "138  1.215686e+10  \n",
            "139  5.065987e+10  \n",
            "140  1.493170e+10  \n",
            "141  5.782658e+09  \n",
            "\n",
            "[142 rows x 7 columns]\n"
          ]
        }
      ],
      "source": [
        "gapminder_07 = gapminder.query(\"year == 2007\").reset_index(drop=True)\n",
        "gapminder_07"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "📝**Task:** Create a list of numeric variables (life expectancy, population, GDP per capita, and GDP) and string variables (continent)."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 23,
      "metadata": {},
      "outputs": [],
      "source": [
        "num_vars = [\"life_exp\",\"pop\",\"gdp_per_cap\",\"gdp\"]\n",
        "str_vars = [\"continent\"]"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "📝**Task:** Subset the new data frame to only include these variables. **Remember** you can add elements to a list by using `+`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 24,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>life_exp</th>\n",
              "      <th>pop</th>\n",
              "      <th>gdp_per_cap</th>\n",
              "      <th>gdp</th>\n",
              "      <th>continent</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>43.828</td>\n",
              "      <td>31889923</td>\n",
              "      <td>974.580338</td>\n",
              "      <td>3.107929e+10</td>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>76.423</td>\n",
              "      <td>3600523</td>\n",
              "      <td>5937.029526</td>\n",
              "      <td>2.137641e+10</td>\n",
              "      <td>Europe</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>72.301</td>\n",
              "      <td>33333216</td>\n",
              "      <td>6223.367465</td>\n",
              "      <td>2.074449e+11</td>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>42.731</td>\n",
              "      <td>12420476</td>\n",
              "      <td>4797.231267</td>\n",
              "      <td>5.958390e+10</td>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>75.320</td>\n",
              "      <td>40301927</td>\n",
              "      <td>12779.379640</td>\n",
              "      <td>5.150336e+11</td>\n",
              "      <td>Americas</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>137</th>\n",
              "      <td>74.249</td>\n",
              "      <td>85262356</td>\n",
              "      <td>2441.576404</td>\n",
              "      <td>2.081746e+11</td>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>138</th>\n",
              "      <td>73.422</td>\n",
              "      <td>4018332</td>\n",
              "      <td>3025.349798</td>\n",
              "      <td>1.215686e+10</td>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>139</th>\n",
              "      <td>62.698</td>\n",
              "      <td>22211743</td>\n",
              "      <td>2280.769906</td>\n",
              "      <td>5.065987e+10</td>\n",
              "      <td>Asia</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>140</th>\n",
              "      <td>42.384</td>\n",
              "      <td>11746035</td>\n",
              "      <td>1271.211593</td>\n",
              "      <td>1.493170e+10</td>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>141</th>\n",
              "      <td>43.487</td>\n",
              "      <td>12311143</td>\n",
              "      <td>469.709298</td>\n",
              "      <td>5.782658e+09</td>\n",
              "      <td>Africa</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>142 rows × 5 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "     life_exp       pop   gdp_per_cap           gdp continent\n",
              "0      43.828  31889923    974.580338  3.107929e+10      Asia\n",
              "1      76.423   3600523   5937.029526  2.137641e+10    Europe\n",
              "2      72.301  33333216   6223.367465  2.074449e+11    Africa\n",
              "3      42.731  12420476   4797.231267  5.958390e+10    Africa\n",
              "4      75.320  40301927  12779.379640  5.150336e+11  Americas\n",
              "..        ...       ...           ...           ...       ...\n",
              "137    74.249  85262356   2441.576404  2.081746e+11      Asia\n",
              "138    73.422   4018332   3025.349798  1.215686e+10      Asia\n",
              "139    62.698  22211743   2280.769906  5.065987e+10      Asia\n",
              "140    42.384  11746035   1271.211593  1.493170e+10    Africa\n",
              "141    43.487  12311143    469.709298  5.782658e+09    Africa\n",
              "\n",
              "[142 rows x 5 columns]"
            ]
          },
          "execution_count": 24,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_07 = gapminder_07.filter(items=num_vars+str_vars)\n",
        "gapminder_07"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "We will now apply `normalise` to numeric variables"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 30,
      "metadata": {},
      "outputs": [],
      "source": [
        "# Create a function that normalises variables\n",
        "def normalise(var):\n",
        "  return (var-var.mean())/var.std()\n",
        "\n",
        "# Apply normalise function over all numeric variables\n",
        "gapminder_07_normal = gapminder_07.filter(items=num_vars).\\\n",
        "  apply(lambda x: normalise(x), axis=0)"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "👉 **NOTE:** Normalising continuous features is good practice in machine learning, and becomes essential when dealing with algorithms that employ some kind of distance measure, such as principle components analysis.\n",
        "\n",
        "Along with normalising features, we need to transform categorical features into one-hot encoded dummy (that is, 0 or 1) features. One-hot simply means that a reference category in a feature will not appear in the transformed data frame. To apply one-hot encoding, we can use `get_dummies` in `Pandas`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 28,
      "metadata": {},
      "outputs": [
        {
          "name": "stdout",
          "output_type": "stream",
          "text": [
            "     Americas  Asia  Europe  Oceania\n",
            "0           0     1       0        0\n",
            "1           0     0       1        0\n",
            "2           0     0       0        0\n",
            "3           0     0       0        0\n",
            "4           1     0       0        0\n",
            "..        ...   ...     ...      ...\n",
            "137         0     1       0        0\n",
            "138         0     1       0        0\n",
            "139         0     1       0        0\n",
            "140         0     0       0        0\n",
            "141         0     0       0        0\n",
            "\n",
            "[142 rows x 4 columns]\n"
          ]
        }
      ],
      "source": [
        "gapminder_07_dummies = pd.get_dummies(gapminder_07[\"continent\"], \n",
        "                                      columns=[\"continent\"], \n",
        "                                      drop_first=True,\n",
        "                                      dtype=int)\n",
        "print(gapminder_07_dummies)"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "After having transformed our continuous and categorical features, we can concatenate the two into a new data frame."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 31,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "<div>\n",
              "<style scoped>\n",
              "    .dataframe tbody tr th:only-of-type {\n",
              "        vertical-align: middle;\n",
              "    }\n",
              "\n",
              "    .dataframe tbody tr th {\n",
              "        vertical-align: top;\n",
              "    }\n",
              "\n",
              "    .dataframe thead th {\n",
              "        text-align: right;\n",
              "    }\n",
              "</style>\n",
              "<table border=\"1\" class=\"dataframe\">\n",
              "  <thead>\n",
              "    <tr style=\"text-align: right;\">\n",
              "      <th></th>\n",
              "      <th>life_exp</th>\n",
              "      <th>pop</th>\n",
              "      <th>gdp_per_cap</th>\n",
              "      <th>gdp</th>\n",
              "      <th>Americas</th>\n",
              "      <th>Asia</th>\n",
              "      <th>Europe</th>\n",
              "      <th>Oceania</th>\n",
              "    </tr>\n",
              "  </thead>\n",
              "  <tbody>\n",
              "    <tr>\n",
              "      <th>0</th>\n",
              "      <td>-1.919936</td>\n",
              "      <td>-0.082178</td>\n",
              "      <td>-0.832468</td>\n",
              "      <td>-0.288250</td>\n",
              "      <td>0</td>\n",
              "      <td>1</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>1</th>\n",
              "      <td>0.779886</td>\n",
              "      <td>-0.273813</td>\n",
              "      <td>-0.446584</td>\n",
              "      <td>-0.295646</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>1</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>2</th>\n",
              "      <td>0.438463</td>\n",
              "      <td>-0.072401</td>\n",
              "      <td>-0.424318</td>\n",
              "      <td>-0.153810</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>3</th>\n",
              "      <td>-2.010799</td>\n",
              "      <td>-0.214066</td>\n",
              "      <td>-0.535216</td>\n",
              "      <td>-0.266522</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>4</th>\n",
              "      <td>0.688525</td>\n",
              "      <td>-0.025195</td>\n",
              "      <td>0.085483</td>\n",
              "      <td>0.080659</td>\n",
              "      <td>1</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>...</th>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "      <td>...</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>137</th>\n",
              "      <td>0.599815</td>\n",
              "      <td>0.279371</td>\n",
              "      <td>-0.718394</td>\n",
              "      <td>-0.153254</td>\n",
              "      <td>0</td>\n",
              "      <td>1</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>138</th>\n",
              "      <td>0.531315</td>\n",
              "      <td>-0.270983</td>\n",
              "      <td>-0.672999</td>\n",
              "      <td>-0.302674</td>\n",
              "      <td>0</td>\n",
              "      <td>1</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>139</th>\n",
              "      <td>-0.356947</td>\n",
              "      <td>-0.147739</td>\n",
              "      <td>-0.730898</td>\n",
              "      <td>-0.273324</td>\n",
              "      <td>0</td>\n",
              "      <td>1</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>140</th>\n",
              "      <td>-2.039541</td>\n",
              "      <td>-0.218635</td>\n",
              "      <td>-0.809402</td>\n",
              "      <td>-0.300559</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "    <tr>\n",
              "      <th>141</th>\n",
              "      <td>-1.948180</td>\n",
              "      <td>-0.214807</td>\n",
              "      <td>-0.871728</td>\n",
              "      <td>-0.307533</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "      <td>0</td>\n",
              "    </tr>\n",
              "  </tbody>\n",
              "</table>\n",
              "<p>142 rows × 8 columns</p>\n",
              "</div>"
            ],
            "text/plain": [
              "     life_exp       pop  gdp_per_cap       gdp  Americas  Asia  Europe  \\\n",
              "0   -1.919936 -0.082178    -0.832468 -0.288250         0     1       0   \n",
              "1    0.779886 -0.273813    -0.446584 -0.295646         0     0       1   \n",
              "2    0.438463 -0.072401    -0.424318 -0.153810         0     0       0   \n",
              "3   -2.010799 -0.214066    -0.535216 -0.266522         0     0       0   \n",
              "4    0.688525 -0.025195     0.085483  0.080659         1     0       0   \n",
              "..        ...       ...          ...       ...       ...   ...     ...   \n",
              "137  0.599815  0.279371    -0.718394 -0.153254         0     1       0   \n",
              "138  0.531315 -0.270983    -0.672999 -0.302674         0     1       0   \n",
              "139 -0.356947 -0.147739    -0.730898 -0.273324         0     1       0   \n",
              "140 -2.039541 -0.218635    -0.809402 -0.300559         0     0       0   \n",
              "141 -1.948180 -0.214807    -0.871728 -0.307533         0     0       0   \n",
              "\n",
              "     Oceania  \n",
              "0          0  \n",
              "1          0  \n",
              "2          0  \n",
              "3          0  \n",
              "4          0  \n",
              "..       ...  \n",
              "137        0  \n",
              "138        0  \n",
              "139        0  \n",
              "140        0  \n",
              "141        0  \n",
              "\n",
              "[142 rows x 8 columns]"
            ]
          },
          "execution_count": 31,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_07_cleaned = pd.concat([gapminder_07_normal, gapminder_07_dummies], axis=1)\n",
        "gapminder_07_cleaned"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "👉 **NOTE:** These kinds of transformations and concatenations will be employed **a lot** during this course, so please be sure to get used to them.\n",
        "\n",
        "As a final (optional) step, we can convert our Pandas data frame to a Numpy array by employing the `to_numpy` method."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 32,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "array([[-1.91993566, -0.08217844, -0.8324684 , ...,  1.        ,\n",
              "         0.        ,  0.        ],\n",
              "       [ 0.77988582, -0.27381326, -0.446584  , ...,  0.        ,\n",
              "         1.        ,  0.        ],\n",
              "       [ 0.43846339, -0.07240145, -0.42431811, ...,  0.        ,\n",
              "         0.        ,  0.        ],\n",
              "       ...,\n",
              "       [-0.35694651, -0.14773926, -0.73089796, ...,  1.        ,\n",
              "         0.        ,  0.        ],\n",
              "       [-2.03954118, -0.21863487, -0.8094021 , ...,  0.        ,\n",
              "         0.        ,  0.        ],\n",
              "       [-1.94818045, -0.21480678, -0.87172762, ...,  0.        ,\n",
              "         0.        ,  0.        ]], shape=(142, 8))"
            ]
          },
          "execution_count": 32,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_07_cleaned.to_numpy()"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "🏆**Challenge:** Which countries had above average life expectancy in 1952?\n",
        "\n",
        "Try these steps: 1. Define a function that returns a booleian if a value in a feature exceeds the average value. 2. Include only the year 1952 by using `query`. 3. Subset the data frame to only include country and life expectancy using `filter`. 4. Create a new booleian variable using the user defined function using `assign`. 5. Include all rows with above average life expectancy using `query`. 6. Subset the data frame to only include country using square bracket indexing. 7. Pull the unique values into an array using `unique`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 64,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/plain": [
              "array(['Albania', 'Argentina', 'Australia', 'Austria', 'Bahrain',\n",
              "       'Belgium', 'Bosnia and Herzegovina', 'Brazil', 'Bulgaria',\n",
              "       'Canada', 'Chile', 'Colombia', 'Costa Rica', 'Croatia', 'Cuba',\n",
              "       'Czech Republic', 'Denmark', 'Finland', 'France', 'Germany',\n",
              "       'Greece', 'Hong Kong, China', 'Hungary', 'Iceland', 'Ireland',\n",
              "       'Israel', 'Italy', 'Jamaica', 'Japan', 'Korea, Dem. Rep.',\n",
              "       'Kuwait', 'Lebanon', 'Mauritius', 'Mexico', 'Montenegro',\n",
              "       'Netherlands', 'New Zealand', 'Norway', 'Panama', 'Paraguay',\n",
              "       'Poland', 'Portugal', 'Puerto Rico', 'Reunion', 'Romania',\n",
              "       'Serbia', 'Singapore', 'Slovak Republic', 'Slovenia', 'Spain',\n",
              "       'Sri Lanka', 'Sweden', 'Switzerland', 'Taiwan', 'Thailand',\n",
              "       'Trinidad and Tobago', 'United Kingdom', 'United States',\n",
              "       'Uruguay', 'Venezuela'], dtype=object)"
            ]
          },
          "execution_count": 64,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "# Returns a booleian array based on whether a quantitative feature has above average values\n",
        "def is_above_average(var):\n",
        "  return np.where(var > np.mean(var), True, False)\n",
        "\n",
        "# Answer to be provided\n",
        "gapminder.\\\n",
        "  query(\"year == 1952\").\\\n",
        "    filter(items=[\"country\",\"life_exp\"]).\\\n",
        "      assign(above_average = lambda x: is_above_average(x[\"life_exp\"])).\\\n",
        "        query(\"above_average == True\")\\\n",
        "         [\"country\"].\\\n",
        "          unique()"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "## 📋 Part 2: Data visualisation with `lets_plot`(45 min)\n",
        "\n",
        "They say a picture paints a thousand words, and we are, more often than not, inclined to agree with them (whoever \"they\" are)! Thankfully, we can use perhaps the most widely used package, `lets_plot` (click [here](https://lets-plot.org/) for documentation) to paint such pictures or (more accurately) build such graphs.\n",
        "\n",
        "### 2.1 Histograms\n",
        "\n",
        "Suppose we want to plot the distribution of an outcome of interest. We can use a histogram to plot the distribution of life expectancy in `gapminder`.\n",
        "\n",
        "::: callout-tip\n",
        "**📝 Note: Step-by-step**\n",
        "\n",
        "If you are not familiar with using `lets_plot` for plotting, the line by line construction of the plot below might be useful. The `+` operator is used to add layers to the plot.\n",
        ":::"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 65,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"CijYJL\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"CijYJL\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"life_exp\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "}]\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"histogram\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "\"..count..\":[1.0,0.0,0.0,5.0,8.0,16.0,21.0,39.0,60.0,67.0,81.0,82.0,74.0,78.0,69.0,59.0,71.0,65.0,73.0,66.0,63.0,82.0,100.0,158.0,120.0,102.0,69.0,52.0,20.0,3.0],\n",
              "\"life_exp\":[23.251531999999997,25.310115999999997,27.368699999999997,29.427283999999997,31.485867999999996,33.54445199999999,35.603035999999996,37.66162,39.720203999999995,41.77878799999999,43.837371999999995,45.895956,47.954539999999994,50.01312399999999,52.071707999999994,54.130292,56.18887599999999,58.24745999999999,60.30604399999999,62.364627999999996,64.42321199999999,66.48179599999999,68.54038,70.598964,72.65754799999999,74.71613199999999,76.77471599999998,78.8333,80.89188399999999,82.95046799999999]\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"2\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x26c3fc07ef0>"
            ]
          },
          "execution_count": 65,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "(\n",
        "  ggplot(gapminder, aes(\"life_exp\")) +\n",
        "  geom_histogram()\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "👥 **WORK IN PAIRS/GROUPS:**\n",
        "\n",
        "Change a few parameters in the `geom_histogram` layer. \n",
        "-   Set `bins` to 40. \n",
        "-   Set `alpha` to 0.75. \n",
        "\n",
        "Afterwards, \n",
        "\n",
        "-   Specify `panel_grid_major_x=element_blank()` in the `theme` function. \n",
        "-   Edit the x and y axes using `labs`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 33,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"j2TX95\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"j2TX95\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"life_exp\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "}]\n",
              "},\n",
              "\"theme\":{\n",
              "\"panel_grid_major_x\":{\n",
              "\"blank\":true\n",
              "}\n",
              "},\n",
              "\"guides\":{\n",
              "\"x\":{\n",
              "\"title\":\"Life expectancy (years)\"\n",
              "},\n",
              "\"y\":{\n",
              "\"title\":\"Frequency\"\n",
              "}\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"histogram\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"bins\":40.0,\n",
              "\"alpha\":0.75,\n",
              "\"data\":{\n",
              "\"..count..\":[1.0,0.0,0.0,0.0,3.0,5.0,8.0,13.0,18.0,26.0,28.0,53.0,47.0,60.0,63.0,58.0,61.0,51.0,56.0,50.0,43.0,55.0,48.0,49.0,56.0,49.0,44.0,48.0,54.0,61.0,95.0,122.0,93.0,85.0,64.0,54.0,45.0,24.0,12.0,2.0],\n",
              "\"life_exp\":[23.32979425,24.85652275,26.383251249999997,27.909979749999998,29.43670825,30.96343675,32.49016525,34.016893749999994,35.54362225,37.070350749999996,38.59707924999999,40.12380775,41.650536249999995,43.17726474999999,44.703993249999996,46.230721749999994,47.75745024999999,49.284178749999995,50.81090724999999,52.33763574999999,53.864364249999994,55.39109274999999,56.917821249999996,58.44454974999999,59.97127824999999,61.498006749999995,63.02473524999999,64.55146374999998,66.07819225,67.60492074999999,69.13164924999998,70.65837775,72.18510624999999,73.71183474999998,75.23856325,76.76529174999999,78.29202024999998,79.81874875,81.34547724999999,82.87220574999998]\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"1\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cf13e30>"
            ]
          },
          "execution_count": 33,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "(\n",
        "  ggplot(gapminder, aes(\"life_exp\")) +\n",
        "  geom_histogram(alpha = 0.75, bins = 40) +\n",
        "  theme(panel_grid_major_x=element_blank()) +\n",
        "  labs(x = \"Life expectancy (years)\", y = \"Frequency\")\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot2.png)\n",
        "\n",
        "### 2.2 Bar graphs\n",
        "\n",
        "We counted the number of countries in each continent that are in `gapminder`. We will do something similar, using a subset of the data in 1997. We can use `lets_plot` to create a bar graph. We first need to specify what we want on the x and y axes, which constitute the first and second argument of the `aes` function."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 34,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"J8Hr5N\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"J8Hr5N\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"continent\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "}]\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"bar\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "\"..count..\":[33.0,30.0,52.0,25.0,2.0],\n",
              "\"continent\":[\"Asia\",\"Europe\",\"Africa\",\"Americas\",\"Oceania\"]\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"2\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cebf130>"
            ]
          },
          "execution_count": 34,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_97 = gapminder.query(\"year == 1997\")\n",
        "\n",
        "(\n",
        "  ggplot(gapminder_97,aes(\"continent\")) +\n",
        "  geom_bar()\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot3.png)\n",
        "\n",
        "👥 **WORK IN PAIRS/GROUPS:**\n",
        "\n",
        "-   Create a new data frame that counts the number of observations for each continent.\n",
        "-   Set `stat=\"identity\"` in `geom_bar`.\n",
        "-   Specify `panel_grid_major_x=element_blank()` in the `theme` function.\n",
        "-   Edit the x and y axes using `labs`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 35,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"yOOBrf\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"yOOBrf\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "\"continent\":[\"Africa\",\"Asia\",\"Europe\",\"Americas\",\"Oceania\"],\n",
              "\"count\":[52.0,33.0,30.0,25.0,2.0]\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"continent\",\n",
              "\"y\":\"count\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"count\"\n",
              "}]\n",
              "},\n",
              "\"theme\":{\n",
              "\"panel_grid_major_x\":{\n",
              "\"blank\":true\n",
              "}\n",
              "},\n",
              "\"guides\":{\n",
              "\"x\":{\n",
              "\"title\":\"Continent\"\n",
              "},\n",
              "\"y\":{\n",
              "\"title\":\"Count\"\n",
              "}\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"bar\",\n",
              "\"stat\":\"identity\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"3\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cf15ef0>"
            ]
          },
          "execution_count": 35,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_97_count = gapminder_97.value_counts(\"continent\").reset_index()\n",
        "\n",
        "(\n",
        "  ggplot(gapminder_97_count, aes(\"continent\",\"count\")) +\n",
        "  geom_bar(stat=\"identity\") +\n",
        "  theme(panel_grid_major_x=element_blank()) +\n",
        "  labs(x=\"Continent\",y=\"Count\")\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot4.png)\n",
        "\n",
        "👉 **NOTE:** Don't like vertical bars? Well, we can also create horizontal bars too by specifying `orientation=\"y\"` in `geom_bar`. Remember to switch \"count\" and \"continent\".\n",
        "\n",
        "### 2.3 Scatter plots\n",
        "\n",
        "Scatter plots are the best way to graphically summarise the relationship between two quantitative features. Suppose we wanted to visualise the \"effect\" of GDP per capita on life expectancy. We can use a scatter plot to achieve this."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 36,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"vRnoLh\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"vRnoLh\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "\"life_exp\":[28.801,30.332,31.997,34.02,36.088,38.438,39.854,40.822,41.674,41.763,42.129,43.828,55.23,59.28,64.82,66.22,67.69,68.93,70.42,72.0,71.581,72.95,75.651,76.423,43.077,45.685,48.303,51.407,54.518,58.014,61.368,65.799,67.744,69.152,70.994,72.301,30.015,31.999,34.0,35.985,37.928,39.483,39.942,39.906,40.647,40.963,41.003,42.731,62.485,64.399,65.142,65.634,67.065,68.481,69.942,70.774,71.868,73.275,74.34,75.32,69.12,70.33,70.93,71.1,71.93,73.49,74.74,76.32,77.56,78.83,80.37,81.235,66.8,67.48,69.54,70.14,70.63,72.17,73.18,74.94,76.04,77.51,78.98,79.829,50.939,53.832,56.923,59.923,63.3,65.593,69.052,70.75,72.601,73.925,74.795,75.635,37.484,39.348,41.216,43.453,45.252,46.923,50.009,52.819,56.018,59.412,62.013,64.062,68.0,69.24,70.25,70.94,71.44,72.8,73.93,75.35,76.46,77.53,78.32,79.441,38.223,40.358,42.618,44.885,47.014,49.19,50.904,52.337,53.919,54.777,54.406,56.728,40.414,41.89,43.428,45.032,46.714,50.023,53.859,57.251,59.957,62.05,63.883,65.554,53.82,58.45,61.93,64.79,67.45,69.86,70.69,71.14,72.178,73.244,74.09,74.852,47.622,49.618,51.52,53.298,56.024,59.319,61.484,63.622,62.745,52.556,46.634,50.728,50.917,53.285,55.665,57.632,59.504,61.489,63.336,65.205,67.057,69.388,71.006,72.39,59.6,66.61,69.51,70.42,70.9,70.81,71.08,71.34,71.19,70.32,72.14,73.005,31.975,34.906,37.814,40.697,43.591,46.137,48.122,49.557,50.26,50.324,50.65,52.295,39.031,40.533,42.045,43.548,44.057,45.91,47.471,48.211,44.736,45.326,47.36,49.58,39.417,41.366,43.415,45.415,40.317,31.22,50.957,53.914,55.803,56.534,56.752,59.723,38.523,40.428,42.643,44.799,47.049,49.355,52.961,54.985,54.314,52.199,49.856,50.43,68.75,69.96,71.3,72.13,72.88,74.21,75.76,76.86,77.95,78.61,79.77,80.653,35.463,37.464,39.475,41.478,43.457,46.775,48.295,50.485,49.396,46.066,43.308,44.741,38.092,39.881,41.716,43.601,45.569,47.383,49.517,51.051,51.724,51.573,50.525,50.651,54.745,56.074,57.924,60.523,63.441,67.052,70.565,72.492,74.126,75.816,77.86,78.553,44.0,50.54896,44.50136,58.38112,63.11888,63.96736,65.525,67.274,68.69,70.426,72.028,72.961,50.643,55.118,57.863,59.963,61.623,63.837,66.653,67.768,68.421,70.313,71.682,72.889,40.715,42.46,44.467,46.472,48.944,50.939,52.933,54.926,57.939,60.66,62.974,65.152,39.143,40.652,42.122,44.056,45.989,47.804,47.784,47.412,45.548,42.587,44.966,46.462,42.111,45.053,48.435,52.04,54.907,55.625,56.695,57.47,56.433,52.962,52.97,55.322,57.206,60.026,62.842,65.424,67.849,70.75,73.45,74.752,75.713,77.26,78.123,78.782,40.477,42.469,44.93,47.35,49.801,52.374,53.983,54.655,52.044,47.991,46.832,48.328,61.21,64.77,67.13,68.5,69.61,70.64,70.46,71.52,72.527,73.68,74.876,75.748,59.421,62.325,65.246,68.29,70.723,72.649,73.717,74.174,74.414,76.151,77.158,78.273,66.87,69.03,69.9,70.38,70.29,70.71,70.96,71.58,72.4,74.01,75.51,76.486,70.78,71.81,72.35,72.96,73.47,74.69,74.63,74.8,75.33,76.11,77.18,78.332,34.812,37.328,39.693,42.074,44.366,46.519,48.812,50.04,51.604,53.157,53.373,54.791,45.928,49.828,53.459,56.751,59.631,61.788,63.727,66.046,68.457,69.957,70.847,72.235,48.357,51.356,54.64,56.678,58.796,61.31,64.342,67.231,69.613,72.312,74.173,74.994,41.893,44.444,46.992,49.293,51.137,53.319,56.006,59.797,63.674,67.217,69.806,71.338,45.262,48.57,52.307,55.855,58.207,56.696,56.604,63.154,66.798,69.535,70.734,71.878,34.482,35.983,37.485,38.987,40.516,42.024,43.662,45.664,47.545,48.245,49.348,51.579,35.928,38.047,40.158,42.189,44.142,44.535,43.89,46.453,49.991,53.378,55.24,58.04,34.078,36.667,40.059,42.115,43.515,44.51,44.916,46.684,48.091,49.402,50.725,52.947,66.55,67.49,68.75,69.83,70.87,72.52,74.55,74.83,75.7,77.13,78.37,79.313,67.41,68.93,70.51,71.55,72.38,73.83,74.89,76.34,77.46,78.64,79.59,80.657,37.003,38.999,40.489,44.598,48.69,52.79,56.564,60.19,61.366,60.461,56.761,56.735,30.0,32.065,33.896,35.857,38.308,41.842,45.58,49.265,52.644,55.861,58.041,59.448,67.5,69.1,70.3,70.8,71.0,72.5,73.8,74.847,76.07,77.34,78.67,79.406,43.149,44.779,46.452,48.072,49.875,51.756,53.744,55.729,57.501,58.556,58.453,60.022,65.86,67.86,69.51,71.0,72.34,73.68,75.24,76.67,77.03,77.869,78.256,79.483,42.023,44.142,46.954,50.016,53.738,56.029,58.137,60.782,63.373,66.322,68.978,70.259,33.609,34.558,35.753,37.197,38.842,40.762,42.891,45.552,48.576,51.455,53.676,56.007,32.5,33.489,34.488,35.492,36.486,37.465,39.327,41.245,43.266,44.873,45.504,46.388,37.579,40.696,43.59,46.243,48.042,49.923,51.461,53.636,55.089,56.671,58.137,60.916,41.912,44.665,48.041,50.924,53.884,57.402,60.909,64.492,66.399,67.659,68.565,70.198,60.96,64.75,67.65,70.0,72.0,73.6,75.45,76.2,77.601,80.0,81.495,82.208,64.03,66.41,67.96,69.5,69.76,69.95,69.39,69.58,69.17,71.04,72.59,73.338,72.49,73.47,73.68,73.73,74.46,76.11,76.99,77.23,78.77,78.95,80.5,81.757,37.373,40.249,43.605,47.193,50.651,54.208,56.596,58.553,60.223,61.765,62.879,64.698,37.468,39.918,42.518,45.964,49.203,52.702,56.159,60.137,62.681,66.041,68.588,70.65,44.869,47.181,49.325,52.469,55.234,57.702,59.62,63.04,65.742,68.042,69.451,70.964,45.32,48.437,51.457,54.459,56.95,60.413,62.038,65.044,59.461,58.811,57.046,59.545,66.91,68.9,70.29,71.08,71.28,72.03,73.1,74.36,75.467,76.122,77.783,78.885,65.39,67.84,69.39,70.75,71.63,73.06,74.45,75.6,76.93,78.269,79.696,80.745,65.94,67.81,69.24,71.06,72.19,73.48,74.98,76.42,77.44,78.82,80.24,80.546,58.53,62.61,65.61,67.51,69.0,70.11,71.21,71.77,71.766,72.262,72.047,72.567,63.03,65.5,68.73,71.43,73.42,75.38,77.11,78.67,79.36,80.69,82.0,82.603,43.158,45.669,48.126,51.629,56.528,61.134,63.739,65.869,68.015,69.772,71.263,72.535,42.27,44.686,47.949,50.654,53.559,56.155,58.766,59.339,59.285,54.407,50.992,54.11,50.056,54.081,56.656,59.942,63.983,67.159,69.1,70.647,69.978,67.727,66.662,67.297,47.453,52.681,55.292,57.716,62.612,64.766,67.123,69.81,72.244,74.647,77.045,78.623,55.565,58.033,60.47,64.624,67.712,69.343,71.309,74.174,75.19,76.156,76.904,77.588,55.928,59.489,62.094,63.87,65.421,66.099,66.983,67.926,69.292,70.265,71.028,71.993,42.138,45.047,47.747,48.492,49.767,52.208,55.078,57.18,59.685,55.558,44.593,42.592,38.48,39.486,40.502,41.536,42.614,43.764,44.852,46.027,40.802,42.221,43.753,45.678,42.723,45.289,47.808,50.227,52.773,57.442,62.155,66.234,68.755,71.555,72.737,73.952,36.681,38.865,40.848,42.881,44.851,46.881,48.969,49.35,52.214,54.978,57.286,59.443,36.256,37.207,38.41,39.487,41.766,43.767,45.642,47.457,49.42,47.495,45.009,48.303,48.463,52.102,55.737,59.371,63.01,65.256,68.0,69.5,70.693,71.938,73.044,74.241,33.685,35.307,36.936,38.487,39.977,41.714,43.916,46.364,48.388,49.903,51.818,54.467,40.543,42.338,44.248,46.289,48.437,50.852,53.599,56.145,58.333,60.43,62.247,64.164,50.986,58.089,60.246,61.557,62.944,64.93,66.711,68.74,69.745,70.736,71.954,72.801,50.789,55.19,58.299,60.11,62.361,65.032,67.405,69.498,71.455,73.67,74.902,76.195,42.244,45.248,48.251,51.253,53.754,55.491,57.489,60.222,61.271,63.625,65.033,66.803,59.164,61.448,63.728,67.178,70.636,73.066,74.101,74.865,75.435,75.445,73.981,74.543,42.873,45.423,47.924,50.335,52.862,55.73,59.65,62.677,65.393,67.66,69.615,71.164,31.286,33.779,36.161,38.113,40.328,42.495,42.795,42.861,44.284,46.344,44.026,42.082,36.319,41.905,45.108,49.379,53.07,56.059,58.056,58.339,59.32,60.328,59.908,62.069,41.725,45.226,48.386,51.159,53.867,56.437,58.968,60.835,61.999,58.909,51.479,52.906,36.157,37.686,39.393,41.472,43.971,46.748,49.594,52.537,55.727,59.426,61.34,63.785,72.13,72.99,73.23,73.82,73.75,75.24,76.05,76.83,77.42,78.03,78.53,79.762,69.39,70.26,71.24,71.52,71.89,72.22,73.84,74.32,76.33,77.55,79.11,80.204,42.314,45.432,48.632,51.884,55.151,57.47,59.298,62.008,65.843,68.426,70.836,72.899,37.444,38.598,39.487,40.118,40.546,41.291,42.598,44.555,47.391,51.313,54.496,56.867,36.324,37.802,39.36,41.04,42.821,44.514,45.826,46.886,47.472,47.464,46.608,46.859,72.67,73.44,73.47,74.08,74.34,75.37,75.97,75.89,77.32,78.32,79.05,80.196,37.578,40.08,43.165,46.988,52.143,57.367,62.728,67.734,71.197,72.499,74.193,75.64,43.436,45.557,47.67,49.8,51.929,54.043,56.158,58.245,60.838,61.818,63.61,65.483,55.191,59.201,61.817,64.071,66.216,68.681,70.472,71.523,72.462,73.738,74.712,75.537,62.649,63.196,64.361,64.951,65.815,66.353,66.874,67.378,68.225,69.4,70.755,71.752,43.902,46.263,49.096,51.445,55.448,58.447,61.406,64.134,66.458,68.386,69.906,71.421,47.752,51.334,54.757,56.393,58.065,60.06,62.082,64.151,66.458,68.564,70.303,71.688,61.31,65.77,67.64,69.61,70.85,70.67,71.32,70.98,70.99,72.75,74.67,75.563,59.82,61.51,64.39,66.6,69.26,70.41,72.77,74.06,74.86,75.97,77.29,78.098,64.28,68.54,69.62,71.1,72.16,73.44,73.75,74.63,73.911,74.917,77.778,78.746,52.724,55.09,57.666,60.542,64.274,67.064,69.885,71.913,73.615,74.772,75.744,76.442,61.05,64.1,66.8,66.8,69.21,69.46,69.66,69.53,69.36,69.72,71.322,72.476,40.0,41.5,43.0,44.1,44.6,45.0,46.218,44.02,23.599,36.087,43.413,46.242,46.471,48.945,51.893,54.425,56.48,58.55,60.351,61.728,62.742,63.306,64.337,65.528,39.875,42.868,45.914,49.901,53.886,58.69,63.012,66.295,68.768,70.533,71.626,72.777,37.278,39.329,41.454,43.563,45.815,48.879,52.379,55.769,58.196,60.187,61.6,63.062,57.996,61.685,64.531,66.914,68.7,70.3,70.162,71.218,71.659,72.232,73.213,74.002,30.331,31.57,32.767,34.113,35.4,36.788,38.445,40.006,38.333,39.897,41.012,42.568,60.396,63.179,65.798,67.946,69.521,70.795,71.76,73.56,75.788,77.158,78.77,79.972,64.36,67.45,70.33,70.98,70.35,70.45,70.8,71.08,71.38,72.71,73.8,74.663,65.57,67.85,69.15,69.18,69.82,70.97,71.063,72.25,73.64,75.13,76.66,77.926,32.978,34.977,36.981,38.977,40.973,41.974,42.955,44.501,39.658,43.795,45.936,48.159,45.009,47.985,49.951,51.927,53.696,55.527,58.161,60.834,61.888,60.236,53.365,49.339,64.94,66.66,69.69,71.44,73.06,74.39,76.3,76.9,77.57,78.77,79.78,80.941,57.593,61.456,62.192,64.266,65.042,65.949,68.757,69.011,70.379,70.457,70.815,72.396,38.635,39.624,40.87,42.858,45.083,47.8,50.338,51.744,53.556,55.373,56.369,58.556,41.407,43.424,44.992,46.633,49.552,52.537,55.561,57.678,58.474,54.289,43.869,39.613,71.86,72.49,73.37,74.16,74.72,75.44,76.42,77.19,78.16,79.39,80.04,80.884,69.62,70.56,71.32,72.77,73.78,75.39,76.21,77.41,78.03,79.37,80.62,81.701,45.883,48.284,50.305,53.655,57.296,61.195,64.59,66.974,69.249,71.527,73.053,74.143,58.5,62.4,65.2,67.5,69.39,70.59,72.16,73.4,74.26,75.25,76.99,78.4,41.215,42.974,44.246,45.757,47.62,49.919,50.608,51.535,50.44,48.466,49.651,52.517,50.848,53.63,56.061,58.285,60.405,62.494,64.597,66.084,67.298,67.521,68.564,70.616,38.596,41.208,43.922,46.769,49.759,52.887,55.471,56.941,58.061,58.39,57.561,58.42,59.1,61.8,64.9,65.4,65.9,68.3,68.832,69.582,69.862,69.465,68.976,69.819,44.6,47.1,49.579,52.053,55.602,59.837,64.048,66.894,70.001,71.973,73.042,73.923,43.585,48.079,52.098,54.336,57.005,59.507,61.036,63.108,66.146,68.835,70.845,71.777,39.978,42.571,45.344,48.051,51.016,50.35,49.849,51.509,48.825,44.578,47.813,51.542,69.18,70.42,70.76,71.36,72.01,72.76,74.04,75.007,76.42,77.218,78.471,79.425,68.44,69.49,70.21,70.76,71.34,73.38,74.65,75.02,76.09,76.81,77.31,78.242,66.071,67.044,68.253,68.468,68.673,69.481,70.805,71.918,72.752,74.223,75.307,76.384,55.088,57.907,60.77,63.479,65.712,67.456,68.557,70.19,71.15,72.146,72.766,73.747,40.412,42.887,45.363,47.838,50.254,55.764,58.816,62.82,67.662,70.672,73.017,74.249,43.16,45.671,48.127,51.631,56.532,60.765,64.406,67.046,69.718,71.096,72.37,73.422,32.548,33.97,35.18,36.984,39.848,44.175,49.113,52.922,55.599,58.02,60.308,62.698,42.038,44.077,46.023,47.768,50.107,51.386,51.821,50.821,46.1,40.238,39.193,42.384,48.451,50.469,52.358,53.995,55.635,57.674,60.363,62.351,60.377,46.809,39.989,43.487],\n",
              "\"gdp_per_cap\":[779.4453145,820.8530296,853.10071,836.1971382,739.9811058,786.11336,978.0114388,852.3959448,649.3413952,635.341351,726.7340548,974.5803384,1601.056136,1942.284244,2312.888958,2760.196931,3313.422188,3533.00391,3630.880722,3738.932735,2497.437901,3193.054604,4604.211737,5937.029526,2449.008185,3013.976023,2550.81688,3246.991771,4182.663766,4910.416756,5745.160213,5681.358539,5023.216647,4797.295051,5288.040382,6223.367465,3520.610273,3827.940465,4269.276742,5522.776375,5473.288005,3008.647355,2756.953672,2430.208311,2627.845685,2277.140884,2773.287312,4797.231267,5911.315053,6856.856212,7133.166023,8052.953021,9443.038526,10079.02674,8997.897412,9139.671389,9308.41871,10967.28195,8797.640716,12779.37964,10039.59564,10949.64959,12217.22686,14526.12465,16788.62948,18334.19751,19477.00928,21888.88903,23424.76683,26997.93657,30687.75473,34435.36744,6137.076492,8842.59803,10750.72111,12834.6024,16661.6256,19749.4223,21597.08362,23687.82607,27042.01868,29095.92066,32417.60769,36126.4927,9867.084765,11635.79945,12753.27514,14804.6727,18268.65839,19340.10196,19211.14731,18524.02406,19035.57917,20292.01679,23403.55927,29796.04834,684.2441716,661.6374577,686.3415538,721.1860862,630.2336265,659.8772322,676.9818656,751.9794035,837.8101643,972.7700352,1136.39043,1391.253792,8343.105127,9714.960623,10991.20676,13149.04119,16672.14356,19117.97448,20979.84589,22525.56308,25575.57069,27561.19663,30485.88375,33692.60508,1062.7522,959.6010805,949.4990641,1035.831411,1085.796879,1029.161251,1277.897616,1225.85601,1191.207681,1232.975292,1372.877931,1441.284873,2677.326347,2127.686326,2180.972546,2586.886053,2980.331339,3548.097832,3156.510452,2753.69149,2961.699694,3326.143191,3413.26269,3822.137084,973.5331948,1353.989176,1709.683679,2172.352423,2860.16975,3528.481305,4126.613157,4314.114757,2546.781445,4766.355904,6018.975239,7446.298803,851.2411407,918.2325349,983.6539764,1214.709294,2263.611114,3214.857818,4551.14215,6205.88385,7954.111645,8647.142313,11003.60508,12569.85177,2108.944355,2487.365989,3336.585802,3429.864357,4985.711467,6660.118654,7030.835878,7807.095818,6950.283021,7957.980824,8131.212843,9065.800825,2444.286648,3008.670727,4254.337839,5577.0028,6597.494398,7612.240438,8224.191647,8239.854824,6302.623438,5970.38876,7696.777725,10680.79282,543.2552413,617.1834648,722.5120206,794.8265597,854.7359763,743.3870368,807.1985855,912.0631417,931.7527731,946.2949618,1037.645221,1217.032994,339.2964587,379.5646281,355.2032273,412.9775136,464.0995039,556.1032651,559.603231,621.8188189,631.6998778,463.1151478,446.4035126,430.0706916,368.4692856,434.0383364,496.9136476,523.4323142,421.6240257,524.9721832,624.4754784,683.8955732,682.3031755,734.28517,896.2260153,1713.778686,1172.667655,1313.048099,1399.607441,1508.453148,1684.146528,1783.432873,2367.983282,2602.664206,1793.163278,1694.337469,1934.011449,2042.09524,11367.16112,12489.95006,13462.48555,16076.58803,18970.57086,22090.88306,22898.79214,26626.51503,26342.88426,28954.92589,33328.96507,36319.23501,1071.310713,1190.844328,1193.068753,1136.056615,1070.013275,1109.374338,956.7529907,844.8763504,747.9055252,740.5063317,738.6906068,706.016537,1178.665927,1308.495577,1389.817618,1196.810565,1104.103987,1133.98495,797.9081006,952.386129,1058.0643,1004.961353,1156.18186,1704.063724,3939.978789,4315.622723,4519.094331,5106.654313,5494.024437,4756.763836,5095.665738,5547.063754,7596.125964,10118.05318,10778.78385,13171.63885,400.448611,575.9870009,487.6740183,612.7056934,676.9000921,741.2374699,962.4213805,1378.904018,1655.784158,2289.234136,3119.280896,4959.114854,2144.115096,2323.805581,2492.351109,2678.729839,3264.660041,3815.80787,4397.575659,4903.2191,5444.648617,6117.361746,5755.259962,7006.580419,1102.990936,1211.148548,1406.648278,1876.029643,1937.577675,1172.603047,1267.100083,1315.980812,1246.90737,1173.618235,1075.811558,986.1478792,780.5423257,905.8602303,896.3146335,861.5932424,904.8960685,795.757282,673.7478181,672.774812,457.7191807,312.188423,241.1658765,277.5518587,2125.621418,2315.056572,2464.783157,2677.939642,3213.152683,3259.178978,4879.507522,4201.194937,4016.239529,3484.164376,3484.06197,3632.557798,2627.009471,2990.010802,3460.937025,4161.727834,5118.146939,5926.876967,5262.734751,5629.915318,6160.416317,6677.045314,7723.447195,9645.06142,1388.594732,1500.895925,1728.869428,2052.050473,2378.201111,2517.736547,2602.710169,2156.956069,1648.073791,1786.265407,1648.800823,1544.750112,3119.23652,4338.231617,5477.890018,6960.297861,9164.090127,11305.38517,13221.82184,13822.58394,8447.794873,9875.604515,11628.38895,14619.22272,5586.53878,6092.174359,5180.75591,5690.268015,5305.445256,6380.494966,7316.918107,7532.924763,5592.843963,5431.990415,6340.646683,8948.102923,6876.14025,8256.343918,10136.86713,11399.44489,13108.4536,14800.16062,15377.22855,16310.4434,14297.02122,16048.51424,17596.21022,22833.30851,9692.385245,11099.65935,13583.31351,15937.21123,18866.20721,20422.9015,21688.04048,25116.17581,26406.73985,29804.34567,32166.50006,35278.41874,2669.529475,2864.969076,3020.989263,3020.050513,3694.212352,3081.761022,2879.468067,2880.102568,2377.156192,1895.016984,1908.260867,2082.481567,1397.717137,1544.402995,1662.137359,1653.723003,2189.874499,2681.9889,2861.092386,2899.842175,3044.214214,3614.101285,4563.808154,6025.374752,3522.110717,3780.546651,4086.114078,4579.074215,5280.99471,6679.62326,7213.791267,6481.776993,7103.702595,7429.455877,5773.044512,6873.262326,1418.822445,1458.915272,1693.335853,1814.880728,2024.008147,2785.493582,3503.729636,3885.46071,3794.755195,4173.181797,4754.604414,5581.180998,3048.3029,3421.523218,3776.803627,4358.595393,4520.246008,5138.922374,4098.344175,4140.442097,4444.2317,5154.825496,5351.568666,5728.353514,375.6431231,426.0964081,582.8419714,915.5960025,672.4122571,958.5668124,927.8253427,966.8968149,1132.055034,2814.480755,7703.4959,12154.08975,328.9405571,344.1618859,380.9958433,468.7949699,514.3242082,505.7538077,524.8758493,521.1341333,582.8585102,913.47079,765.3500015,641.3695236,362.1462796,378.9041632,419.4564161,516.1186438,566.2439442,556.8083834,577.8607471,573.7413142,421.3534653,515.8894013,530.0535319,690.8055759,6424.519071,7545.415386,9371.842561,10921.63626,14358.8759,15605.42283,18533.15761,21141.01223,20647.16499,23723.9502,28204.59057,33207.0844,7029.809327,8662.834898,10560.48553,12999.91766,16107.19171,18292.63514,20293.89746,22066.44214,24703.79615,25889.78487,28926.03234,30470.0167,4293.476475,4976.198099,6631.459222,8358.761987,11401.94841,21745.57328,15113.36194,11864.40844,13522.15752,14722.84188,12521.71392,13206.48452,485.2306591,520.9267111,599.650276,734.7829124,756.0868363,884.7552507,835.8096108,611.6588611,665.6244126,653.7301704,660.5855997,752.7497265,7144.114393,10187.82665,12902.46291,14745.62561,18016.18027,20512.92123,22031.53274,24639.18566,26505.30317,27788.88416,30035.80198,32170.37442,911.2989371,1043.561537,1190.041118,1125.69716,1178.223708,993.2239571,876.032569,847.0061135,925.060154,1005.245812,1111.984578,1327.60891,3530.690067,4916.299889,6017.190733,8513.097016,12724.82957,14195.52428,15268.42089,16120.52839,17541.49634,18747.69814,22514.2548,27538.41188,2428.237769,2617.155967,2750.364446,3242.531147,4031.408271,4879.992748,4820.49479,4246.485974,4439.45084,4684.313807,4858.347495,5186.050003,510.1964923,576.2670245,686.3736739,708.7595409,741.6662307,874.6858643,857.2503577,805.5724718,794.3484384,869.4497668,945.5835837,942.6542111,299.850319,431.7904566,522.0343725,715.5806402,820.2245876,764.7259628,838.1239671,736.4153921,745.5398706,796.6644681,575.7047176,579.231743,1840.366939,1726.887882,1796.589032,1452.057666,1654.456946,1874.298931,2011.159549,1823.015995,1456.309517,1341.726931,1270.364932,1201.637154,2194.926204,2220.487682,2291.156835,2538.269358,2529.842345,3203.208066,3121.760794,3023.096699,3081.694603,3160.454906,3099.72866,3548.330846,3054.421209,3629.076457,4692.648272,6197.962814,8315.928145,11186.14125,14560.53051,20038.47269,24757.60301,28377.63219,30209.01516,39724.97867,5263.673816,6040.180011,7550.359877,9326.64467,10168.65611,11674.83737,12545.99066,12986.47998,10535.62855,11712.7768,14843.93556,18008.94444,7267.688428,9244.001412,10350.15906,13319.89568,15798.06362,19654.96247,23269.6075,26923.20628,25144.39201,28061.09966,31163.20196,36180.78919,546.5657493,590.061996,658.3471509,700.7706107,724.032527,813.337323,855.7235377,976.5126756,1164.406809,1458.817442,1746.769454,2452.210407,749.6816546,858.9002707,849.2897701,762.4317721,1111.107907,1382.702056,1516.872988,1748.356961,2383.140898,3119.335603,2873.91287,3540.651564,3035.326002,3290.257643,4187.329802,5906.731805,9613.818607,11888.59508,7608.334602,6642.881371,7235.653188,8263.590301,9240.761975,11605.71449,4129.766056,6229.333562,8341.737815,8931.459811,9576.037596,14688.23507,14517.90711,11643.57268,3745.640687,3076.239795,4390.717312,4471.061906,5210.280328,5599.077872,6631.597314,7655.568963,9530.772896,11150.98113,12618.32141,13872.86652,17558.81555,24521.94713,34077.04939,40675.99635,4086.522128,5385.278451,7105.630706,8393.741404,12786.93223,13306.61921,15367.0292,17122.47986,18051.52254,20896.60924,21905.59514,25523.2771,4931.404155,6248.656232,8243.58234,10022.40131,12269.27378,14255.98475,16537.4835,19207.23482,22013.64486,24675.02446,27968.09817,28569.7197,2898.530881,4756.525781,5246.107524,6124.703451,7433.889293,6650.195573,6068.05135,6351.237495,7404.923685,7121.924704,6994.774861,7320.880262,3216.956347,4317.694365,6576.649461,9847.788607,14778.78636,16610.37701,19384.10571,22375.94189,26824.89511,28816.58499,28604.5919,31656.06806,1546.907807,1886.080591,2348.009158,2741.796252,2110.856309,2852.351568,4161.415959,4448.679912,3431.593647,3645.379572,3844.917194,4519.461171,853.540919,944.4383152,896.9663732,1056.736457,1222.359968,1267.613204,1348.225791,1361.936856,1341.921721,1360.485021,1287.514732,1463.249282,1088.277758,1571.134655,1621.693598,2143.540609,3701.621503,4106.301249,4106.525293,4106.492315,3726.063507,1690.756814,1646.758151,1593.06548,1030.592226,1487.593537,1536.344387,2029.228142,3030.87665,4657.22102,5622.942464,8533.088805,12104.27872,15993.52796,19233.98818,23348.13973,108382.3529,113523.1329,95458.11176,80894.88326,109347.867,59265.47714,31354.03573,28118.42998,34932.91959,40300.61996,35110.10566,47306.98978,4834.804067,6089.786934,5714.560611,6006.983042,7486.384341,8659.696836,7640.519521,5377.091329,6890.806854,8754.96385,9313.93883,10461.05868,298.8462121,335.9971151,411.8006266,498.6390265,496.5815922,745.3695408,797.2631074,773.9932141,977.4862725,1186.147994,1275.184575,1569.331442,575.5729961,620.9699901,634.1951625,713.6036483,803.0054535,640.3224383,572.1995694,506.1138573,636.6229191,609.1739508,531.4823679,414.5073415,2387.54806,3448.284395,6757.030816,18772.75169,21011.49721,21951.21176,17364.27538,11770.5898,9640.138501,9467.446056,9534.677467,12057.49928,1443.011715,1589.20275,1643.38711,1634.047282,1748.562982,1544.228586,1302.878658,1155.441948,1040.67619,986.2958956,894.6370822,1044.770126,369.1650802,416.3698064,427.9010856,495.5147806,584.6219709,663.2236766,632.8039209,635.5173634,563.2000145,692.2758103,665.4231186,759.3499101,1831.132894,1810.066992,2036.884944,2277.742396,2849.09478,3827.921571,4920.355951,5249.802653,7277.912802,10132.90964,10206.97794,12451.6558,452.3369807,490.3821867,496.1743428,545.0098873,581.3688761,686.3952693,618.0140641,684.1715576,739.014375,790.2579846,951.4097518,1042.581557,743.1159097,846.1202613,1055.896036,1421.145193,1586.851781,1497.492223,1481.150189,1421.603576,1361.369784,1483.136136,1579.019543,1803.151496,1967.955707,2034.037981,2529.067487,2475.387562,2575.484158,3710.982963,3688.037739,4783.586903,6058.253846,7425.705295,9021.815894,10956.99112,3478.125529,4131.546641,4581.609385,5754.733883,6809.40669,7674.929108,9611.147541,8688.156003,9472.384295,9767.29753,10742.44053,11977.57496,786.5668575,912.6626085,1056.353958,1226.04113,1421.741975,1647.511665,2000.603139,2338.008304,1785.402016,1902.2521,2140.739323,3095.772271,2647.585601,3682.259903,4649.593785,5907.850937,7778.414017,9595.929905,11222.58762,11732.51017,7003.339037,6465.613349,6557.194282,9253.896111,1688.20357,1642.002314,1566.353493,1711.04477,1930.194975,2370.619976,2702.620356,2755.046991,2948.047252,2982.101858,3258.495584,3820.17523,468.5260381,495.5868333,556.6863539,566.6691539,724.9178037,502.3197334,462.2114149,389.8761846,410.8968239,472.3460771,633.6179466,823.6856205,331.0,350.0,388.0,349.0,357.0,371.0,424.0,385.0,347.0,415.0,611.0,944.0,2423.780443,2621.448058,3173.215595,3793.694753,3746.080948,3876.485958,4191.100511,3693.731337,3804.537999,3899.52426,4072.324751,4811.060429,545.8657229,597.9363558,652.3968593,676.4422254,674.7881296,694.1124398,718.3730947,775.6324501,897.7403604,1010.892138,1057.206311,1091.359778,8941.571858,11276.19344,12790.84956,15363.25136,18794.74567,21209.0592,21399.46046,23651.32361,26790.94961,30246.13063,33724.75778,36797.93332,10556.57566,12247.39532,13175.678,14463.91893,16046.03728,16233.7177,17632.4104,19007.19129,18363.32494,21050.41377,23189.80135,25185.00911,3112.363948,3457.415947,3634.364406,4643.393534,4688.593267,5486.371089,3470.338156,2955.984375,2170.151724,2253.023004,2474.548819,2749.320965,761.879376,835.5234025,997.7661127,1054.384891,954.2092363,808.8970728,909.7221354,668.3000228,581.182725,580.3052092,601.0745012,619.6768924,1077.281856,1100.592563,1150.927478,1014.514104,1698.388838,1981.951806,1576.97375,1385.029563,1619.848217,1624.941275,1615.286395,2013.977305,10095.42172,11653.97304,13450.40151,16361.87647,18965.05551,23311.34939,26298.63531,31540.9748,33965.66115,41283.16433,44683.97525,49357.19017,1828.230307,2242.746551,2924.638113,4720.942687,10618.03855,11848.34392,12954.79101,18115.22313,18616.70691,19702.05581,19774.83687,22316.19287,684.5971438,747.0835292,803.3427418,942.4082588,1049.938981,1175.921193,1443.429832,1704.686583,1971.829464,2049.350521,2092.712441,2605.94758,2480.380334,2961.800905,3536.540301,4421.009084,5364.249663,5351.912144,7009.601598,7034.779161,6618.74305,7113.692252,7356.031934,9809.185636,1952.308701,2046.154706,2148.027146,2299.376311,2523.337977,3248.373311,4258.503604,3998.875695,4196.411078,4247.400261,3783.674243,4172.838464,3758.523437,4245.256698,4957.037982,5788.09333,5937.827283,6281.290855,6434.501797,6360.943444,4446.380924,5838.347657,5909.020073,7408.905561,1272.880995,1547.944844,1649.552153,1814.12743,1989.37407,2373.204287,2603.273765,2189.634995,2279.324017,2536.534925,2650.921068,3190.481016,4029.329699,4734.253019,5338.752143,6557.152776,8006.506993,9508.141454,8451.531004,9082.351172,7738.881247,10159.58368,12002.23908,15389.92468,3068.319867,3774.571743,4727.954889,6361.517993,9022.247417,10172.48572,11753.84291,13039.30876,16207.26663,17641.03156,19970.90787,20509.64777,3081.959785,3907.156189,5108.34463,6929.277714,9123.041742,9770.524921,10330.98915,12281.34191,14641.58711,16999.4333,18855.60618,19328.70901,2718.885295,2769.451844,3173.72334,4021.175739,5047.658563,4319.804067,5267.219353,5303.377488,6101.255823,6071.941411,6316.1652,7670.122558,3144.613186,3943.370225,4734.997586,6470.866545,8011.414402,9356.39724,9605.314053,9696.273295,6598.409903,7346.547557,7885.360081,10808.47561,493.3238752,540.2893983,597.4730727,510.9637142,590.5806638,670.0806011,881.5706467,847.991217,737.0685949,589.9445051,785.6537648,863.0884639,879.5835855,860.7369026,1071.551119,1384.840593,1532.985254,1737.561657,1890.218117,1516.525457,1428.777814,1339.076036,1353.09239,1598.435089,6459.554823,8157.591248,11626.41975,16903.04886,24837.42865,34167.7626,33693.17525,21198.26136,24841.61777,20586.69019,19014.54118,21654.83194,1450.356983,1567.653006,1654.988723,1612.404632,1597.712056,1561.769116,1518.479984,1441.72072,1367.899369,1392.368347,1519.635262,1712.472136,3581.459448,4981.090891,6289.629157,7991.707066,10522.06749,12980.66956,15181.0927,15870.87851,9325.068238,7914.320304,7236.075251,9786.534714,879.7877358,1004.484437,1116.639877,1206.043465,1353.759762,1348.285159,1465.010784,1294.447788,1068.696278,574.6481576,699.489713,862.5407561,2315.138227,2843.104409,3674.735572,4977.41854,8597.756202,11210.08948,15169.16112,18861.53081,24769.8912,33519.4766,36023.1054,47143.17964,5074.659104,6093.26298,7481.107598,8412.902397,9674.167626,10922.66404,11348.54585,12037.26758,9498.467723,12126.23065,13638.77837,18678.31435,4215.041741,5862.276629,7402.303395,9405.489397,12383.4862,15277.03017,17866.72175,18678.53492,14214.71681,17161.10735,20660.01936,25768.25759,1135.749842,1258.147413,1369.488336,1284.73318,1254.576127,1450.992513,1176.807031,1093.244963,926.9602964,930.5964284,882.0818218,926.1410683,4725.295531,5487.104219,5768.729717,7114.477971,7765.962636,8028.651439,8568.266228,7825.823398,7225.069258,7479.188244,7710.946444,9269.657808,3834.034742,4564.80241,5693.843879,7993.512294,10638.75131,13236.92117,13926.16997,15764.98313,18603.06452,20445.29896,24835.47166,28821.0637,1083.53203,1072.546602,1074.47196,1135.514326,1213.39553,1348.775651,1648.079789,1876.766827,2153.739222,2664.477257,3015.378833,3970.095407,1615.991129,1770.337074,1959.593767,1687.997641,1659.652775,2202.988423,1895.544073,1507.819159,1492.197043,1632.210764,1993.398314,2602.394995,1148.376626,1244.708364,1856.182125,2613.101665,3364.836625,3781.410618,3895.384018,3984.839812,3553.0224,3876.76846,4128.116943,4513.480643,8527.844662,9911.878226,12329.44192,15258.29697,17832.02464,18855.72521,20667.38125,23586.92927,23880.01683,25266.59499,29341.63093,33859.74835,14734.23275,17909.48973,20431.0927,22966.14432,27195.11304,26982.29052,28397.71512,30281.70459,31871.5303,32135.32301,34480.95771,37506.41907,1643.485354,2117.234893,2193.037133,1881.923632,2571.423014,3195.484582,3761.837715,3116.774285,3340.542768,4014.238972,4090.925331,4184.548089,1206.947913,1507.86129,1822.879028,2643.858681,4062.523897,5596.519826,7426.354774,11054.56175,15215.6579,20206.82098,23235.42329,28718.27684,716.6500721,698.5356073,722.0038073,848.2186575,915.9850592,962.4922932,874.2426069,831.8220794,825.682454,789.1862231,899.0742111,1107.482182,757.7974177,793.5774148,1002.199172,1295.46066,1524.358936,1961.224635,2393.219781,2982.653773,4616.896545,5852.625497,5913.187529,7458.396327,859.8086567,925.9083202,1067.53481,1477.59676,1649.660188,1532.776998,1344.577953,1202.201361,1034.298904,982.2869243,886.2205765,882.9699438,3023.271928,4100.3934,4997.523971,5621.368472,6619.551419,7899.554209,9119.528607,7388.597823,7370.990932,8792.573126,11460.60023,18008.50924,1468.475631,1395.232468,1660.30321,1932.360167,2753.285994,3120.876811,3560.233174,3810.419296,4332.720164,4876.798614,5722.895655,7092.923025,1969.10098,2218.754257,2322.869908,2826.356387,3450.69638,4269.122326,4241.356344,5089.043686,5678.348271,6601.429915,6508.085718,8458.276384,734.753484,774.3710692,767.2717398,908.9185217,950.735869,843.7331372,682.2662268,617.7244065,644.1707969,816.559081,927.7210018,1056.380121,9979.508487,11283.17795,12477.17707,14142.85089,15895.11641,17428.74846,18232.42452,21664.78767,22705.09254,26074.53136,29478.99919,33203.26128,13990.48208,14847.12712,16173.14586,19530.36557,21806.03594,24072.63213,25009.55914,29884.35041,32003.93224,35767.43303,39097.09955,42951.65309,5716.766744,6150.772969,5603.357717,5444.61962,5703.408898,6504.339663,6920.223051,7452.398969,8137.004775,9230.240708,7727.002004,10611.46299,7689.799761,9802.466526,8422.974165,9541.474188,10505.25966,13143.95095,11152.41011,9883.584648,10733.92631,10165.49518,8605.047831,11415.80569,605.0664917,676.2854478,772.0491602,637.1232887,699.5016441,713.5371196,707.2357863,820.7994449,989.0231487,1385.896769,1764.456677,2441.576404,1515.592329,1827.067742,2198.956312,2649.715007,3133.409277,3682.831494,4336.032082,5107.197384,6017.654756,7110.667619,4515.487575,3025.349798,781.7175761,804.8304547,825.6232006,862.4421463,1265.047031,1829.765177,1977.55701,1971.741538,1879.496673,2117.484526,2234.820827,2280.769906,1147.388831,1311.956766,1452.725766,1777.077318,1773.498265,1588.688299,1408.678565,1213.315116,1210.884633,1071.353818,1071.613938,1271.211593,406.8841148,518.7642681,527.2721818,569.7950712,799.3621758,685.5876821,788.8550411,706.1573059,693.4207856,792.4499603,672.0386227,469.7092981]\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"gdp_per_cap\",\n",
              "\"y\":\"life_exp\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "}]\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"point\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"4\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10ce97410>"
            ]
          },
          "execution_count": 36,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "(\n",
        "  ggplot(gapminder, aes(\"gdp_per_cap\",\"life_exp\")) +\n",
        "  geom_point()\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot5.png)\n",
        "\n",
        "👥 **WORK IN PAIRS/GROUPS:**\n",
        "\n",
        "-   Create a new feature that takes the natural log of GDP per capita.\n",
        "-   Specify `alpha=0.5` in `geom_point`.\n",
        "-   Edit the x and y axes using `labs`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 37,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"fAJmt6\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"fAJmt6\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "\"life_exp\":[28.801,30.332,31.997,34.02,36.088,38.438,39.854,40.822,41.674,41.763,42.129,43.828,55.23,59.28,64.82,66.22,67.69,68.93,70.42,72.0,71.581,72.95,75.651,76.423,43.077,45.685,48.303,51.407,54.518,58.014,61.368,65.799,67.744,69.152,70.994,72.301,30.015,31.999,34.0,35.985,37.928,39.483,39.942,39.906,40.647,40.963,41.003,42.731,62.485,64.399,65.142,65.634,67.065,68.481,69.942,70.774,71.868,73.275,74.34,75.32,69.12,70.33,70.93,71.1,71.93,73.49,74.74,76.32,77.56,78.83,80.37,81.235,66.8,67.48,69.54,70.14,70.63,72.17,73.18,74.94,76.04,77.51,78.98,79.829,50.939,53.832,56.923,59.923,63.3,65.593,69.052,70.75,72.601,73.925,74.795,75.635,37.484,39.348,41.216,43.453,45.252,46.923,50.009,52.819,56.018,59.412,62.013,64.062,68.0,69.24,70.25,70.94,71.44,72.8,73.93,75.35,76.46,77.53,78.32,79.441,38.223,40.358,42.618,44.885,47.014,49.19,50.904,52.337,53.919,54.777,54.406,56.728,40.414,41.89,43.428,45.032,46.714,50.023,53.859,57.251,59.957,62.05,63.883,65.554,53.82,58.45,61.93,64.79,67.45,69.86,70.69,71.14,72.178,73.244,74.09,74.852,47.622,49.618,51.52,53.298,56.024,59.319,61.484,63.622,62.745,52.556,46.634,50.728,50.917,53.285,55.665,57.632,59.504,61.489,63.336,65.205,67.057,69.388,71.006,72.39,59.6,66.61,69.51,70.42,70.9,70.81,71.08,71.34,71.19,70.32,72.14,73.005,31.975,34.906,37.814,40.697,43.591,46.137,48.122,49.557,50.26,50.324,50.65,52.295,39.031,40.533,42.045,43.548,44.057,45.91,47.471,48.211,44.736,45.326,47.36,49.58,39.417,41.366,43.415,45.415,40.317,31.22,50.957,53.914,55.803,56.534,56.752,59.723,38.523,40.428,42.643,44.799,47.049,49.355,52.961,54.985,54.314,52.199,49.856,50.43,68.75,69.96,71.3,72.13,72.88,74.21,75.76,76.86,77.95,78.61,79.77,80.653,35.463,37.464,39.475,41.478,43.457,46.775,48.295,50.485,49.396,46.066,43.308,44.741,38.092,39.881,41.716,43.601,45.569,47.383,49.517,51.051,51.724,51.573,50.525,50.651,54.745,56.074,57.924,60.523,63.441,67.052,70.565,72.492,74.126,75.816,77.86,78.553,44.0,50.54896,44.50136,58.38112,63.11888,63.96736,65.525,67.274,68.69,70.426,72.028,72.961,50.643,55.118,57.863,59.963,61.623,63.837,66.653,67.768,68.421,70.313,71.682,72.889,40.715,42.46,44.467,46.472,48.944,50.939,52.933,54.926,57.939,60.66,62.974,65.152,39.143,40.652,42.122,44.056,45.989,47.804,47.784,47.412,45.548,42.587,44.966,46.462,42.111,45.053,48.435,52.04,54.907,55.625,56.695,57.47,56.433,52.962,52.97,55.322,57.206,60.026,62.842,65.424,67.849,70.75,73.45,74.752,75.713,77.26,78.123,78.782,40.477,42.469,44.93,47.35,49.801,52.374,53.983,54.655,52.044,47.991,46.832,48.328,61.21,64.77,67.13,68.5,69.61,70.64,70.46,71.52,72.527,73.68,74.876,75.748,59.421,62.325,65.246,68.29,70.723,72.649,73.717,74.174,74.414,76.151,77.158,78.273,66.87,69.03,69.9,70.38,70.29,70.71,70.96,71.58,72.4,74.01,75.51,76.486,70.78,71.81,72.35,72.96,73.47,74.69,74.63,74.8,75.33,76.11,77.18,78.332,34.812,37.328,39.693,42.074,44.366,46.519,48.812,50.04,51.604,53.157,53.373,54.791,45.928,49.828,53.459,56.751,59.631,61.788,63.727,66.046,68.457,69.957,70.847,72.235,48.357,51.356,54.64,56.678,58.796,61.31,64.342,67.231,69.613,72.312,74.173,74.994,41.893,44.444,46.992,49.293,51.137,53.319,56.006,59.797,63.674,67.217,69.806,71.338,45.262,48.57,52.307,55.855,58.207,56.696,56.604,63.154,66.798,69.535,70.734,71.878,34.482,35.983,37.485,38.987,40.516,42.024,43.662,45.664,47.545,48.245,49.348,51.579,35.928,38.047,40.158,42.189,44.142,44.535,43.89,46.453,49.991,53.378,55.24,58.04,34.078,36.667,40.059,42.115,43.515,44.51,44.916,46.684,48.091,49.402,50.725,52.947,66.55,67.49,68.75,69.83,70.87,72.52,74.55,74.83,75.7,77.13,78.37,79.313,67.41,68.93,70.51,71.55,72.38,73.83,74.89,76.34,77.46,78.64,79.59,80.657,37.003,38.999,40.489,44.598,48.69,52.79,56.564,60.19,61.366,60.461,56.761,56.735,30.0,32.065,33.896,35.857,38.308,41.842,45.58,49.265,52.644,55.861,58.041,59.448,67.5,69.1,70.3,70.8,71.0,72.5,73.8,74.847,76.07,77.34,78.67,79.406,43.149,44.779,46.452,48.072,49.875,51.756,53.744,55.729,57.501,58.556,58.453,60.022,65.86,67.86,69.51,71.0,72.34,73.68,75.24,76.67,77.03,77.869,78.256,79.483,42.023,44.142,46.954,50.016,53.738,56.029,58.137,60.782,63.373,66.322,68.978,70.259,33.609,34.558,35.753,37.197,38.842,40.762,42.891,45.552,48.576,51.455,53.676,56.007,32.5,33.489,34.488,35.492,36.486,37.465,39.327,41.245,43.266,44.873,45.504,46.388,37.579,40.696,43.59,46.243,48.042,49.923,51.461,53.636,55.089,56.671,58.137,60.916,41.912,44.665,48.041,50.924,53.884,57.402,60.909,64.492,66.399,67.659,68.565,70.198,60.96,64.75,67.65,70.0,72.0,73.6,75.45,76.2,77.601,80.0,81.495,82.208,64.03,66.41,67.96,69.5,69.76,69.95,69.39,69.58,69.17,71.04,72.59,73.338,72.49,73.47,73.68,73.73,74.46,76.11,76.99,77.23,78.77,78.95,80.5,81.757,37.373,40.249,43.605,47.193,50.651,54.208,56.596,58.553,60.223,61.765,62.879,64.698,37.468,39.918,42.518,45.964,49.203,52.702,56.159,60.137,62.681,66.041,68.588,70.65,44.869,47.181,49.325,52.469,55.234,57.702,59.62,63.04,65.742,68.042,69.451,70.964,45.32,48.437,51.457,54.459,56.95,60.413,62.038,65.044,59.461,58.811,57.046,59.545,66.91,68.9,70.29,71.08,71.28,72.03,73.1,74.36,75.467,76.122,77.783,78.885,65.39,67.84,69.39,70.75,71.63,73.06,74.45,75.6,76.93,78.269,79.696,80.745,65.94,67.81,69.24,71.06,72.19,73.48,74.98,76.42,77.44,78.82,80.24,80.546,58.53,62.61,65.61,67.51,69.0,70.11,71.21,71.77,71.766,72.262,72.047,72.567,63.03,65.5,68.73,71.43,73.42,75.38,77.11,78.67,79.36,80.69,82.0,82.603,43.158,45.669,48.126,51.629,56.528,61.134,63.739,65.869,68.015,69.772,71.263,72.535,42.27,44.686,47.949,50.654,53.559,56.155,58.766,59.339,59.285,54.407,50.992,54.11,50.056,54.081,56.656,59.942,63.983,67.159,69.1,70.647,69.978,67.727,66.662,67.297,47.453,52.681,55.292,57.716,62.612,64.766,67.123,69.81,72.244,74.647,77.045,78.623,55.565,58.033,60.47,64.624,67.712,69.343,71.309,74.174,75.19,76.156,76.904,77.588,55.928,59.489,62.094,63.87,65.421,66.099,66.983,67.926,69.292,70.265,71.028,71.993,42.138,45.047,47.747,48.492,49.767,52.208,55.078,57.18,59.685,55.558,44.593,42.592,38.48,39.486,40.502,41.536,42.614,43.764,44.852,46.027,40.802,42.221,43.753,45.678,42.723,45.289,47.808,50.227,52.773,57.442,62.155,66.234,68.755,71.555,72.737,73.952,36.681,38.865,40.848,42.881,44.851,46.881,48.969,49.35,52.214,54.978,57.286,59.443,36.256,37.207,38.41,39.487,41.766,43.767,45.642,47.457,49.42,47.495,45.009,48.303,48.463,52.102,55.737,59.371,63.01,65.256,68.0,69.5,70.693,71.938,73.044,74.241,33.685,35.307,36.936,38.487,39.977,41.714,43.916,46.364,48.388,49.903,51.818,54.467,40.543,42.338,44.248,46.289,48.437,50.852,53.599,56.145,58.333,60.43,62.247,64.164,50.986,58.089,60.246,61.557,62.944,64.93,66.711,68.74,69.745,70.736,71.954,72.801,50.789,55.19,58.299,60.11,62.361,65.032,67.405,69.498,71.455,73.67,74.902,76.195,42.244,45.248,48.251,51.253,53.754,55.491,57.489,60.222,61.271,63.625,65.033,66.803,59.164,61.448,63.728,67.178,70.636,73.066,74.101,74.865,75.435,75.445,73.981,74.543,42.873,45.423,47.924,50.335,52.862,55.73,59.65,62.677,65.393,67.66,69.615,71.164,31.286,33.779,36.161,38.113,40.328,42.495,42.795,42.861,44.284,46.344,44.026,42.082,36.319,41.905,45.108,49.379,53.07,56.059,58.056,58.339,59.32,60.328,59.908,62.069,41.725,45.226,48.386,51.159,53.867,56.437,58.968,60.835,61.999,58.909,51.479,52.906,36.157,37.686,39.393,41.472,43.971,46.748,49.594,52.537,55.727,59.426,61.34,63.785,72.13,72.99,73.23,73.82,73.75,75.24,76.05,76.83,77.42,78.03,78.53,79.762,69.39,70.26,71.24,71.52,71.89,72.22,73.84,74.32,76.33,77.55,79.11,80.204,42.314,45.432,48.632,51.884,55.151,57.47,59.298,62.008,65.843,68.426,70.836,72.899,37.444,38.598,39.487,40.118,40.546,41.291,42.598,44.555,47.391,51.313,54.496,56.867,36.324,37.802,39.36,41.04,42.821,44.514,45.826,46.886,47.472,47.464,46.608,46.859,72.67,73.44,73.47,74.08,74.34,75.37,75.97,75.89,77.32,78.32,79.05,80.196,37.578,40.08,43.165,46.988,52.143,57.367,62.728,67.734,71.197,72.499,74.193,75.64,43.436,45.557,47.67,49.8,51.929,54.043,56.158,58.245,60.838,61.818,63.61,65.483,55.191,59.201,61.817,64.071,66.216,68.681,70.472,71.523,72.462,73.738,74.712,75.537,62.649,63.196,64.361,64.951,65.815,66.353,66.874,67.378,68.225,69.4,70.755,71.752,43.902,46.263,49.096,51.445,55.448,58.447,61.406,64.134,66.458,68.386,69.906,71.421,47.752,51.334,54.757,56.393,58.065,60.06,62.082,64.151,66.458,68.564,70.303,71.688,61.31,65.77,67.64,69.61,70.85,70.67,71.32,70.98,70.99,72.75,74.67,75.563,59.82,61.51,64.39,66.6,69.26,70.41,72.77,74.06,74.86,75.97,77.29,78.098,64.28,68.54,69.62,71.1,72.16,73.44,73.75,74.63,73.911,74.917,77.778,78.746,52.724,55.09,57.666,60.542,64.274,67.064,69.885,71.913,73.615,74.772,75.744,76.442,61.05,64.1,66.8,66.8,69.21,69.46,69.66,69.53,69.36,69.72,71.322,72.476,40.0,41.5,43.0,44.1,44.6,45.0,46.218,44.02,23.599,36.087,43.413,46.242,46.471,48.945,51.893,54.425,56.48,58.55,60.351,61.728,62.742,63.306,64.337,65.528,39.875,42.868,45.914,49.901,53.886,58.69,63.012,66.295,68.768,70.533,71.626,72.777,37.278,39.329,41.454,43.563,45.815,48.879,52.379,55.769,58.196,60.187,61.6,63.062,57.996,61.685,64.531,66.914,68.7,70.3,70.162,71.218,71.659,72.232,73.213,74.002,30.331,31.57,32.767,34.113,35.4,36.788,38.445,40.006,38.333,39.897,41.012,42.568,60.396,63.179,65.798,67.946,69.521,70.795,71.76,73.56,75.788,77.158,78.77,79.972,64.36,67.45,70.33,70.98,70.35,70.45,70.8,71.08,71.38,72.71,73.8,74.663,65.57,67.85,69.15,69.18,69.82,70.97,71.063,72.25,73.64,75.13,76.66,77.926,32.978,34.977,36.981,38.977,40.973,41.974,42.955,44.501,39.658,43.795,45.936,48.159,45.009,47.985,49.951,51.927,53.696,55.527,58.161,60.834,61.888,60.236,53.365,49.339,64.94,66.66,69.69,71.44,73.06,74.39,76.3,76.9,77.57,78.77,79.78,80.941,57.593,61.456,62.192,64.266,65.042,65.949,68.757,69.011,70.379,70.457,70.815,72.396,38.635,39.624,40.87,42.858,45.083,47.8,50.338,51.744,53.556,55.373,56.369,58.556,41.407,43.424,44.992,46.633,49.552,52.537,55.561,57.678,58.474,54.289,43.869,39.613,71.86,72.49,73.37,74.16,74.72,75.44,76.42,77.19,78.16,79.39,80.04,80.884,69.62,70.56,71.32,72.77,73.78,75.39,76.21,77.41,78.03,79.37,80.62,81.701,45.883,48.284,50.305,53.655,57.296,61.195,64.59,66.974,69.249,71.527,73.053,74.143,58.5,62.4,65.2,67.5,69.39,70.59,72.16,73.4,74.26,75.25,76.99,78.4,41.215,42.974,44.246,45.757,47.62,49.919,50.608,51.535,50.44,48.466,49.651,52.517,50.848,53.63,56.061,58.285,60.405,62.494,64.597,66.084,67.298,67.521,68.564,70.616,38.596,41.208,43.922,46.769,49.759,52.887,55.471,56.941,58.061,58.39,57.561,58.42,59.1,61.8,64.9,65.4,65.9,68.3,68.832,69.582,69.862,69.465,68.976,69.819,44.6,47.1,49.579,52.053,55.602,59.837,64.048,66.894,70.001,71.973,73.042,73.923,43.585,48.079,52.098,54.336,57.005,59.507,61.036,63.108,66.146,68.835,70.845,71.777,39.978,42.571,45.344,48.051,51.016,50.35,49.849,51.509,48.825,44.578,47.813,51.542,69.18,70.42,70.76,71.36,72.01,72.76,74.04,75.007,76.42,77.218,78.471,79.425,68.44,69.49,70.21,70.76,71.34,73.38,74.65,75.02,76.09,76.81,77.31,78.242,66.071,67.044,68.253,68.468,68.673,69.481,70.805,71.918,72.752,74.223,75.307,76.384,55.088,57.907,60.77,63.479,65.712,67.456,68.557,70.19,71.15,72.146,72.766,73.747,40.412,42.887,45.363,47.838,50.254,55.764,58.816,62.82,67.662,70.672,73.017,74.249,43.16,45.671,48.127,51.631,56.532,60.765,64.406,67.046,69.718,71.096,72.37,73.422,32.548,33.97,35.18,36.984,39.848,44.175,49.113,52.922,55.599,58.02,60.308,62.698,42.038,44.077,46.023,47.768,50.107,51.386,51.821,50.821,46.1,40.238,39.193,42.384,48.451,50.469,52.358,53.995,55.635,57.674,60.363,62.351,60.377,46.809,39.989,43.487],\n",
              "\"log_gdp_per_cap\":[6.65858253145061,6.710344079542025,6.7488776061731235,6.7288643965296195,6.606624653169548,6.667101005948512,6.885521366080938,6.748051142822249,6.475958611063048,6.454162415088657,6.588560598778614,6.882006956185797,7.3784187754675905,7.571620004773058,7.746252653296928,7.923057307977371,8.105736827926755,8.169903754066713,8.197230520971159,8.226555484710984,7.823020645749118,8.068733293727343,8.43472675857576,8.688964207477106,7.803438399127065,8.011015423945773,7.844168931950976,8.085484237760244,8.338703587145247,8.499114096208949,8.656113077274341,8.644945662517772,8.521825773817342,8.47580750701102,8.573203018037916,8.736066432319678,8.166389626576143,8.250082199944066,8.359199710611406,8.616635979311402,8.60763481263532,8.009245873018866,7.921881607110396,7.7957322573518,7.8739196583218405,7.730675936789145,7.927788650778689,8.475794211096686,8.684623598845723,8.833004337544303,8.87251045734714,8.993794138051967,9.153033085102253,9.218211983392335,9.104746208135857,9.120379710738412,9.138674507295269,9.302671751374756,9.082238864066401,9.455588185282302,9.214292117534265,9.301062733819364,9.410602272438497,9.583704007294104,9.728457119580886,9.81652331134468,9.8769900376892,9.993734436830607,10.061549153161671,10.20351571873272,10.331618985276355,10.446839438460565,8.722103766381814,9.087336007249453,9.282728111306046,9.459900115030328,9.720863485990712,9.890879519268548,9.980313566963968,10.072716527218166,10.205147182695773,10.278353259827803,10.38645700138868,10.494781744976294,9.196959725577335,9.361841784162335,9.453543391325656,9.602698132895169,9.812942214272915,9.86993604096857,9.86324597860175,9.826823766455338,9.85406509493633,9.917982826105911,10.060643395334472,10.302131057660269,6.5283148299451605,6.494717758944778,6.531375395655606,6.580897198575116,6.446090586357295,6.492053805899945,6.517644486142878,6.622708934611989,6.730791540571114,6.880147708105913,7.035612228614056,7.237960628192509,9.029190743445964,9.18142230858665,9.304850846647033,9.484104121639621,9.72149455536055,9.858384243755154,9.951317536356102,10.022406080166313,10.149392904940367,10.224164143270645,10.325019027588773,10.425033658433435,6.968617237337909,6.866517656954901,6.855934544575183,6.942959678882372,6.990069447088882,6.936499430068043,7.152971519052232,7.111394662616664,7.082722929468553,7.117185464034122,7.224664495042041,7.27329026764787,7.892573943822745,7.662790436401128,7.687526178318643,7.858210135216749,7.999789760902462,8.174166916893737,8.057222409064673,7.920697650500901,7.99351860346229,8.109568710840593,8.135423913422155,8.248564991242306,6.880931922638724,7.210810459348687,7.444063649344297,7.683565925258006,7.958636255199423,8.168622832141459,8.325212290685116,8.369647427661596,7.842585662503011,8.46933733048957,8.702672297733029,8.915472382958686,6.7466954500086835,6.82245066453523,6.891274185208894,7.102260062948873,7.724716655134386,8.075538411121467,8.423133502422273,8.73325312907424,8.981444262003048,9.064984176902943,9.305978232632727,9.439056509151994,7.653942795588259,7.81897959381266,8.112703348182645,8.14027599331388,8.51433139377688,8.803892579291649,8.858060879301203,8.962788319389409,8.846537660175745,8.981930581333305,9.003465372596262,9.11226446181318,7.801508599829684,8.009253641263683,8.35569440939104,8.626406778451468,8.794445219267608,8.937512814622393,9.014835290803777,9.01673800430171,8.748721244431566,8.694567323194475,8.948557042982998,9.27620234384137,6.297579267134175,6.4251663294625985,6.582734057356836,6.678123926953756,6.750792621599208,6.6112168199925225,6.693569716684048,6.815709221997922,6.8370675145824675,6.852554319395607,6.944709214357873,7.104171203548545,5.8268742346593845,5.939024880360337,5.872690096943856,6.023393144989665,6.140098977295902,6.320954005602284,6.327228016541635,6.432648762364485,6.44841440445838,6.137975722517728,6.101223279763741,6.063949594245329,5.909357358359132,6.073132862918194,6.208416263717119,6.260407727204409,6.044113982557917,6.263345276806949,6.436912062821926,6.52780523520695,6.525474097921642,6.598897468148486,6.798192630384196,7.446455969467286,7.067036479437452,7.18010650653358,7.243947076998406,7.318839999449235,7.429014202893443,7.486295366299766,7.769793936012759,7.864290893983798,7.491736533597361,7.435047069657183,7.56735159585094,7.621731638185456,9.338483873950643,9.432679604712911,9.507662248112165,9.685119333029016,9.850644155254352,10.002920271169712,10.0388394431731,10.18966280392805,10.178953470441412,10.273495620167752,10.414182119814592,10.500102769993411,6.976638143218513,7.082417853843466,7.084284050781023,7.035318435187155,6.975426333921048,7.011551475931208,6.863545250195883,6.739190285762191,6.6172766667889995,6.607334184245234,6.604879168601673,6.559638660731664,7.072138507543978,7.176633341758144,7.236927807444,7.087415434888468,7.0067894135245234,7.03349321259469,6.68199342866275,6.858970550247933,6.964196385618775,6.912704365027406,7.052878355198191,7.440771103403637,8.27893061877495,8.36999690897985,8.41606688350189,8.53829973549822,8.611416314588539,8.467322849624956,8.536145602086554,8.621024013409052,8.935393654733069,9.222076550817512,9.285335022699966,9.48582122511248,5.992585446165848,6.356085092559288,6.189647187358004,6.4178847120011495,6.517523687576009,6.60832104615557,6.869452380228331,7.229044272760341,7.41202998707831,7.735972602183335,8.045357772187176,8.508982546938816,7.6704822035856415,7.750961457415556,7.82098176444493,7.893098020487883,8.090910914084231,8.246907682768153,8.388808681466552,8.497647227579174,8.60238850013503,8.718886195306647,8.657869491638822,8.854605046315045,7.005780801631342,7.099324401595935,7.2489650457646535,7.536912930614041,7.569193850746487,7.066981383025801,7.144486169308767,7.182337531231853,7.128421660643506,7.067846764369959,6.980830593414075,6.8938063222615895,6.659988967440948,6.808885022949902,6.798291504794806,6.758783282728785,6.80782009566868,6.679294217235631,6.512855883776405,6.5114106703836185,6.126255853488985,5.743606925395059,5.48548498096406,5.626007793925132,7.661819470608614,7.74718940337078,7.8098591138446345,7.8928029875355845,8.075007878372732,8.089230595402649,8.492799576131716,8.343124272616027,8.29810130326825,8.155983517112023,8.155954124840612,8.19769230667367,7.873601394932181,8.003032279087236,8.149294647861597,8.33368561173511,8.540547726545977,8.687252703516034,8.568406085261751,8.6358496798128,8.725899638172447,8.806430850415639,8.95201607120467,9.17420129338591,7.236047530562877,7.313817492120939,7.45522296403461,7.626594802966717,7.774099645163406,7.831115581251145,7.864308553808698,7.676453279468725,7.407362485559231,7.48788235453928,7.4078035287270225,7.342617436444727,8.045343545730137,8.375222082538711,8.608475272586062,8.847977548533427,9.12304787842245,9.333034454896824,9.489623913290565,9.534059050901192,9.041660724519623,9.197822804604517,9.361204710551752,9.590092566363756,8.628115193697838,8.714760334583811,8.55270625317842,8.646512628823025,8.576488978911527,8.761000954207338,8.897944494666142,8.927038660121497,8.629243195809135,8.60006090469353,8.754736042702763,9.099196824890761,8.835812763327882,9.018737143588222,9.223934267868263,9.341319939337353,9.481012614041456,9.60239331239602,9.640643028512477,9.699560881021132,9.567806488255522,9.683371553556718,9.775438829430662,10.035975648428332,9.17909582990832,9.314669697640314,9.516597370613331,9.676411982834672,9.845127622399774,9.924412172711843,9.984516257646696,10.13126737212987,10.18137455390007,10.302409489713996,10.378680819163689,10.471026688821484,7.889657509272804,7.96031283537331,8.013339627259205,8.013028836387974,8.214522644759358,8.033256472964537,7.965360857112705,7.965581186384721,7.773660174919537,7.546983080009345,7.55394756540886,7.641315522621272,7.24259556827282,7.34239270365264,7.415859618811405,7.410784390579362,7.6915995147757785,7.894313925130598,7.958958784035217,7.972411592079819,8.020998089041683,8.19259849649272,8.425912675479731,8.703734955926622,8.16681572439397,8.237623894824319,8.315349694224249,8.42925212021302,8.57186975100221,8.80681686673666,8.883749926611829,8.776749979067699,8.868371419353098,8.913207901786514,8.660954865407097,8.835394138005636,7.257582542481131,7.285448474184002,7.434455739895437,7.503775029939101,7.612835055596883,7.932180365383544,8.1615832904036,8.264996842643468,8.241375180502672,8.33643404470373,8.466868777827852,8.627155681368093,8.022340288509426,8.13784111628214,8.23663332956841,8.379905126870375,8.416321697893878,8.544598681604745,8.318338311362988,8.328557847833041,8.399362287114169,8.547688544471686,8.585145005510944,8.653183423565686,5.928639551979159,6.0546656307590005,6.367916088543337,6.819575222079366,6.510871630249935,6.865439265220727,6.832843506775383,6.874091783348178,7.031789874189153,7.942533067076475,8.949429517796077,9.405420997068234,5.795877056933626,5.841112145223142,5.942788465093433,6.150165508469744,6.242853821865599,6.2260500048381,6.263161757102596,6.2560074621687525,6.367944464271515,6.817251399390018,6.640333247497604,6.463605770695175,5.892048217497306,5.9372833055598635,6.038959625634964,6.246336668896708,6.33902498217052,6.322221165300542,6.359332917673699,6.352178622610927,6.043472066463646,6.245892403941878,6.272978005030555,6.537858417895999,8.767877054199166,8.928695424159669,9.145465000617612,9.298501078733246,9.57212355959389,9.655373750140422,9.827316710050795,9.958970140491724,9.935333300344379,10.074240373816302,10.247240029848188,10.410518517683292,8.85791486167728,9.066797303414635,9.264874534422983,9.472698302577461,9.68702114129836,9.814253806458106,9.91807550210132,10.001813278280753,10.114712201090327,10.161603763325196,10.272497241658359,10.324498420059683,8.364852050884766,8.512421444518866,8.799580152778805,9.03106560743326,9.341539532936492,9.987165488446005,9.6233345282015,9.38129831006267,9.512084916747098,9.597155436148702,9.435219529853573,9.48846323946509,6.184624363655609,6.255609362184756,6.396346611946114,6.5995750983073735,6.628156232413991,6.7853110538787735,6.728400848870891,6.416174710594722,6.500725566418527,6.482694682834976,6.493126713899836,6.623732804027337,8.874044134886145,9.228948820838767,9.465173495382052,9.598701748969178,9.79902553699504,9.928810270470864,10.000230012500982,10.112093367588322,10.185100111575695,10.232391369357545,10.310145348442681,10.37880125895925,6.814870985145645,6.950394696508247,7.081743138449813,7.02615782048292,7.071763251105474,6.900956174466742,6.775403269469005,6.7417079124536325,6.82985876674933,6.912987379640876,7.013901606009321,7.191134791182141,8.169248617224632,8.500311471469766,8.702375773737394,9.04936108203667,9.451310448008796,9.56068200235676,9.633541980291556,9.687848794059395,9.772324572435704,9.838826257995544,10.021903934253213,10.223337104679452,7.79492107530898,7.869843498385153,7.9194887076980285,8.08410952200414,8.301871040796645,8.492899012780514,8.480631855303104,8.353847090259235,8.398285963092004,8.451974717990161,8.488453637469147,8.553727607972682,6.234795930537076,6.356571137475322,6.531422193562933,6.563516316538657,6.608899318111834,6.773864809668725,6.753730008607452,6.691553169767877,6.677522204280044,6.767860559612504,6.851802285777785,6.848699525142248,5.703283413479219,6.067940416400617,6.257733433415924,6.5730942973278506,6.709578190075088,6.639517551088849,6.731166021641448,6.601794350960146,6.614108614464839,6.680433596546526,6.355594886193959,6.3617026444730875,7.5177202540904,7.454076155367556,7.493645163911727,7.28073690947216,7.411228104659828,7.535989965018514,7.606466724765777,7.5082475486390745,7.2836607865217715,7.2017128176913205,7.147059486207595,7.091440200971258,7.693903704898547,7.705482127317536,7.736812137116096,7.839237772666581,7.8359122655507045,8.071908107224624,8.04615247866526,8.014036982028363,8.033234920444638,8.058471253814995,8.039069857610091,8.174232587672845,8.02434539670065,8.196733475329522,8.453752365617301,8.731975938652381,9.025928008368183,9.32243090269319,9.586069757213272,9.905409339220585,10.116887913146167,10.253356514886638,10.315895674047942,10.5897354544633,8.568584506039647,8.706189093630451,8.929350506934906,9.140630601078245,9.227065337734802,9.365191152675624,9.437156424196026,9.471664093728362,9.262517987486524,9.36843555913829,9.605346681023127,9.79862382679109,8.8911935595823,9.131730124141988,9.244757166691368,9.497014112231879,9.667642655808292,9.886085128416335,10.054903388943083,10.200743880678232,10.132390168954027,10.242139542482288,10.34699325337033,10.496283571591434,6.303654610169521,6.380227609345564,6.489732377155714,6.55218060195628,6.584836318172118,6.701145934934976,6.751947353999071,6.883987730877647,7.059967059501227,7.2853814152655705,7.465523334625443,7.804745103733494,6.6196486558881515,6.755652815942696,6.744400435617459,6.636513025264948,7.0131129109358055,7.231794875339298,7.324406250068779,7.466431746465631,7.776174601662223,8.045375310370526,7.96342974954754,8.172066046818411,8.01807411189315,8.098721151635525,8.339818531008813,8.683847963364693,9.170956780686979,9.383334822905258,8.936999583542034,8.801301089760795,8.886775916676685,9.019614433215251,9.131379626062532,9.359252884241174,8.325976039314368,8.737024633667891,9.029026844749668,9.097335133186714,9.167019173295264,9.594802116953339,9.583138139560093,9.36250960546314,8.22834795928978,8.031463284349092,8.387247889532347,8.40538122225756,8.558388939058093,8.630357197450067,8.799600976334768,8.943188630999266,9.162281094723824,9.319282766739018,9.44290511694744,9.537690162467365,9.773311413319613,10.107323796766845,10.436379398263155,10.61339342717752,8.315449551844253,8.591424296888741,8.86864280590882,9.035241636175714,9.456179008883149,9.496016875368166,9.639979532229228,9.748147490841262,9.800985311450198,9.947342187470431,9.99449736901839,10.147346142223887,8.503379044942077,8.740121716734015,9.017190278427117,9.212577997623582,9.414853349319293,9.56493207998499,9.713384810690288,9.863042300581146,9.999417760992046,10.113546855587918,10.238819788532053,10.26010268418614,7.97195929485448,8.467272802793707,8.565241656594964,8.720085618077562,8.91380445876353,8.802401542691134,8.710792802858457,8.756404954002244,8.909900420774385,8.870933291470687,8.852918700877536,8.898485854096965,8.076190957672294,8.37047682695776,8.791280694176857,9.195002202057134,9.600948077463563,9.717782900122176,9.872208715948268,10.015741637899518,10.19708565733282,10.268706368084192,10.261322539864274,10.362685133050313,7.344013254097813,7.542256193457546,7.7613230812810965,7.916368551015673,7.654848977791199,7.955899044624217,8.333610670100578,8.400362682162507,8.1407800523741,8.201215774000511,8.254507345687035,8.416148055764452,6.749393483674624,6.850590375332894,6.799018373284752,6.9629406236562374,7.108538669184907,7.144891044297698,7.206544778187409,7.216663124401262,7.201857985726038,7.215596548239869,7.160469074802525,7.288414777482674,6.9923516871137705,7.3595533475041846,7.391226313008342,7.670214231041686,8.216526246685492,8.32027796288563,8.320332522420989,8.320324491755143,8.223107595299695,7.432931526611057,7.406563877258059,7.373415413898915,6.937888892686943,7.3049150174596,7.337161098862706,7.615410774115982,8.016607180089805,8.446174201594228,8.634610375979493,9.0517066857249,9.401314282301552,9.679939416888713,9.86443421073548,10.058272590972555,11.593420558609495,11.63976190923324,11.466442809932659,11.300905853333136,11.602289519741827,10.989782242438755,10.353098269195346,10.244180511506714,10.461184918333121,10.604122131439276,10.466244278556982,10.764413339037992,8.483595883177966,8.714368373885014,8.65077268980866,8.70067791180639,8.920841228450222,9.066434993553408,8.941220879979552,8.589902861787248,8.837943462231266,9.077376115642323,9.139267355988009,9.255414944734394,5.699929100269407,5.81710257391444,6.020539316204965,6.211882440188406,6.207747804726955,6.61388012331784,6.681184746520149,6.6515631062386325,6.88498424828265,7.078466355921174,7.15084621182101,7.358404974528663,6.355366059562457,6.431282755645009,6.4523567343551935,6.570327693780758,6.688361505330923,6.461971859327899,6.349487828081648,6.226761658402394,6.456177516556191,6.4121038604219525,6.275670023007831,6.027090686119344,7.778022201995214,8.145632109530693,8.818338844145593,9.840161719261975,9.95282505308015,9.996577622446521,9.762170235542749,9.373359309449134,9.173690754825033,9.155614461951426,9.162690691243208,9.397442092229253,7.274487677244035,7.370987754369352,7.404514701971134,7.398815211353809,7.466549576443781,7.34227976755417,7.1723314476925015,7.052238188729315,6.947625963550533,6.893956406539954,6.796418141291025,6.951552165081218,5.9112439158153105,6.031573823064691,6.058892060460917,6.20559718299215,6.370965434762582,6.4971123037266585,6.450160612546764,6.454439412679547,6.33363483058957,6.539984446007515,6.500423106894256,6.632462685785543,7.512690122051632,7.501119135729635,7.619176931568249,7.73094005420622,7.954756601757815,8.250077264118444,8.501136154633608,8.565945764974389,8.892599397061584,9.223543786009959,9.230826877139284,9.429608889033608,6.1144274345830345,6.195185059941061,6.2069273625447705,6.300803936333545,6.365385454012112,6.5314536561045475,6.42651121464211,6.528208701378354,6.605317372721125,6.672359454935857,6.857944833893097,6.9494551827472755,6.610852034838689,6.740661502340346,6.962145008662037,7.259218299513596,7.369507320449572,7.311547136653916,7.300574219659522,7.259540792312798,7.216246665968072,7.30191413563213,7.364559390999823,7.497291244077236,7.58475057075323,7.617778249492179,7.8356059315650075,7.814152253450816,7.853792817948089,8.219052070104222,8.212849817541699,8.472945942194759,8.709176893317414,8.912702948385299,9.107400911420072,9.30173298996425,8.15424878655734,8.326407105197692,8.429805609488586,8.657778079079016,8.826060272000081,8.945714335670068,9.17067890596963,9.069715998158916,9.156135927985744,9.186795097766336,9.28195757967851,9.390791427138863,6.667677725242776,6.816366270676608,6.962578595603427,7.111545664057411,7.259638141718719,7.4070213467764825,7.601203983579141,7.757054693779342,7.487398887903356,7.550793779011864,7.668906526389485,8.037792676125699,7.88140340971963,8.211281946754317,8.444535136700948,8.684037412628841,8.959107742535375,9.16909431932682,9.325683778155126,9.370118914491124,8.85414231959231,8.774253158940985,8.788318089533377,9.132799943015339,7.431420266225739,7.403671699268385,7.356505580962649,7.4448594395346746,7.565376300112482,7.770906793168574,7.901977083581836,7.921189778065872,7.9888982816746665,8.000383652353667,8.08902089054672,8.248051572276294,6.149591677722147,6.20574258221079,6.3220019824047435,6.339775630601859,6.586058274220437,6.219236836068335,6.136022394419783,5.965829213333098,6.0183421462415785,6.1577119311102635,6.451446164909682,6.713788928589531,5.802118375377063,5.857933154483459,5.961005339623274,5.855071922202427,5.877735781779639,5.916202062607435,6.049733455231958,5.953243334287785,5.849324779946859,6.028278520230698,6.415096959171596,6.8501261661455,7.793083766766791,7.871482137969257,8.062500736013023,8.241095692065858,8.228465491953687,8.262684341312713,8.340718630219216,8.214392428552811,8.243949843475754,8.268609840061625,8.311969307260537,8.478672802210884,6.302373016744891,6.393484319858631,6.480653056664149,6.516847041746571,6.514398759527167,6.542633964386349,6.57698906458844,6.653678761198801,6.799880895538102,6.918588524901557,6.963385152254503,6.995179700490467,9.098466675755006,9.330449007106369,9.456485316135497,9.639733661384565,9.841332624116353,9.962183690139309,9.971120988542578,10.071174358950202,10.195819408345681,10.317123542219564,10.42598749884112,10.513196962797394,9.264504230050656,9.413068566425785,9.486127833186055,9.579412477666029,9.683217199631356,9.69484569774033,9.777493987543831,9.852572675487073,9.8181107447283,9.954675495449408,10.051467864020562,10.134004219902934,8.043137828331645,8.148276752903572,8.198189520758671,8.443200742989394,8.452887873378042,8.610022312119094,8.152007319489666,7.991586996194853,7.682552362984302,7.720028150766271,7.8138133630314375,7.919109238352224,6.635788243943405,6.728058357816719,6.905518892833778,6.960712834169096,6.860882972662072,6.695671681275859,6.813139207169115,6.504737208633439,6.365065208267468,6.363554187892443,6.398718888915774,6.4291982009074955,6.982196347653054,7.003604007379495,7.048323398917519,6.922165061584215,7.437435338299622,7.591837398751715,7.363262941295014,7.233476763519989,7.39008773062707,7.39322695564929,7.387267554300202,7.6078668045953535,9.219837305010588,9.363402434331544,9.506764236629031,9.702709302234608,9.850353381089452,10.056695619243195,10.177272327465944,10.359042766824459,10.43310532706233,10.628210052498586,10.70737022080105,10.806838731715855,7.511103732590244,7.715456532594046,7.980926029960471,8.459763780459888,9.270309583764263,9.379943383210337,9.46922096087072,9.804507920669824,9.83181467750998,9.888478265118952,9.892165543297152,10.013067831605971,6.5288305540161,6.616176998426324,6.688781449532241,6.848438576441265,6.95648732812959,7.069807113458015,7.27477738829213,7.44113655053284,7.586717052721257,7.625278202910314,7.646216322180451,7.865551642363522,7.816167187884382,7.993552776160787,8.170904212141355,8.39412324856288,8.587511787501867,8.585209186127738,8.855036145032674,8.858621577584552,8.797660759301952,8.869776692084349,8.903275926932842,9.191074535454053,7.576767900473375,7.6237175575337295,7.672305093351506,7.74039319601381,7.833337898052313,8.085909630329683,8.356673110888163,8.293768524342696,8.341984933735649,8.354062371457553,8.23845083846387,8.336351770041206,8.231781456330422,8.353557567588549,8.508563660286962,8.663558212362215,8.68909856817114,8.74533078849894,8.769429696209459,8.757931985584413,8.39984576876673,8.672203100343284,8.684235288361398,8.910438009810923,7.149038110291206,7.344683423023936,7.408259107628651,7.503359876278364,7.595575330552742,7.771996340804566,7.864525072341735,7.691490139969962,7.731634194216554,7.838554225884947,7.882662431421013,8.067926973126148,8.301355313315165,8.462579235836216,8.58274722356042,8.788311759671918,8.9880098641775,9.159903705735847,9.042102887844724,9.114088377723832,8.954012414405344,9.226172743914965,9.392848501364389,9.64146833273388,8.028885416179008,8.236042209684777,8.46124801779676,8.758022305999726,9.10744874133428,9.227441876090147,9.371935522271308,9.475723824873153,9.693214978056083,9.777982806309033,9.902031887069048,9.928650677345093,8.0333209674497,8.270565070964496,8.538630683553107,8.843510860778018,9.118558551803602,9.187125471433868,9.242903312611743,9.415836471789572,9.591621190792461,9.740935287188602,9.844565558738122,9.869346783112231,7.907977257276984,7.926404689345817,8.062660732826064,8.299329611239415,8.526679763859006,8.370965325348482,8.569257865284124,8.576099158461872,8.716249901595493,8.711433669984118,8.750867530828282,8.945087873112389,8.053446168247508,8.2797910237128,8.462736495799563,8.77506531129337,8.98862260399967,9.14381558511826,9.170071771492697,9.17949689427738,8.79458397519984,8.901985761717766,8.97276316493657,9.288085884038223,6.201165906014376,6.292104918780773,6.392709216179257,6.236298578289071,6.38110622884836,6.507398005298783,6.781705142358862,6.742870278426945,6.60268096099263,6.380028473323383,6.666516192580365,6.760517193245892,6.779448597183153,6.7577888858379795,6.976862521661787,7.233340316690606,7.334972259777932,7.460238062852498,7.544447507215055,7.324177113667596,7.264574682286113,7.199735129739869,7.210147911131397,7.376780360230913,8.77331568155375,9.006704214184872,9.361035352030672,9.735249290557134,10.120107014183576,10.439037864475624,10.425050580994759,9.961674445976238,10.12027566154345,9.932400038678859,9.852959290699694,9.982983892824338,7.2795650002859755,7.357334879226639,7.411549473890668,7.385481903975178,7.376327919852211,7.3535745064140245,7.325465102969886,7.2735926236366195,7.221031534804262,7.238761422987137,7.326225625834978,7.445693299055102,8.183525663360221,8.513404200447454,8.746657390404753,8.986159666253233,9.261229996449995,9.47121657290302,9.627806031231179,9.672241168636095,9.14046156223323,8.976429094212028,8.886834245991238,9.18876271106754,6.779680669058424,6.912229690854728,7.018079345116951,7.095100417267865,7.210641009369533,7.206588811385204,7.289617882516225,7.165839464658583,6.974194752788752,6.35375795368416,6.550351087778671,6.759882401238381,7.7472246740250625,7.952651835915101,8.209236455860903,8.512666670159028,9.059256541455326,9.324569498193135,9.627019772197864,9.844879719913733,10.117384130063584,10.419881939915639,10.491915828228683,10.760944625212437,8.532014630028257,8.714939010321517,8.920136134605492,9.03752180603338,9.177214480708518,9.298595179252525,9.336844895758885,9.395762747924133,9.158885772263838,9.403126207562439,9.520672365069794,9.835118469488423,8.346414773352521,8.676293310365535,8.909546500352526,9.149048776213318,9.424119105950727,9.634105683227947,9.790695141408419,9.835130278300156,9.562033101973185,9.750400901845707,9.935955679749666,10.156898687468367,7.035048365524381,7.137395610840445,7.222192471689864,7.158306333745384,7.1345530471060234,7.280003092981144,7.07056014394647,6.99690558291905,6.831910734443653,6.835825701449175,6.782284820184787,6.831026564620053,8.460685384215294,8.610155930682131,8.66020718223198,8.869887137628433,8.957505699051607,8.990771852488555,9.055820684007127,8.965184236453895,8.885312098817042,8.919879541550007,8.950396214406169,9.134501743962709,8.251672985119047,8.426130508412129,8.647140848951375,8.986385528403389,9.27225839792593,9.490765262419163,9.54152508065564,9.665546501886613,9.83108160526978,9.925508255337956,10.120028219106631,10.268861777269361,6.987981382145385,6.977791101606424,6.979584619883523,7.034840977934542,7.1011779316378885,7.206952534741823,7.407366124952757,7.537305802444621,7.674960783534783,7.8877631661402665,8.011480750687628,8.286545405382396,7.387703749585776,7.478925244725132,7.580492469008187,7.431298277643077,7.414363687784394,7.697570091681862,7.547261186070939,7.3184196202892595,7.308004838397601,7.397690671793448,7.597596156741887,7.864187451932577,7.04610459450442,7.1266565356758464,7.526277036225904,7.868293172149886,8.121134689637175,8.237852398364613,8.267547545999479,8.290252392674834,8.175553900532535,8.262757214453435,8.325576635975251,8.414823896039525,9.051091931183445,9.201489137721001,9.419745333171344,9.63287869769317,9.788751256850304,9.844571871429592,9.936311951374265,10.068447993093663,10.08079727228016,10.137238446150942,10.286762637404955,10.429982223586265,9.597928823914517,9.793086003861617,9.924813171452367,10.041776424388496,10.210792568494865,10.202936023118916,10.254063967391613,10.318299000220556,10.36946842320244,10.37771110941194,10.448162500203953,10.532267372509699,7.404574481595142,7.657866220577292,7.693042680787075,7.540049741213616,7.85221472659843,8.069494024168668,8.232662870971975,8.044553862965001,8.113888578157397,8.297603062212968,8.316526465746207,8.339153993580391,7.09585006606529,7.3184475615786075,7.508172413727509,7.879994750602133,8.309559708959153,8.629900223894298,8.912790408171672,9.310598449810142,9.630080301619703,9.913775498668072,10.053433259128045,10.26528902275493,6.57458767697621,6.5489861542051875,6.5820304121476205,6.743138453379564,6.820000053624075,6.869526059064947,6.7733579194021525,6.723618571077705,6.716210261348988,6.671002317204401,6.8013655795630115,7.009844413302285,6.6304160909513525,6.676551096392306,6.909952036342884,7.166621632887287,7.3293292308252775,7.581324370885215,7.780394926983974,8.000568711067123,8.437478014848532,8.674645642449985,8.684940310014362,8.917095700517491,6.756709872283188,6.830775223481237,6.973107353470419,7.298172236137634,7.408324599018033,7.334836400579989,7.203835453202606,7.091909622695587,6.9414790887293245,6.8898834492820376,6.786965877295205,6.7832911612930324,8.014094943760721,8.318838199308887,8.516697862961351,8.634330413602209,8.79778288515478,8.974561607622418,9.118173393903568,8.907693256155886,8.905307430961969,9.081662681174514,9.3466703649879,9.798599660729607,7.291980156016817,7.240816324099541,7.414755521297405,7.566497419240721,7.920550384236635,8.045869270488895,8.177581320015761,8.245494513509101,8.373950837148023,8.492244261849477,8.652230189672556,8.866852808894505,7.5853323622403,7.704701171869773,7.750558729480343,7.946743665342809,8.146331338931944,8.359163540833258,8.352638389524152,8.534845210949708,8.644415672066291,8.795041558336353,8.780800639426129,9.042900694727205,6.599535047036679,6.6520511762769,6.642841027800283,6.812255455066505,6.857236283695021,6.737836256446285,6.52541994354892,6.426042413820259,6.467963903519355,6.705099268641489,6.83273104297148,6.962603362563226,9.208289118293099,9.331068218528745,9.43165642002307,9.556964537940104,9.673767197004778,9.765876332149476,9.810956854989678,9.983443533525937,10.030344519277985,10.168714306936607,10.291433396948166,10.4104033814055,9.546132525965046,9.6055616655824,9.691107482814038,9.879725742077515,9.989942088435198,10.088830877935184,10.127013396367241,10.305090224699548,10.373614056732428,10.484793066468567,10.573803562915057,10.66783041537953,8.65115867001181,8.724333038918997,8.631121289361014,8.602383174340977,8.648819327341347,8.780224873572523,8.842203280895717,8.916291268801494,9.004177427522096,9.13024040603009,8.95247622728388,9.269690109944033,8.947650023279008,9.190389319317504,9.0387182711137,9.163403279548284,9.259631330727073,9.483716927969423,9.31941090689372,9.198630543562635,9.281164687611662,9.226754439091394,9.060104267142684,9.342754138163883,6.405338355628243,6.516615246975446,6.649048227012898,6.456963182684458,6.550368144495912,6.570234460230789,6.561364112842929,6.710278798124853,6.896717737516639,7.234102695735002,7.47559809024944,7.800399176888787,7.3235616177698,7.510467633961013,7.695738123145036,7.882207368668639,8.049876916636691,8.211437163022612,8.37471494207735,8.538406075597258,8.702452886983368,8.869351416982619,8.415268450053805,8.014781999482128,6.661493519430466,6.690631639956548,6.716138495834397,6.759768069902146,7.142864579126638,7.511942919022698,7.5896175285562135,7.586672460649771,7.53875929285103,7.657984118825365,7.711916336943752,7.732268343018254,7.045244057938765,7.179275016217178,7.281196909349893,7.482725337618906,7.480709295921684,7.370663985557544,7.250407355993754,7.101111657564247,7.099106473119055,6.976678378168391,6.976921144304792,7.147725734910312,6.0085284146781195,6.251449575937743,6.267716889218364,6.345276771973746,6.683814129420996,6.530276400523569,6.670582579124827,6.559838025563655,6.54163700915007,6.675129362180636,6.51031581312752,6.152113988664138]\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"log_gdp_per_cap\",\n",
              "\"y\":\"life_exp\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"log_gdp_per_cap\"\n",
              "}]\n",
              "},\n",
              "\"guides\":{\n",
              "\"x\":{\n",
              "\"title\":\"Logged GDP per capita (USD)\"\n",
              "},\n",
              "\"y\":{\n",
              "\"title\":\"Average life expectancy (years)\"\n",
              "}\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"point\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"alpha\":0.5,\n",
              "\"data\":{\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"5\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cf34c30>"
            ]
          },
          "execution_count": 37,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder[\"log_gdp_per_cap\"] = np.log(gapminder[\"gdp_per_cap\"])\n",
        "\n",
        "(\n",
        "  ggplot(gapminder, aes(\"log_gdp_per_cap\",\"life_exp\")) +\n",
        "  geom_point(alpha=0.5) +\n",
        "  labs(x=\"Logged GDP per capita (USD)\", y = \"Average life expectancy (years)\")\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot6.png)\n",
        "\n",
        "### 2.4 Line plots\n",
        "\n",
        "Line plots can be used to track parameters of interest over time. We will do this by calculating then plotting average GDP per capita for each year."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 38,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"VGAd4A\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"VGAd4A\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "\"year\":[1952.0,1957.0,1962.0,1967.0,1972.0,1977.0,1982.0,1987.0,1992.0,1997.0,2002.0,2007.0],\n",
              "\"gdp_per_cap\":[3725.276045801408,4299.408344876057,4725.812341961972,5483.653046835211,6770.082815282394,7313.166420794366,7518.901672698592,7900.920217940845,8158.608520688732,9090.175363102113,9917.848364685917,11680.071819878169]\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"year\",\n",
              "\"y\":\"gdp_per_cap\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "}]\n",
              "},\n",
              "\"guides\":{\n",
              "\"x\":{\n",
              "\"title\":\"Year\"\n",
              "}\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"line\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"6\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cf2c560>"
            ]
          },
          "execution_count": 38,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gdp_per_cap_by_year = gapminder.groupby(\"year\", as_index=False)[\"gdp_per_cap\"].mean()\n",
        "\n",
        "(\n",
        "  ggplot(gdp_per_cap_by_year, aes(\"year\", \"gdp_per_cap\")) +\n",
        "  geom_line() +\n",
        "  labs(x = \"Year\")\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot7.png)\n",
        "\n",
        "👥 **WORK IN PAIRS/GROUPS:**\n",
        "\n",
        "-   Change `linetype=\"dotted\"` in `geom_line`.\n",
        "-   Add a `geom_point` and `geom_area` layer.\n",
        "-   Change the `breaks` parameter in `scale_x_continuous` to an array from 1952 to 2007 by increments of 10.\n",
        "-   Specify `panel_grid_major_x=element_blank()` in the `theme` function.\n",
        "-   Edit the x and y axes using `labs`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 40,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"S5kS7n\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"S5kS7n\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "\"year\":[1952.0,1957.0,1962.0,1967.0,1972.0,1977.0,1982.0,1987.0,1992.0,1997.0,2002.0,2007.0],\n",
              "\"gdp_per_cap\":[3725.276045801408,4299.408344876057,4725.812341961972,5483.653046835211,6770.082815282394,7313.166420794366,7518.901672698592,7900.920217940845,8158.608520688732,9090.175363102113,9917.848364685917,11680.071819878169]\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"year\",\n",
              "\"y\":\"gdp_per_cap\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "}]\n",
              "},\n",
              "\"theme\":{\n",
              "\"panel_grid_major_x\":{\n",
              "\"blank\":true\n",
              "}\n",
              "},\n",
              "\"guides\":{\n",
              "\"x\":{\n",
              "\"title\":\"Year\"\n",
              "},\n",
              "\"y\":{\n",
              "\"title\":\"Average GDP per capita (USD)\"\n",
              "}\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[{\n",
              "\"aesthetic\":\"x\",\n",
              "\"breaks\":[1952.0,1962.0,1972.0,1982.0,1992.0,2002.0]\n",
              "}],\n",
              "\"layers\":[{\n",
              "\"geom\":\"line\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"linetype\":\"dotted\",\n",
              "\"data\":{\n",
              "}\n",
              "},{\n",
              "\"geom\":\"point\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "}\n",
              "},{\n",
              "\"geom\":\"area\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"8\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cf7b1d0>"
            ]
          },
          "execution_count": 40,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "(\n",
        "  ggplot(gdp_per_cap_by_year, aes(\"year\", \"gdp_per_cap\")) +\n",
        "  geom_line(linetype=\"dotted\") +\n",
        "  geom_point() +\n",
        "  geom_area() +\n",
        "  scale_x_continuous(breaks=np.arange(1952,2007,10)) +\n",
        "  theme(panel_grid_major_x=element_blank()) +\n",
        "  labs(x = \"Year\", y = \"Average GDP per capita (USD)\")\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot8.png)\n",
        "\n",
        "👉 **NOTE:** We think this line plot works in this context, and that it is useful to introduce you to this style of charting progress over time. However, for others contexts filling the area under the line may not work, so do bear this in mind when making decisions!\n",
        "\n",
        "### 2.5 Box plots\n",
        "\n",
        "Box plots are an excellent way to summarise the distribution of values across different strata over key quantities of interest (e.g. median, interquartile range). Suppose we want to show the distribution of populations by continent in `gapminder` in 1982 (excluding Oceania). We can use box plots to graphically illustrate this."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 41,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"tt0nGz\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"tt0nGz\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"continent\",\n",
              "\"y\":\"pop\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"log_gdp_per_cap\"\n",
              "}]\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"boxplot\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "\"..middle..\":[1.4441916E7,8962461.0,5668228.5,5968349.0],\n",
              "\"..lower..\":[3858421.0,4620150.5,1516971.5,3279001.0],\n",
              "\"..upper..\":[4.882716E7,1.83335635E7,1.03255845E7,1.8125129E7],\n",
              "\"..ymin..\":[377967.0,233997.0,98593.0,1116479.0],\n",
              "\"..ymax..\":[9.3074406E7,3.798331E7,2.0367053E7,2.9341374E7],\n",
              "\"continent\":[\"Asia\",\"Europe\",\"Africa\",\"Americas\"]\n",
              "}\n",
              "},{\n",
              "\"geom\":\"point\",\n",
              "\"stat\":\"boxplot_outlier\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"show_legend\":false,\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "\"..middle..\":[1.4441916E7,1.4441916E7,1.4441916E7,1.4441916E7,8962461.0,8962461.0,8962461.0,8962461.0,8962461.0,5668228.5,5668228.5,5668228.5,5668228.5,5668228.5,5968349.0,5968349.0,5968349.0],\n",
              "\"..lower..\":[3858421.0,3858421.0,3858421.0,3858421.0,4620150.5,4620150.5,4620150.5,4620150.5,4620150.5,1516971.5,1516971.5,1516971.5,1516971.5,1516971.5,3279001.0,3279001.0,3279001.0],\n",
              "\"..upper..\":[4.882716E7,4.882716E7,4.882716E7,4.882716E7,1.83335635E7,1.83335635E7,1.83335635E7,1.83335635E7,1.83335635E7,1.03255845E7,1.03255845E7,1.03255845E7,1.03255845E7,1.03255845E7,1.8125129E7,1.8125129E7,1.8125129E7],\n",
              "\"..ymin..\":[377967.0,377967.0,377967.0,377967.0,233997.0,233997.0,233997.0,233997.0,233997.0,98593.0,98593.0,98593.0,98593.0,98593.0,1116479.0,1116479.0,1116479.0],\n",
              "\"..ymax..\":[9.3074406E7,9.3074406E7,9.3074406E7,9.3074406E7,3.798331E7,3.798331E7,3.798331E7,3.798331E7,3.798331E7,2.0367053E7,2.0367053E7,2.0367053E7,2.0367053E7,2.0367053E7,2.9341374E7,2.9341374E7,2.9341374E7],\n",
              "\"continent\":[\"Asia\",\"Asia\",\"Asia\",\"Asia\",\"Europe\",\"Europe\",\"Europe\",\"Europe\",\"Europe\",\"Africa\",\"Africa\",\"Africa\",\"Africa\",\"Africa\",\"Americas\",\"Americas\",\"Americas\"],\n",
              "\"pop\":[1.000281E9,7.08E8,1.53343E8,1.18454974E8,5.4433565E7,7.8335266E7,5.6535636E7,4.7328791E7,5.6339704E7,3.0646495E7,4.5681811E7,3.8111756E7,7.3039376E7,3.1140029E7,1.28962939E8,7.1640904E7,2.32187835E8]\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"9\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cf7aba0>"
            ]
          },
          "execution_count": 41,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_oceania_82 = gapminder.query(\"continent != 'Oceania' & year == 1982\")\n",
        "\n",
        "(\n",
        "  ggplot(gapminder_oceania_82, aes(\"continent\", \"pop\")) +\n",
        "  geom_boxplot() \n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot9.png)\n",
        "\n",
        "👥 **WORK IN PAIRS/GROUPS:**\n",
        "\n",
        "-   Add `scale_y_log10` to the plot.\n",
        "-   Specify `panel_grid_major_x=element_blank()` in the `theme` function.\n",
        "-   Edit the x and y axes using `labs`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 42,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"r6fafA\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"r6fafA\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"continent\",\n",
              "\"y\":\"pop\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"log_gdp_per_cap\"\n",
              "}]\n",
              "},\n",
              "\"theme\":{\n",
              "\"panel_grid_major_x\":{\n",
              "\"blank\":true\n",
              "}\n",
              "},\n",
              "\"guides\":{\n",
              "\"x\":{\n",
              "\"title\":\"Continent\"\n",
              "},\n",
              "\"y\":{\n",
              "\"title\":\"Population\"\n",
              "}\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[{\n",
              "\"aesthetic\":\"y\",\n",
              "\"trans\":\"log10\"\n",
              "}],\n",
              "\"layers\":[{\n",
              "\"geom\":\"boxplot\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "\"..middle..\":[1.4441915999999998E7,8962184.790817013,5665951.0735603785,5968348.999999995],\n",
              "\"..lower..\":[3858420.9999999986,4615520.733389025,1513321.8295365965,3279000.99999999],\n",
              "\"..upper..\":[4.88271599999999E7,1.7886690977012083E7,1.0269498535998615E7,1.812512899999997E7],\n",
              "\"..ymin..\":[377966.99999999977,1861251.9999999956,98592.99999999988,1116478.9999999986],\n",
              "\"..ymax..\":[1.0002809999999969E9,7.833526599999988E7,7.303937599999978E7,2.321878349999993E8],\n",
              "\"continent\":[\"Asia\",\"Europe\",\"Africa\",\"Americas\"]\n",
              "}\n",
              "},{\n",
              "\"geom\":\"point\",\n",
              "\"stat\":\"boxplot_outlier\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"show_legend\":false,\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "\"..middle..\":[1.4441915999999998E7,8962184.790817013,8962184.790817013,5665951.0735603785,5968348.999999995],\n",
              "\"..lower..\":[3858420.9999999986,4615520.733389025,4615520.733389025,1513321.8295365965,3279000.99999999],\n",
              "\"..upper..\":[4.88271599999999E7,1.7886690977012083E7,1.7886690977012083E7,1.0269498535998615E7,1.812512899999997E7],\n",
              "\"..ymin..\":[377966.99999999977,1861251.9999999956,1861251.9999999956,98592.99999999988,1116478.9999999986],\n",
              "\"..ymax..\":[1.0002809999999969E9,7.833526599999988E7,7.833526599999988E7,7.303937599999978E7,2.321878349999993E8],\n",
              "\"continent\":[\"Asia\",\"Europe\",\"Europe\",\"Africa\",\"Americas\"],\n",
              "\"pop\":[NaN,233996.99999999968,562547.9999999994,NaN,NaN]\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"10\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cf7ac30>"
            ]
          },
          "execution_count": 42,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "(\n",
        "  ggplot(gapminder_oceania_82, aes(\"continent\", \"pop\")) +\n",
        "  geom_boxplot() +\n",
        "  scale_y_log10() +\n",
        "  theme(panel_grid_major_x=element_blank()) +\n",
        "  labs(x=\"Continent\", y=\"Population\")\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot10.png)\n",
        "\n",
        "### 2.6 Density plots\n",
        "\n",
        "Finally, we can understand the distribution of continuous features using density plots. Here's an example that looks at the distribution of GDP in 1952."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 43,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"VPVaZb\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"VPVaZb\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"gdp\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"log_gdp_per_cap\"\n",
              "}]\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[],\n",
              "\"layers\":[{\n",
              "\"geom\":\"density\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "\"..density..\":[2.962780689658235E-11,2.962780689658235E-11,3.3129295847823014E-11,2.7439260016162712E-11,2.7439260016162712E-11,1.8766284685578627E-11,1.2692171638292632E-11,9.873842290999854E-12,8.519594421807653E-12,7.12884331106434E-12,7.12884331106434E-12,5.471836042778219E-12,4.088747169903597E-12,3.2377050713238283E-12,2.730349832900464E-12,2.406807500848717E-12,2.2618784633789097E-12,2.25348121377089E-12,2.1964747103938274E-12,1.918961208828925E-12,1.4318610639579417E-12,9.517575152836015E-13,7.384532667559065E-13,8.00585214732735E-13,9.34701848652265E-13,1.0891805858886541E-12,1.3826509464082436E-12,1.693940227475907E-12,1.6770383498434599E-12,1.3072023406965705E-12,8.917406915922807E-13,5.793785658504116E-13,3.267048675991807E-13,1.3739932809927555E-13,3.9553625998208626E-14,7.553145578274222E-15,1.1532815777366569E-15,2.2830596141469725E-15,1.4931729964549825E-14,6.573167598812587E-14,1.8838319490180586E-13,3.513962491630622E-13,4.2661779726804685E-13,3.371141421207666E-13,1.7350871284767397E-13,5.955291451901203E-14,2.371002675400144E-14,5.4272580463602265E-14,1.6217071812841675E-13,3.2585802660024123E-13,4.2970433118728155E-13,3.873023206184061E-13,2.979109884700152E-13,3.1936864709966507E-13,4.4058116143737396E-13,5.307027142385065E-13,5.517160002895069E-13,5.337556747681013E-13,4.40157184438744E-13,2.6730743062090317E-13,1.1027460387278921E-13,3.005565284742673E-14,6.028620242259094E-15,6.485629092853702E-15,3.2300376919164224E-14,1.1539977045840086E-13,2.688062742027434E-13,4.0853811352478193E-13,4.115952243145694E-13,3.061763384936119E-13,2.60799054669169E-13,3.449418330717935E-13,4.2887587955975803E-13,3.707610260021794E-13,2.1053261688857017E-13,7.790390020673396E-14,1.876529001691663E-14,2.9420302982572735E-15,3.002111401510876E-16,1.9938533741057925E-17,8.618788893601428E-19,2.4248554959448603E-20,4.440297550126792E-22,5.292061415517394E-24,4.105104986479402E-26,2.0725749862363816E-28,6.810560060804879E-31,1.4566056596108908E-33,2.0276255255893216E-36,1.8370462809237057E-39,1.0832757879921562E-42,4.15761782923228E-46,1.0385721845289067E-49,1.6885557291802827E-53,1.7868196966942975E-57,2.522369613092246E-60,3.0557082160621493E-56,2.532939784115136E-52,1.3665465836828216E-48,4.7985564628019663E-45,1.0966887145840917E-41,1.6313334611626018E-38,1.5793876589185315E-35,9.952253165706354E-33,4.081698321207794E-30,1.089550941708551E-27,1.892963224877852E-25,2.1405577599692543E-23,1.5754595287650491E-21,7.547387517896586E-20,2.353581956454187E-18,4.778497090321965E-17,6.319737050324921E-16,5.4512345478731245E-15,3.07630570676926E-14,1.144474377966527E-13,2.85731478624444E-13,4.970498837468585E-13,6.414062054814502E-13,6.535031768853861E-13,5.290559570170977E-13,3.2156837452440077E-13,1.3741881994217917E-13,3.959439682781802E-14,7.53649970614464E-15,9.393363216159864E-16,7.638586977725784E-17,4.046797984350978E-18,1.3959298381501364E-19,3.134502133646418E-21,4.5812761768497053E-23,4.358142457551549E-25,2.6984048689336576E-27,1.0874293228645292E-29,2.852214691034707E-32,4.869117285844803E-35,5.410095771084824E-38,3.912427496097629E-41,1.841510751846901E-44,5.641424921643583E-48,1.1248379498361383E-51,1.4597473636975618E-55,1.2329686392596974E-59,6.778177731386653E-64,2.4252720698208444E-68,5.647998456072378E-73,8.560818002292304E-78,8.44544523953104E-83,5.42271059611632E-88,2.266193007649191E-93,6.164018792790123E-99,1.09123328395129E-104,1.2573548024646622E-110,9.429413410766812E-117,4.602545596256436E-123,1.4621708229556683E-129,3.023324185658473E-136,4.06872103399332E-143,3.5638359207693143E-150,2.0317210426964774E-157,7.5387101779165E-165,1.820608672292434E-172,2.8616911329367455E-180,2.927625353443648E-188,1.949372486822909E-196,8.448133261850037E-205,2.3829408949375653E-213,4.374741299709023E-222,5.227307791236047E-231,4.0652790349494224E-240,2.057734025473219E-249,6.779141647618266E-259,1.4536062076183658E-268,2.0286420364823755E-278,1.8426831432685312E-288,1.0893877727174308E-298,4.19180354788799E-309,1.0498E-319,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,4.35736147E-316,1.5039423861938628E-305,3.3785114871178175E-295,4.939763893143688E-285,4.7008194800649144E-275,2.911572087494631E-265,1.173729747535062E-255,3.079606779913379E-246,5.259070750860679E-237,5.84533425862254E-228,4.2285968231409557E-219,1.9909932997694795E-210,6.101405775641573E-202,1.2169608585487862E-193,1.579828347923344E-185,1.3348417622436147E-177,7.340679450339782E-170,2.627418933241873E-162,6.120811167452134E-155,9.280582345103682E-148,9.158578383907948E-141,5.882575310333444E-134,2.459198107694482E-127,6.691232600699291E-121,1.1849644714835747E-114,1.365812623218982E-108,1.0246215984795222E-102,5.002907594462341E-97,1.5898935497743046E-91,3.288517856615245E-86,4.427096130534062E-81,3.879040298208283E-76,2.2121584012682063E-71,8.210975235229685E-67,1.9836262836683092E-62,3.118972582590551E-58,3.1919042203949835E-54,2.126056177495887E-50,9.216928356057562E-47,2.6006641951398866E-43,4.776050787494429E-40,5.708738965139674E-37,4.441176415988801E-34,2.2487566242357234E-31,7.41094307566829E-29,1.58961184454061E-26,2.2191941241986516E-24,2.0164436429370983E-22,1.1925139667229549E-20,4.590156055264834E-19,1.1499475134268658E-17,1.8750600338320314E-16,1.989935165040189E-15,1.3745143152486056E-14,6.179398439309816E-14,1.8081302891206924E-13,3.443497998672145E-13,4.2683157540384974E-13],\n",
              "\"..quantile..\":[0.25,0.5,0.5,0.5,0.75,0.75,0.75,0.75,0.75,0.75,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0],\n",
              "\"gdp\":[5.27846905494405E7,5.27846905494405E7,4.366267348983276E9,8.679750007417112E9,8.679750007417112E9,1.2993232665850948E10,1.7306715324284786E10,2.1620197982718624E10,2.593368064115246E10,3.0247163299586292E10,3.0247163299586292E10,3.456064595802013E10,3.8874128616453964E10,4.31876112748878E10,4.750109393332163E10,5.181457659175547E10,5.612805925018931E10,6.044154190862314E10,6.475502456705698E10,6.906850722549081E10,7.338198988392465E10,7.769547254235849E10,8.200895520079233E10,8.632243785922617E10,9.063592051765999E10,9.494940317609383E10,9.926288583452766E10,1.035763684929615E11,1.0788985115139534E11,1.1220333380982918E11,1.1651681646826302E11,1.2083029912669684E11,1.2514378178513068E11,1.2945726444356451E11,1.3377074710199835E11,1.380842297604322E11,1.4239771241886603E11,1.4671119507729987E11,1.510246777357337E11,1.5533816039416754E11,1.5965164305260138E11,1.6396512571103522E11,1.6827860836946906E11,1.725920910279029E11,1.769055736863367E11,1.8121905634477054E11,1.8553253900320438E11,1.898460216616382E11,1.9415950432007205E11,1.984729869785059E11,2.0278646963693973E11,2.0709995229537357E11,2.114134349538074E11,2.1572691761224124E11,2.2004040027067508E11,2.2435388292910892E11,2.2866736558754276E11,2.329808482459766E11,2.3729433090441043E11,2.4160781356284424E11,2.4592129622127808E11,2.502347788797119E11,2.5454826153814575E11,2.588617441965796E11,2.6317522685501343E11,2.6748870951344727E11,2.718021921718811E11,2.7611567483031494E11,2.804291574887488E11,2.847426401471826E11,2.8905612280561646E11,2.933696054640503E11,2.9768308812248413E11,3.0199657078091797E11,3.063100534393518E11,3.1062353609778564E11,3.149370187562195E11,3.192505014146533E11,3.2356398407308716E11,3.27877466731521E11,3.3219094938995483E11,3.365044320483887E11,3.408179147068225E11,3.4513139736525635E11,3.494448800236901E11,3.5375836268212396E11,3.580718453405578E11,3.6238532799899164E11,3.666988106574255E11,3.710122933158593E11,3.7532577597429315E11,3.79639258632727E11,3.839527412911608E11,3.8826622394959467E11,3.925797066080285E11,3.9689318926646234E11,4.012066719248962E11,4.0552015458333E11,4.0983363724176385E11,4.141471199001977E11,4.1846060255863153E11,4.2277408521706537E11,4.270875678754992E11,4.3140105053393304E11,4.357145331923669E11,4.400280158508007E11,4.4434149850923456E11,4.486549811676684E11,4.5296846382610223E11,4.572819464845361E11,4.615954291429699E11,4.6590891180140375E11,4.702223944598376E11,4.745358771182714E11,4.7884935977670526E11,4.8316284243513904E11,4.874763250935729E11,4.917898077520067E11,4.9610329041044055E11,5.004167730688744E11,5.047302557273082E11,5.0904373838574207E11,5.133572210441759E11,5.1767070370260974E11,5.219841863610436E11,5.262976690194774E11,5.3061115167791125E11,5.349246343363451E11,5.3923811699477893E11,5.4355159965321277E11,5.478650823116466E11,5.521785649700804E11,5.564920476285143E11,5.608055302869481E11,5.65119012945382E11,5.694324956038158E11,5.737459782622496E11,5.780594609206835E11,5.823729435791173E11,5.866864262375511E11,5.90999908895985E11,5.953133915544188E11,5.996268742128527E11,6.039403568712865E11,6.082538395297203E11,6.125673221881542E11,6.16880804846588E11,6.211942875050219E11,6.255077701634557E11,6.298212528218895E11,6.341347354803234E11,6.384482181387572E11,6.42761700797191E11,6.470751834556249E11,6.513886661140587E11,6.557021487724926E11,6.600156314309264E11,6.643291140893602E11,6.686425967477941E11,6.729560794062279E11,6.772695620646617E11,6.815830447230956E11,6.858965273815294E11,6.902100100399633E11,6.94523492698397E11,6.988369753568308E11,7.031504580152646E11,7.074639406736985E11,7.117774233321323E11,7.160909059905662E11,7.20404388649E11,7.247178713074338E11,7.290313539658677E11,7.333448366243015E11,7.376583192827354E11,7.419718019411692E11,7.46285284599603E11,7.505987672580369E11,7.549122499164707E11,7.592257325749045E11,7.635392152333384E11,7.678526978917722E11,7.72166180550206E11,7.764796632086399E11,7.807931458670737E11,7.851066285255076E11,7.894201111839414E11,7.937335938423752E11,7.980470765008091E11,8.023605591592429E11,8.066740418176768E11,8.109875244761106E11,8.153010071345444E11,8.196144897929783E11,8.239279724514121E11,8.28241455109846E11,8.325549377682798E11,8.368684204267136E11,8.411819030851475E11,8.454953857435813E11,8.498088684020151E11,8.54122351060449E11,8.584358337188828E11,8.627493163773167E11,8.670627990357505E11,8.713762816941843E11,8.756897643526182E11,8.80003247011052E11,8.843167296694858E11,8.886302123279197E11,8.929436949863535E11,8.972571776447874E11,9.015706603032212E11,9.05884142961655E11,9.101976256200889E11,9.145111082785227E11,9.188245909369565E11,9.231380735953904E11,9.274515562538242E11,9.31765038912258E11,9.360785215706919E11,9.403920042291257E11,9.447054868875596E11,9.490189695459934E11,9.533324522044272E11,9.576459348628611E11,9.619594175212949E11,9.662729001797286E11,9.705863828381625E11,9.748998654965963E11,9.792133481550302E11,9.83526830813464E11,9.878403134718978E11,9.921537961303317E11,9.964672787887655E11,1.0007807614471993E12,1.0050942441056332E12,1.009407726764067E12,1.0137212094225009E12,1.0180346920809347E12,1.0223481747393685E12,1.0266616573978024E12,1.0309751400562362E12,1.03528862271467E12,1.0396021053731039E12,1.0439155880315377E12,1.0482290706899716E12,1.0525425533484054E12,1.0568560360068392E12,1.0611695186652731E12,1.0654830013237069E12,1.0697964839821407E12,1.0741099666405746E12,1.0784234492990084E12,1.0827369319574423E12,1.0870504146158761E12,1.0913638972743099E12,1.0956773799327438E12,1.0999908625911777E12,1.1043043452496116E12,1.1086178279080454E12,1.1129313105664792E12,1.117244793224913E12,1.121558275883347E12,1.1258717585417808E12,1.1301852412002146E12,1.1344987238586484E12,1.1388122065170823E12,1.143125689175516E12,1.14743917183395E12,1.1517526544923838E12,1.1560661371508176E12,1.1603796198092515E12,1.1646931024676853E12,1.1690065851261191E12,1.173320067784553E12,1.1776335504429868E12,1.1819470331014207E12,1.1862605157598545E12,1.1905739984182883E12,1.1948874810767222E12,1.199200963735156E12,1.2035144463935898E12,1.2078279290520237E12,1.2121414117104575E12,1.2164548943688914E12,1.2207683770273252E12,1.225081859685759E12,1.2293953423441929E12,1.2337088250026267E12,1.2380223076610605E12,1.2423357903194944E12,1.2466492729779282E12,1.250962755636362E12,1.255276238294796E12,1.2595897209532297E12,1.2639032036116636E12,1.2682166862700974E12,1.2725301689285313E12,1.276843651586965E12,1.281157134245399E12,1.2854706169038328E12,1.2897840995622666E12,1.2940975822207004E12,1.2984110648791343E12,1.302724547537568E12,1.307038030196002E12,1.3113515128544358E12,1.3156649955128696E12,1.3199784781713035E12,1.3242919608297373E12,1.3286054434881711E12,1.332918926146605E12,1.3372324088050388E12,1.3415458914634727E12,1.3458593741219065E12,1.3501728567803403E12,1.3544863394387742E12,1.358799822097208E12,1.3631133047556418E12,1.3674267874140757E12,1.3717402700725095E12,1.3760537527309434E12,1.3803672353893772E12,1.3846807180478108E12,1.3889942007062446E12,1.3933076833646785E12,1.3976211660231123E12,1.4019346486815461E12,1.40624813133998E12,1.4105616139984138E12,1.4148750966568477E12,1.4191885793152815E12,1.4235020619737153E12,1.4278155446321492E12,1.432129027290583E12,1.4364425099490168E12,1.4407559926074507E12,1.4450694752658845E12,1.4493829579243184E12,1.4536964405827522E12,1.458009923241186E12,1.4623234058996199E12,1.4666368885580537E12,1.4709503712164875E12,1.4752638538749214E12,1.4795773365333552E12,1.483890819191789E12,1.488204301850223E12,1.4925177845086567E12,1.4968312671670906E12,1.5011447498255244E12,1.5054582324839583E12,1.509771715142392E12,1.514085197800826E12,1.5183986804592598E12,1.5227121631176936E12,1.5270256457761274E12,1.5313391284345613E12,1.535652611092995E12,1.539966093751429E12,1.5442795764098628E12,1.5485930590682966E12,1.5529065417267305E12,1.5572200243851643E12,1.5615335070435981E12,1.565846989702032E12,1.5701604723604658E12,1.5744739550188997E12,1.5787874376773335E12,1.5831009203357673E12,1.5874144029942012E12,1.591727885652635E12,1.5960413683110688E12,1.6003548509695027E12,1.6046683336279365E12,1.6089818162863704E12,1.6132952989448042E12,1.617608781603238E12,1.6219222642616719E12,1.6262357469201057E12,1.6305492295785396E12,1.6348627122369734E12,1.6391761948954072E12,1.643489677553841E12,1.647803160212275E12,1.6521166428707087E12,1.6564301255291426E12,1.6607436081875764E12,1.6650570908460103E12,1.669370573504444E12,1.673684056162878E12,1.6779975388213118E12,1.6823110214797456E12,1.6866245041381794E12,1.6909379867966133E12,1.695251469455047E12,1.699564952113481E12,1.7038784347719148E12,1.7081919174303486E12,1.7125054000887825E12,1.7168188827472163E12,1.7211323654056501E12,1.725445848064084E12,1.7297593307225178E12,1.7340728133809517E12,1.7383862960393855E12,1.7426997786978193E12,1.7470132613562532E12,1.751326744014687E12,1.7556402266731208E12,1.7599537093315547E12,1.7642671919899885E12,1.7685806746484224E12,1.7728941573068562E12,1.77720763996529E12,1.7815211226237239E12,1.7858346052821577E12,1.7901480879405916E12,1.7944615705990254E12,1.7987750532574592E12,1.803088535915893E12,1.807402018574327E12,1.8117155012327607E12,1.8160289838911946E12,1.8203424665496284E12,1.8246559492080623E12,1.828969431866496E12,1.83328291452493E12,1.8375963971833638E12,1.8419098798417976E12,1.8462233625002314E12,1.8505368451586653E12,1.854850327817099E12,1.859163810475533E12,1.8634772931339668E12,1.8677907757924006E12,1.8721042584508345E12,1.8764177411092683E12,1.8807312237677021E12,1.885044706426136E12,1.8893581890845698E12,1.8936716717430037E12,1.8979851544014375E12,1.9022986370598713E12,1.9066121197183052E12,1.910925602376739E12,1.9152390850351729E12,1.9195525676936067E12,1.9238660503520405E12,1.9281795330104744E12,1.932493015668908E12,1.9368064983273418E12,1.9411199809857756E12,1.9454334636442095E12,1.9497469463026433E12,1.9540604289610771E12,1.958373911619511E12,1.9626873942779448E12,1.9670008769363787E12,1.9713143595948125E12,1.9756278422532463E12,1.9799413249116802E12,1.984254807570114E12,1.9885682902285479E12,1.9928817728869817E12,1.9971952555454155E12,2.0015087382038494E12,2.0058222208622832E12,2.010135703520717E12,2.014449186179151E12,2.0187626688375847E12,2.0230761514960186E12,2.0273896341544524E12,2.0317031168128862E12,2.03601659947132E12,2.040330082129754E12,2.0446435647881877E12,2.0489570474466216E12,2.0532705301050554E12,2.0575840127634893E12,2.061897495421923E12,2.066210978080357E12,2.0705244607387908E12,2.0748379433972246E12,2.0791514260556584E12,2.0834649087140923E12,2.0877783913725261E12,2.09209187403096E12,2.0964053566893938E12,2.1007188393478276E12,2.1050323220062615E12,2.1093458046646953E12,2.1136592873231292E12,2.117972769981563E12,2.1222862526399968E12,2.1265997352984307E12,2.1309132179568645E12,2.1352267006152983E12,2.1395401832737322E12,2.143853665932166E12,2.1481671485905999E12,2.1524806312490337E12,2.1567941139074675E12,2.1611075965659014E12,2.1654210792243352E12,2.169734561882769E12,2.174048044541203E12,2.1783615271996367E12,2.1826750098580706E12,2.1869884925165044E12,2.1913019751749382E12,2.195615457833372E12,2.1999289404918057E12,2.2042424231502397E12]\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"11\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x10cf7b380>"
            ]
          },
          "execution_count": 43,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "gapminder_52 = gapminder.query(\"year == 1952\")\n",
        "\n",
        "(\n",
        "  ggplot(gapminder_52, aes(\"gdp\")) +\n",
        "  geom_density() \n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot11.png)\n",
        "\n",
        "👥 **WORK IN PAIRS/GROUPS:**\n",
        "\n",
        "-   Add `scale_x_log10` to the plot.\n",
        "-   Specify `panel_grid_major_x=element_blank()` in the `theme` function.\n",
        "-   Edit the x and y axes using `labs`."
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 76,
      "metadata": {},
      "outputs": [
        {
          "data": {
            "text/html": [
              "   <div id=\"gbdQA8\"></div>\n",
              "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
              "   \n",
              "   (function() {\n",
              "   // ----------\n",
              "   \n",
              "   var containerDiv = document.getElementById(\"gbdQA8\");\n",
              "   var observer = new ResizeObserver(function(entries) {\n",
              "       for (let entry of entries) {\n",
              "           var width = containerDiv.clientWidth\n",
              "           if (entry.contentBoxSize && width > 0) {\n",
              "           \n",
              "               // Render plot\n",
              "               if (observer) {\n",
              "                   observer.disconnect();\n",
              "                   observer = null;\n",
              "               }\n",
              "\n",
              "               var plotSpec={\n",
              "\"data\":{\n",
              "},\n",
              "\"mapping\":{\n",
              "\"x\":\"gdp\"\n",
              "},\n",
              "\"data_meta\":{\n",
              "\"series_annotations\":[{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"country\"\n",
              "},{\n",
              "\"type\":\"str\",\n",
              "\"column\":\"continent\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"year\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"life_exp\"\n",
              "},{\n",
              "\"type\":\"int\",\n",
              "\"column\":\"pop\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp_per_cap\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"gdp\"\n",
              "},{\n",
              "\"type\":\"float\",\n",
              "\"column\":\"log_gdp_per_cap\"\n",
              "}]\n",
              "},\n",
              "\"guides\":{\n",
              "\"x\":{\n",
              "\"title\":\"Gross domestic product\"\n",
              "},\n",
              "\"y\":{\n",
              "\"title\":\"Density\"\n",
              "}\n",
              "},\n",
              "\"kind\":\"plot\",\n",
              "\"scales\":[{\n",
              "\"aesthetic\":\"x\",\n",
              "\"trans\":\"log10\"\n",
              "}],\n",
              "\"layers\":[{\n",
              "\"geom\":\"density\",\n",
              "\"mapping\":{\n",
              "},\n",
              "\"data_meta\":{\n",
              "},\n",
              "\"data\":{\n",
              "\"..density..\":[0.028226932199622936,0.028993789020848513,0.02976991491182238,0.03055517209091423,0.03134941421197263,0.032152485222114724,0.032964218193388135,0.0337844341443571,0.03461294086876273,0.03544953178936219,0.0362939848558446,0.03714606150632665,0.038005505712351464,0.03887204312752154,0.0397453803598884,0.040625204387994454,0.041511182139994794,0.042402960254597846,0.04330016504163126,0.044202402658881076,0.04510925952046973,0.04602030295042929,0.046935082093321985,0.04785312909175233,0.04877396053843175,0.049697079208119756,0.050621976072280125,0.051548132596698215,0.05247502331961814,0.0534021187052032,0.05432888826433911,0.05525480393199469,0.05617934368758141,0.05710199540201813,0.05802226089256225,0.05893966016391643,0.05985373581171851,0.06076405756226497,0.061670226920267704,0.06257188189458064,0.06346870177022035,0.06436041189363292,0.06524678843705142,0.0661276631069684,0.06700292776120968,0.06787253889885961,0.06873652198734631,0.06959497559136832,0.0704480752689967,0.07129607720125114,0.072139321522683,0.07297823532201791,0.07381333528367186,0.0746452299429728,0.07547462153014865,0.0763023073805722,0.07712918089135988,0.07795623200718102,0.07878454722101455,0.07961530907857055,0.08044979517814754,0.0812893766607968,0.0821355161887723,0.08298976541335971,0.08385376193625035,0.0847292257716389,0.08561795531916756,0.08652182286067749,0.08744276959644572,0.08838280023917602,0.08934397718644714,0.09032841429460335,0.09133827027917138,0.09237574176882225,0.09344305604164424,0.09454246347404321,0.09567622973397234,0.09684662775137012,0.09805592949969773,0.09930639762328416,0.1006002769458434,0.10193978589601334,0.10332710788608228,0.10476438268024701,0.10625369778877304,0.10779707992430934,0.10939648655638058,0.11105379759971654,0.1127708072715995,0.11454921615283373,0.11639062348624703,0.11829651974585817,0.12026827950895165,0.12230715466233087,0.12441426797295856,0.12659060705201727,0.12883701874018555,0.1311542039405646,0.13354271292425582,0.13600294113203246,0.13853512549391064,0.14113934128667588,0.14381549954755093,0.14656334506022653,0.14938245492740965,0.15227223774182574,0.15523193336532964,0.15826061332336366,0.16135718181947095,0.1645203773719717,0.16774877507217748,0.17104078946074627,0.17439467801588204,0.17780854524416972,0.18128034736186285,0.1848078975514096,0.18838887177501684,0.19202081512401847,0.19570114867986402,0.19942717685959332,0.20319609521584406,0.2070049986586981,0.21085089006403882,0.2147306892306589,0.21864124214607306,0.2225793305188831,0.22654168153372622,0.23052497778320447,0.2345258673298393,0.2385409738500564,0.24256690681141124,0.24660027163384046,0.250637679785546,0.2546757587643397,0.25871116191577775,0.2627405780402499,0.2667607407423668,0.2707684374774631,0.27476051825184317,0.27873390393544795,0.28268559414802263,0.28661267468245877,0.29051232443184943,0.2943818217898603,0.2982185504972962,0.302020004911119,0.30578379467575206,0.3095076487801499,0.31318941898779806,0.3168270826306189,0.3204187447614803,0.3239626396638146,0.32745713172051544,0.3309007156479692,0.33429201610461456,0.33762978668684895,0.3409129083284366,0.3441403871226814,0.3473113515896591,0.35042504941356145,0.3534808436778494,0.3564782086283236,0.3594167249964178,0.3622960749170418,0.365116036477106,0.36787647793243017,0.37057735163214656,0.37321868769089167,0.37580058745006145,0.378323216770212,0.38078679919729563,0.38319160904585686,0.3855379644425506,0.38782622037344,0.38782622037344,0.39005676177844356,0.3922299967360276,0.3943463497808392,0.39640625539637875,0.3984101517240703,0.4003584745291525,0.4022516514627335,0.4040900966580937,0.40587420569784194,0.4076043509869129,0.4092808775645658,0.4109040993864659,0.4124742961057256,0.41399171037929167,0.41545654572337165,0.41686896493869396,0.418229089123244,0.4195369972867352,0.4207927265775013,0.42199627312864174,0.4231475935262505,0.4242466068983123,0.4252931976184339,0.42628721861401053,0.4272284952637054,0.42811682986429694,0.4289520066420528,0.4297337972788363,0.43046196691822464,0.4311362806120167,0.4317565101626979,0.43232244131278774,0.4328338812274994,0.4332906662129407,0.4336926696081554,0.4340398097857385,0.43433205819258536,0.4345694473596592,0.43475207880743905,0.43488013077209636,0.4349538656763792,0.4349736372688043,0.4349398973549874,0.43485320204591127,0.43471421744956557,0.4345237247348119,0.4342826244993971,0.4339919403778862,0.4336528218298377,0.43326654605375164,0.43283451897821595,0.43235827528820786,0.4318394774515617,0.4312799137182455,0.4306814950731538,0.4300462511315808,0.42937632497532746,0.42867396693640475,0.4279415273444923,0.4271814482635323,0.4263962542520789,0.42558854219112296,0.42476097023202064,0.42391624592575766,0.42305711360300974,0.42305711360300974,0.4221863410821888,0.4213067057898803,0.42042098038460873,0.4195319179807405,0.4186422370743857,0.41775460627743055,0.4168716289691781,0.4159958279775326,0.4151296304031299,0.41427535270035776,0.41343518612870245,0.4126111826864094,0.4118052416359781,0.4110190967276003,0.41025430422228837,0.40951223181117374,0.4087940485213481,0.4081007156916755,0.4074329790943769,0.4067913622698154,0.40617616113300675,0.4055874399009243,0.40502502837979143,0.4044885206413422,0.40397727510656023,0.40349041604479347,0.4030268364854554,0.40258520252888724,0.40216395903241975,0.4017613366374029,0.4013753600929469,0.4010038578225486,0.4006444726706346,0.4002946737574791,0.39995176936300453,0.3996129207526862,0.3992751568522466,0.39893538967208264,0.3985904303774366,0.39823700589627004,0.3978717759536372,0.3974913504190773,0.39709230685222024,0.39667120813135653,0.39622462005022235,0.3957491287696213,0.3952413580127692,0.3946979858963555,0.39411576129321974,0.39349151962723766,0.3928221980064022,0.3921048496061586,0.39133665722172456,0.3905149459153407,0.3896371946920915,0.3887010471460313,0.3877043210267751,0.3866450166854069,0.3855213243674204,0.3843316303293716,0.3830745217649381,0.38174879053501254,0.38035343570529395,0.3788876649034812,0.37735089451653103,0.37574274875648267,0.3740630576310106,0.3723118538620423,0.3723118538620423,0.37048936880247324,0.3685960274071503,0.36663244231982056,0.3645994071426549,0.3624978889591831,0.36032902018503854,0.35809408982373386,0.3557945342068294,0.35343192729923695,0.3510079706510926,0.3485244830775961,0.34598339014746565,0.3433867135592774,0.34073656048288314,0.33803511294042804,0.3352846172982613,0.33248737393723454,0.32964572716462176,0.3267620554262036,0.32383876187197375,0.3208782653235209,0.31788299168549844,0.31485536583771645,0.31179780403840784,0.30871270686313745,0.30560245269775965,0.3024693917977752,0.2993158409205344,0.2961440785309517,0.2929563405758877,0.28975481681706355,0.2865416477074318,0.28331892179135354,0.2800886736047328,0.2768528820475031,0.2736134691975901,0.27037229953264336,0.26713117952352433,0.263891857561757,0.26065602418183975,0.2574253125385537,0.2542012990991481,0.2509855045105016,0.24777939460207157,0.24458438148659217,0.24140182472209218,0.23823303250075487,0.23507926283250374,0.2319417246938359,0.22882157911537965,0.22571994018480654,0.22263787594507548,0.21957640917150312,0.2165365180147145,0.21351913650015966,0.2105251548785189,0.2075554198248739,0.20461073448800182,0.20169185839450107,0.19879950721560682,0.19593435240750195,0.19309702073864704,0.1902880937200598,0.18750810695661538,0.18475754943922806,0.18203686279926257,0.17934644054761986,0.17668662732172383,0.17405771816402543,0.17145995785571297,0.16889354032900739,0.16635860818079398,0.16385525230940728,0.16138351169512272,0.15894337334338762,0.15653477240805802,0.1541575925098998,0.15181166626342416,0.1494967760227909,0.14721265485503057,0.14495898774627286,0.14273541304406354,0.14054152413619866,0.13837687136388827,0.1362409641644821,0.1341332734364785,0.13205323411715725,0.13000024796090404,0.12797368650418894,0.12597289420125027,0.1239971917127994,0.12204587932854968,0.12011824050310055,0.11821354548364492,0.11633105500715987,0.11447002404418227,0.11262970556593409,0.11080935431147099,0.1090082305316774,0.10722560368727875,0.10546075607860556,0.10371298638561223,0.10198161309757287,0.100265977812974,0.09856544839135278,0.09687942194017074,0.09520732762126863,0.09354862926296313,0.09190282776542454,0.09026946328860075,0.08864811721357792,0.08703841386990055,0.08544002202299868,0.08385265611744185,0.08227607727327355,0.08071009403416307,0.07915456286750618,0.0776093884179312,0.07607452351691513,0.07454996895235447,0.07303577300299216,0.07153203074357145,0.070038883127446,0.06855651585415647,0.06708515803017323,0.06562508063160129,0.06417659477818126,0.06274004982836455,0.06131583130563157,0.05990435866656082,0.05850608292143134,0.0571214841183803,0.05575106870234995,0.05439536676022617,0.05305492916372613,0.05173032462173741,0.05042213665393035,0.049130960497585335,0.047857399959698434,0.04660206422653421,0.04536556464290441,0.04414851147357307,0.0429515106592754,0.04177516057995381,0.04062004883790762,0.03948674907361902,0.038375817827101956,0.03728779145764911,0.03622318313486997,0.03518247991389845,0.03416613990758382,0.03317458956837308,0.03220822109244021,0.03126738995839473,0.03035241261262171,0.02946356431295891,0.0286010771419881,0.02776513820071504,0.026955887992835856,0.026173419009118105,0.025417774520676895,0.024688947589104733,0.02398688030049006,0.023311463229378868,0.02266253513766815,0.022039882912276047,0.021443241744245153,0.020872295550670158,0.020326677639536127,0.019805971616209034,0.0193097125289387,0.018837388249335785,0.01838844108237874,0.01796226959909744,0.017558230683687732,0.01717564178544677,0.016813783364587342,0.016471901519711056,0.016149210783504682,0.015844897072078545,0.0155581207723058,0.015288019950560368,0.015033713665386016,0.01479430536588627,0.014568886356998729,0.014356539312317234,0.014156341814765626,0.013967369905199097,0.013788701618926338,0.013619420490206787,0.013458619004981074,0.013305401982441067,0.013158889866537017,0.01301822190914664,0.012882559227393238,0.012751087718490636,0.01262302081650247,0.012497602076526485,0.012374107573041622,0.012251848100473993,0.012130171165439052,0.012008462761589584,0.011886148919526072,0.011762697025800878,0.011637616906652413],\n",
              "\"..quantile..\":[0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0],\n",
              "\"gdp\":[5.278469054944038E7,5.389525686810546E7,5.502918900620243E7,5.6186978570138484E7,5.736912750949997E7,5.857614833466853E7,5.9808564339015126E7,6.106690982576953E7,6.235173033966318E7,6.366358290344492E7,6.5003036259373665E7,6.637067111579116E7,6.776708039888333E7,6.919286950973864E7,7.064865658681396E7,7.213507277392341E7,7.365276249386482E7,7.520238372780389E7,7.678460830053677E7,7.840012217175242E7,8.004962573342669E7,8.173383411346866E7,8.345347748575947E7,8.52093013867133E7,8.700206703849673E7,8.883255167905146E7,9.070154889905839E7,9.260986898599131E7,9.455833927541159E7,9.65478045096504E7,9.857912720404105E7,1.0065318802085409E8,1.0277088615110148E8,1.0493313970437108E8,1.0714088610686797E8,1.093950825078255E8,1.1169670619446974E8,1.1404675501571281E8,1.1644624781476316E8,1.1889622487083572E8,1.213977483501535E8,1.239519027664452E8,1.2655979545112324E8,1.2922255703335692E8,1.3194134193025026E8,1.3471732884732735E8,1.3755172128954917E8,1.404457480830847E8,1.434006639080564E8,1.4641774984249705E8,1.4949831391774693E8,1.5264369168554345E8,1.558552467970374E8,1.5913437159398744E8,1.6248248771240366E8,1.6590104669888237E8,1.693915306399124E8,1.729554528044193E8,1.765943582998323E8,1.803098247419515E8,1.841034629389062E8,1.8797691758951223E8,1.919318679963136E8,1.959700287936286E8,2.0009315069091728E8,2.0430302123178554E8,2.086014655689555E8,2.129903472555501E8,2.1747156905301827E8,2.2204707375605986E8,2.2671884503490478E8,2.314889082953221E8,2.3635933155671337E8,2.4133222634868312E8,2.4640974862647766E8,2.5159409970567828E8,2.5688752721656454E8,2.6229232607854944E8,2.6781083949512902E8,2.7344545996975636E8,2.79198630343083E8,2.8507284485204166E8,2.910706502111956E8,2.971946467168431E8,3.034474893743579E8,3.0983188904924154E8,3.16350613642391E8,3.230064892901067E8,3.298024015893332E8,3.3674129684868693E8,3.4382618336579984E8,3.51060132731545E8,3.584462811616967E8,3.6598783085660267E8,3.7368805138948405E8,3.815502811239178E8,3.895779286611631E8,3.9777447431792134E8,4.0614347163520926E8,4.146885489189588E8,4.2341341081303537E8,4.323218399053603E8,4.414176983678167E8,4.5070492963065296E8,4.6018756009213567E8,4.698697008641517E8,4.797555495545432E8,4.898493920869499E8,5.001556045589303E8,5.1067865513918245E8,5.21423106004675E8,5.323936153185464E8,5.43594939249609E8,5.550319340343353E8,5.667095580822521E8,5.786328741256067E8,5.908070514142729E8,6.032373679568295E8,6.159292128088034E8,6.288880884090449E8,6.421196129652497E8,6.556295228897085E8,6.694236752862613E8,6.835080504895883E8,6.97888754657951E8,7.12572022420453E8,7.275642195800045E8,7.428718458731716E8,7.585015377880738E8,7.744600714415809E8,7.907543655170245E8,8.073914842637533E8,8.243786405597743E8,8.417231990388083E8,8.594326792831956E8,8.77514759083927E8,8.959772777692894E8,9.148282396035297E8,9.340758172570468E8,9.537283553495783E8,9.737943740679202E8,9.942825728598087E8,1.0152018342054731E9,1.0365612274685531E9,1.0583700128280568E9,1.0806376452930267E9,1.1033737788016548E9,1.1265882704067056E9,1.150291184548919E9,1.1744927974203823E9,1.1992036014196582E9,1.2244343097007074E9,1.250195860817463E9,1.2764994234661396E9,1.3033564013273416E9,1.3307784380100183E9,1.3587774220994592E9,1.387365492311458E9,1.4165550427549868E9,1.4463587283055453E9,1.4767894700915315E9,1.5078604610961401E9,1.5395851718770378E9,1.5719773564063976E9,1.6050510580338368E9,1.6388206155747828E9,1.6733006695269113E9,1.7085061684174566E9,1.7444523752839823E9,1.7811548742915518E9,1.8186295774890885E9,1.8568927317079334E9,1.8959609256055093E9,1.9358510968571503E9,1.9358510968571503E9,1.9765805394993587E9,2.0181669114274187E9,2.0606282420508614E9,2.1039829401099129E9,2.1482498016565037E9,2.1934480182030997E9,2.239597185043002E9,2.28671730974575E9,2.3348288208312173E9,2.383952576626179E9,2.434109874307321E9,2.485322459134434E9,2.537612533877811E9,2.591002768444159E9,2.645516309704888E9,2.7011767915312524E9,2.7580083450405846E9,2.8160356090581703E9,2.87528374079917E9,2.935778426775332E9,2.9975458939310985E9,3.0606129210141544E9,3.1250068501850724E9,3.1907555988712497E9,3.257887671870313E9,3.326432173708113E9,3.396418821256668E9,3.4678779566177893E9,3.5408405602775373E9,3.615338264537521E9,3.6914033672289023E9,3.7690688457147803E9,3.8483683711872206E9,3.929336323265161E9,4.0120078048993626E9,4.0964186575910134E9,4.1826054769304233E9,4.2706056284628296E9,4.360457263887878E9,4.452199337599888E9,4.545871623576237E9,4.641514732621093E9,4.739170129971745E9,4.838880153275576E9,4.940688030945146E9,5.04463790089958E9,5.150774829700042E9,5.259144832088017E9,5.369794890934707E9,5.482772977609759E9,5.598128072779178E9,5.71591018764031E9,5.836170385603595E9,5.958960804431023E9,6.084334678839671E9,6.212346363581411E9,6.343051357007732E9,6.476506325130656E9,6.612769126189887E9,6.751898835736564E9,6.89395577224507E9,7.039001523263524E9,7.187098972114658E9,7.338312325158131E9,7.492707139626735E9,7.492707139626735E9,7.650350352048315E9,7.811310307265112E9,7.975656788064551E9,8.143461045432753E9,8.314795829444525E9,8.489735420804048E9,8.668355663048185E9,8.850733995427948E9,9.036949486481737E9,9.227082868314518E9,9.421216571598923E9,9.619434761311802E9,9.821823373223814E9,1.002847015115582E10,1.0239464685019386E10,1.0454898449657866E10,1.0674864844504414E10,1.089945923407471E10,1.1128778989311106E10,1.1362923529797089E10,1.1601994366860023E10,1.1846095147579865E10,1.209533169972526E10,1.234981207763361E10,1.260964660905733E10,1.2874947942995659E10,1.3145831098532295E10,1.3422413514701092E10,1.3704815101400932E10,1.399315829138116E10,1.4287568093321836E10,1.4588172146029343E10,1.489510077377404E10,1.520848704279039E10,1.55284668189673E10,1.5855178826751083E10,1.618876470928846E10,1.6529369089835018E10,1.6877139634454508E10,1.7232227116039204E10,1.7594785479675716E10,1.796497190938602E10,1.8342946896274338E10,1.872887430810569E10,1.9122921460349716E10,1.9525259188719086E10,1.9936061923233315E10,2.0355507763841713E10,2.0783778557636913E10,2.122105997769347E10,2.1667541603564205E10,2.212341700347109E10,2.258888381822535E10,2.3064143846912205E10,2.3549403134379955E10,2.404487206056817E10,2.455076543171638E10,2.506730257349199E10,2.559470742607562E10,2.6133208641250175E10,2.6683039681529476E10,2.7244438921374363E10,2.7817649750538296E10,2.8402920679586067E10,2.9000505447633904E10,2.9610663132355602E10,3.023365826230395E10,3.086976093159394E10,3.086976093159394E10,3.1519246917000114E10,3.218239779751768E10,3.2859501076437695E10,3.3550850305992077E10,3.4256745214621105E10,3.497749183691695E10,3.571340264630378E10,3.6464796690505875E10,3.7231999729870384E10,3.801534437859599E10,3.881517024893559E10,3.963182409843279E10,4.04656599802545E10,4.131703939668815E10,4.218633145586849E10,4.3073913031799995E10,4.3980168927748085E10,4.490549204306687E10,4.5850283543538055E10,4.681495303529161E10,4.779991874238755E10,4.880560768813408E10,4.983245588021841E10,5.088090849973501E10,5.1951420094189575E10,5.304445477456481E10,5.416048641653157E10,5.529999886589159E10,5.64634861483481E10,5.7651452683683464E10,5.8864413504446976E10,6.010289447924367E10,6.136743254071834E10,6.265857591833914E10,6.397688437607713E10,6.53229294550883E10,6.6697294721498405E10,6.810057601940386E10,6.953338172919618E10,7.099633303131625E10,7.249006417556612E10,7.40152227560801E10,7.55724699920832E10,7.716248101455807E10,7.878594515894133E10,8.044356626398051E10,8.213606297687497E10,8.386416906484023E10,8.562863373322754E10,8.743022195033304E10,8.926971477904456E10,9.114790971546391E10,9.306562103465613E10,9.502368014366862E10,9.702293594198184E10,9.906425518954361E10,1.0114852288254265E11,1.0327664263709221E11,1.0544953708098714E11,1.0766814825369548E11,1.0993343801477869E11,1.122463884608912E11,1.1460800235156476E11,1.1701930354394212E11,1.1948133743666321E11,1.2199517142309023E11,1.245618953540633E11,1.2718262201039757E11,1.298584875853215E11,1.3259065217705983E11,1.3538030029178654E11,1.3822864135715594E11,1.4113691024664212E11,1.4410636781490363E11,1.4713830144441797E11,1.5023402560361798E11,1.5339488241676028E11,1.566222422458009E11,1.5991750428449963E11,1.632820971650235E11,1.6671747957732785E11,1.702251409015493E11,1.7380660185372083E11,1.7746341514505917E11,1.8119716615513162E11,1.850094736191863E11,1.8890199032993558E11,1.9287640385411313E11,1.9693443726410236E11,2.0107784988496298E11,2.053084380571655E11,2.0962803591538174E11,2.1403851618366107E11,2.185417909873275E11,2.2313981268196802E11,2.2783457469986368E11,2.3262811241421283E11,2.3752250402156497E11,2.4251987144279233E11,2.4762238124303482E11,2.5283224557099902E11,2.581517231180076E11,2.6358312009724503E11,2.6912879124358762E11,2.7479114083448187E11,2.805726237323022E11,2.864757464486272E11,2.925030682309211E11,2.9865720217207043E11,3.0494081634327386E11,3.1135663495075195E11,3.1790743951680786E11,3.245960700857381E11,3.314254264550918E11,3.38398469432875E11,3.4551822212116565E11,3.5278777122675916E11,3.60210268399389E11,3.6778893159808636E11,3.7552704648630334E11,3.8342796785638965E11,3.9149512108402026E11,3.997320036132526E11,4.081421864727995E11,4.1672931582422504E11,4.254971145427026E11,4.344493838310429E11,4.4359000486766034E11,4.529229404892297E11,4.6245223690874884E11,4.7218202546971454E11,4.821165244372595E11,4.9226004082693164E11,5.02616972271948E11,5.1319180892978204E11,5.239891354288054E11,5.350136328559235E11,5.4627008078602765E11,5.577633593541228E11,5.69498451371075E11,5.814804444838743E11,5.937145333813234E11,6.062060220461572E11,6.189603260545482E11,6.319829749239641E11,6.452796145104509E11,6.588560094563542E11,6.72718045689511E11,6.868717329750513E11,7.013232075208934E11,7.160787346380063E11,7.311447114567283E11,7.465276697001493E11,7.622342785158964E11,7.782713473674956E11,7.946458289865338E11,8.113648223869696E11,8.28435575942841E11,8.458654905307513E11,8.636621227384268E11,8.818331881408411E11,9.00386564645208E11,9.193302959063907E11,9.386725948141726E11,9.584218470538568E11,9.785866147418209E11,9.991756401375667E11,1.020197849433798E12,1.0416623566263619E12,1.0635784674654885E12,1.0859556834902368E12,1.1088037061478208E12,1.1321324409995562E12,1.1559520020153442E12,1.1802727159585146E12,1.205105126862869E12,1.2304600006039768E12,1.2563483295666167E12,1.282781337410453E12,1.3097704839359119E12,1.3373274700525298E12,1.365464242851741E12,1.3941930007864224E12,1.42352619895952E12,1.4534765545237085E12,1.4840570521949246E12,1.515280949881731E12,1.5471617844331218E12,1.5797133775074023E12,1.612949841564365E12,1.6468855859836553E12,1.681535323311856E12,1.7169140756409353E12,1.7530371811209773E12,1.789920300609886E12,1.827579424463048E12,1.866030879465762E12,1.9052913359115964E12,1.945377814829692E12,1.9863076953639636E12,2.0280987223078523E12,2.0707690137972698E12,2.114337069165586E12,2.1588217769639875E12,2.2042424231502285E12]\n",
              "}\n",
              "}],\n",
              "\"metainfo_list\":[],\n",
              "\"spec_id\":\"13\"\n",
              "};\n",
              "               window.letsPlotCall(function() {\n",
              "       \n",
              "               var toolbar = null;\n",
              "               var plotContainer = containerDiv;               \n",
              "               \n",
              "                   var options = {\n",
              "                       sizing: {\n",
              "                           width_mode: \"min\",\n",
              "                           height_mode: \"scaled\",\n",
              "                           width: width\n",
              "                       }\n",
              "                   };\n",
              "                   var fig = LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer, options);\n",
              "                   if (toolbar) {\n",
              "                     toolbar.bind(fig);\n",
              "                   }\n",
              "               });\n",
              "               \n",
              "               break;\n",
              "           }\n",
              "       }\n",
              "   });\n",
              "   \n",
              "   observer.observe(containerDiv);\n",
              "   \n",
              "   // ----------\n",
              "   })();\n",
              "   \n",
              "   </script>"
            ],
            "text/plain": [
              "<lets_plot.plot.core.PlotSpec at 0x26c3fc9f8f0>"
            ]
          },
          "execution_count": 76,
          "metadata": {},
          "output_type": "execute_result"
        }
      ],
      "source": [
        "\n",
        "(\n",
        "  ggplot(gapminder_52, aes(\"gdp\")) +\n",
        "  geom_density() +\n",
        "  scale_x_log10() +\n",
        "  labs(x=\"Gross domestic product\", y=\"Density\")\n",
        ")"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "![](./figures/lab_solutions_plot12.png)"
      ]
    }
  ],
  "metadata": {
    "kernelspec": {
      "display_name": "DS202W",
      "language": "python",
      "name": "python3"
    },
    "language_info": {
      "codemirror_mode": {
        "name": "ipython",
        "version": 3
      },
      "file_extension": ".py",
      "mimetype": "text/x-python",
      "name": "python",
      "nbconvert_exporter": "python",
      "pygments_lexer": "ipython3",
      "version": "3.13.1"
    }
  },
  "nbformat": 4,
  "nbformat_minor": 4
}
